/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.actions;

import org.eclipse.gef.ui.actions.ActionBarContributor;
import org.eclipse.gef.ui.actions.DeleteRetargetAction;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.gef.ui.actions.RedoRetargetAction;
import org.eclipse.gef.ui.actions.UndoRetargetAction;
import org.eclipse.gef.ui.actions.ZoomComboContributionItem;
import org.eclipse.gef.ui.actions.ZoomInRetargetAction;
import org.eclipse.gef.ui.actions.ZoomOutRetargetAction;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.ide.IDEActionFactory;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.texteditor.ITextEditorActionConstants;

/**
 * Constributor class to handle menue events.
 **/
public class VOCLEditorActionBarContributor extends ActionBarContributor {
	private EditAction propertiesDialogAction;
	private AddAttributeAction addAttributeAction;
	private AddMethodAction addMethodAction;
	private AddConditionAction addConditionAction;
	private ConvertAction convertAction;
	private IEditorPart activeEditorPart;
	private Action sampleAction;
	private ResetLabelPositionAction labelAction;
	
	/**
	 * Creates a multi-page contributor.
	 */
	public VOCLEditorActionBarContributor() {
		super();
		
	}

	/**
	 * Returns the action registed with the given text editor.
	 * @return IAction or null if editor is null.
	 */
	protected IAction getAction(ITextEditor editor, String actionID) {
		return (editor == null ? null : editor.getAction(actionID));
	}
	
	/**
	 * Sets the active page.
	 * @param part the edit part
	 */
	public void setActivePage(IEditorPart part) {
		if (activeEditorPart == part)
			return;

		activeEditorPart = part;

		IActionBars actionBars = getActionBars();
		if (actionBars != null) {

			ITextEditor editor =
				(part instanceof ITextEditor) ? (ITextEditor) part : null;

			actionBars.setGlobalActionHandler(
					ActionFactory.DELETE.getId(),
				getAction(editor, ITextEditorActionConstants.DELETE));
			actionBars.setGlobalActionHandler(
					ActionFactory.UNDO.getId(),
				getAction(editor, ITextEditorActionConstants.UNDO));
			actionBars.setGlobalActionHandler(
					ActionFactory.REDO.getId(),
				getAction(editor, ITextEditorActionConstants.REDO));
			actionBars.setGlobalActionHandler(
					ActionFactory.CUT.getId(),
				getAction(editor, ITextEditorActionConstants.CUT));
			actionBars.setGlobalActionHandler(
					ActionFactory.COPY.getId(),
				getAction(editor, ITextEditorActionConstants.COPY));
			actionBars.setGlobalActionHandler(
					ActionFactory.PASTE.getId(),
				getAction(editor, ITextEditorActionConstants.PASTE));
			actionBars.setGlobalActionHandler(
					ActionFactory.SELECT_ALL.getId(),
				getAction(editor, ITextEditorActionConstants.SELECT_ALL));
			actionBars.setGlobalActionHandler(
				ActionFactory.FIND.getId(),
				getAction(editor, ITextEditorActionConstants.FIND));
			actionBars.setGlobalActionHandler(
					IDEActionFactory.BOOKMARK.getId(),
				getAction(editor, IDEActionFactory.BOOKMARK.getId()));
			actionBars.updateActionBars();
		}
	}


	/**
	 * Builds actions.
	 * @see org.eclipse.gef.ui.actions.ActionBarContributor#buildActions()
	 */
	protected void buildActions()
	{
		// PropertiesDialogAction
		propertiesDialogAction = new EditAction(this.getPage().getActiveEditor());
		propertiesDialogAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(
		ISharedImages.IMG_DEF_VIEW));
		addAction(propertiesDialogAction);
		
		// Add Attribute
		addAttributeAction = new AddAttributeAction(this.getPage().getActiveEditor());
		addAttributeAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(
		ISharedImages.IMG_DEF_VIEW));
		addAction(addAttributeAction);
		
		// Add Method
		addMethodAction = new AddMethodAction(this.getPage().getActiveEditor());
		addMethodAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(
		ISharedImages.IMG_DEF_VIEW));
		addAction(addMethodAction);
		
		// Add Condition
		addConditionAction = new AddConditionAction(this.getPage().getActiveEditor());
		addConditionAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(
		ISharedImages.IMG_DEF_VIEW));
		addAction(addConditionAction);
		
		// 	Convert
		convertAction = new ConvertAction(this.getPage().getActiveEditor());
		convertAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(
		ISharedImages.IMG_DEF_VIEW));
		addAction(convertAction);

		// 	Reset Label position
		labelAction = new ResetLabelPositionAction(this.getPage().getActiveEditor());
		labelAction.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(
		ISharedImages.IMG_DEF_VIEW));
		addAction(labelAction);

		addRetargetAction(new UndoRetargetAction());
		addRetargetAction(new RedoRetargetAction());
		addRetargetAction(new DeleteRetargetAction());
    
		addRetargetAction(new ZoomInRetargetAction());
		addRetargetAction(new ZoomOutRetargetAction());
	}

	/**
	 * Declares global action keys.
	 * @see org.eclipse.gef.ui.actions.ActionBarContributor#declareGlobalActionKeys()
	 */
	protected void declareGlobalActionKeys()
	{
		addGlobalActionKey(ActionFactory.PRINT.getId());
		addGlobalActionKey(ActionFactory.DELETE.getId());
	}
    
	/**
	 * Contirbutes to the menu.
	 * @see org.eclipse.ui.part.EditorActionBarContributor#contributeToMenu(org.eclipse.jface.action.IMenuManager)
	 */
	public void contributeToMenu(IMenuManager menuManager)
	{
		super.contributeToMenu(menuManager);

		// add a "View" menu after "Edit"
		MenuManager viewMenu = new MenuManager("View");
		viewMenu.add(getAction(GEFActionConstants.ZOOM_IN));
		viewMenu.add(getAction(GEFActionConstants.ZOOM_OUT));
        
		menuManager.insertAfter(IWorkbenchActionConstants.M_EDIT, viewMenu);
	}



    
	/**
	 * Contributes to the tool bar.
	 * @see org.eclipse.ui.part.EditorActionBarContributor#contributeToToolBar(org.eclipse.jface.action.IToolBarManager)
	 */
	public void contributeToToolBar(IToolBarManager toolBarManager)
	{
		super.contributeToToolBar(toolBarManager);

		toolBarManager.add(getAction(ActionFactory.UNDO.getId()));
		toolBarManager.add(getAction(ActionFactory.REDO.getId()));
    
		toolBarManager.add(new Separator());
		toolBarManager.add(new ZoomComboContributionItem(getPage()));
		
		toolBarManager.add(new Separator());
		toolBarManager.add(propertiesDialogAction);
		toolBarManager.add(new Separator());
		toolBarManager.add(addAttributeAction);
		toolBarManager.add(addMethodAction);
		toolBarManager.add(addConditionAction);
		toolBarManager.add(convertAction);
	}
}
