package vocleditor.commands;

import VOCLModel.ClassifierRole;
import VOCLModel.Context;
import VOCLModel.Method;

/**
 * Clears the Context to "undefined" in an undoable way.
 * This sub command is used by DeleteClassifierRoleCommand and others
 * if their model elements are related to the current constraint context.
 * @author bjoern
 */
public class ClearContextSubCommand 
{
	private Context context;
	private ClassifierRole contextClassifier;
	private Method contextMethod;
	private String contextType;
	private String contextName;
	private boolean contestIsClassifier;

	/**
	 * creates a new ClearContextSubCommand.
	 * @param context the context to be cleared
	 */
	public ClearContextSubCommand( Context context ) 
	{
		this.context = context;
		this.contextClassifier = context.getContextClassifier();
		this.contextMethod = context.getContextMethod();
		this.contextType = context.getType();
		this.contextName = context.getName();
		this.contestIsClassifier = context.isIsClassifier();
	}

	/**
	 * command execution, sets context to undefined
	 *
	 */
	public void execute()
	{			
		context.setContextClassifier( null );
		context.setContextMethod( null );
		context.setType( "undefined" );
		context.setName( "" );
		context.setIsClassifier( false );
		if (contestIsClassifier) contextClassifier.setContext(null);
		else contextMethod.setContext(null);
		
	}
	
	/**
	 * undoes command execution - resets context  
	 */
	public void undo()
	{
		context.setContextClassifier( contextClassifier );
		context.setContextMethod( contextMethod );
		context.setType( contextType );
		context.setName( contextName );
		context.setIsClassifier( contestIsClassifier );	
		if (contestIsClassifier) contextClassifier.setContext(context);
		else contextMethod.setContext(context);
	}

	/**
	 * redoes command execution  
	 */
	public void redo()
	{
		execute();
	}
	
}
