package vocleditor.commands;

import java.util.Iterator;
import java.util.Vector;

import org.eclipse.emf.ecore.EObject;

import vocleditor.model.VOCLTools;

import VOCLModel.Constraint;
import VOCLModel.LetExp;
import VOCLModel.NavExp;
import VOCLModel.VariableDeclaration;

/**
 * Clears the Let Expression to "no variable" in an undoable way.
 * This sub command is used by DeleteClassifierRoleCommand and others
 * if their (deleted) variables are related to the a Let Expression.
 * @author bjoern
 */
public class ClearLetExpsSubCommand 
{
	private Vector letexps   = new Vector();
	private Vector variables = new Vector();
	private Object placeHolder = new Object();

	/**
	 * creates a new ClearLetExpSubCommand. 
	 * @param navexp the navexp there where something is deleted from
	 */
	public ClearLetExpsSubCommand( NavExp navexp ) 
	{
		EObject eo = navexp;
		// find all letexps from the given navexp down to the constraint
		while ( ! (eo instanceof Constraint) ) {
			if ( eo instanceof LetExp ) {
				letexps.add( eo );
				if ( ((LetExp) eo).getLetVar() != null )
					variables.add( ((LetExp) eo).getLetVar() );
				else
					variables.add( placeHolder );
			}
			eo = eo.eContainer();
		}
	}

	/**
	 * command execution.
	 * Sets letexps to "no variable" if there variables are not references from the Let Expression anymore
	 */
	public void execute()
	{			
		for ( Iterator i = letexps.iterator(); i.hasNext(); ) {
			LetExp le = (LetExp) i.next();
			Vector variables = new Vector();
			VOCLTools.collectVariables( (NavExp) le.getLet(), variables );
			if ( le.getLetVar() != null && ! variables.contains( le.getLetVar() ) ) {
				le.setLetVar( null );
			}
		}
	}
	
	/**
	 * undoes command execution - resets context  
	 */
	public void undo()
	{
		for ( int i = 0; i < letexps.size(); i++ ) {
			LetExp le = (LetExp) letexps.get(i);
			if ( variables.get(i) != placeHolder )
				le.setLetVar( (VariableDeclaration) variables.get(i) );
		}
	}

	/**
	 * redoes command execution  
	 */
	public void redo()
	{
		execute();
	}
}
