/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;


import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.swt.widgets.Shell;

import vocleditor.VOCLEditorPlugin;

import VOCLModel.Attribute;
import VOCLModel.ClassifierRole;
import VOCLModel.NavExp;



/**
 * Command for creating a asscociation role.
 */
public class CreateClassifierRoleCommand extends org.eclipse.gef.commands.Command {
	private	ClassifierRole classifierrole;
	private Point loc;
	private NavExp parent;
	private Shell shell;
	private String name= "";
	private String type = "";
	private List  attributes=new ArrayList();
	private List attrVars = new ArrayList();

	/**
	 * The default constructor.
	 */	
	public CreateClassifierRoleCommand() {
		setLabel("create classifier role");
	}
	
	/**
	 * Entry point for command execution.
	 */
	public void execute() {   	
		redo();
		if (loc != null) {
			classifierrole.setX(loc.x);
			classifierrole.setY(loc.y);
		}
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		parent.getClassifierRoles().add( classifierrole );
		classifierrole.setName(name);
		classifierrole.setType(type);
		for (int i=0;i<attributes.size();++i) 
			classifierrole.getAttributes().add(attributes.get(i));
		
		VOCLEditorPlugin.getConstraint( parent ).getAllVariables().addAll( attrVars );
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		parent.getClassifierRoles().remove(classifierrole);
		VOCLEditorPlugin.getConstraint( parent ).getAllVariables().removeAll( attrVars );
	}
	
	/**
	 * Sets the classifier role.
	 * @param classifierrole the classifier role to set
	 */
	public void setClassifierRole(ClassifierRole classifierrole ) {
		this.classifierrole = classifierrole;
	}
	
	/**
	 * Returns the classifier role.
	 * @return the classifier role
	 */
	public ClassifierRole getClassifierRole() {
		return classifierrole;
	}
	
	/**
	 * Sets the location.
	 * @param loc the location to set
	 */
	public void setLocation (Point loc) {
		this.loc = loc;
	}
	
	/**
	 * Returns the parent navigation expression.
	 * @return the parent navigation expression
	 */
	public NavExp getParent() {
		return parent;
	}
	
	/**
	 * Sets the parent navigation expression.
	 * @param newParent the parent navigation expression to set
	 */
	public void setParent(NavExp newParent) {
		parent = newParent;
	}
	
	/**
	 * Sets the classifier role name.
	 * @param name the classifier role name
	 */
	public void setName(String name) {
		this.name=name;
	}
	
	/**
	 * Sets the classifier role type
	 * @param type the classifier role type 
	 */
	public void setType(String type){
		this.type=type;
	}
	
	/**
	 * Sets the list of attributes.
	 * @param attributes the list of attributes to set
	 */
	public void setAttributes(List attributes){
		this.attributes=attributes;
		for ( int i = 0; i < attributes.size(); i++ ) {
			Attribute attr = (Attribute) attributes.get(i);
			if ( attr.getVariable() != null )
				attrVars.add( attr.getVariable() );
		}
	}	
	
	/**
	 * Sets the command attributes.
	 * @param name the classifier role name
	 * @param type the classifier role type
	 * @param attributes the list of attributes
	 */
	public void setCommandAttributes(String name, String type, List attributes) {
		setName(name);
		setType(type);
		setAttributes(attributes);		
	}
}
