/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;


import org.eclipse.draw2d.geometry.Point;
import org.eclipse.swt.widgets.Shell;

import VOCLModel.IfExp;
import VOCLModel.NavExp;

/**
 * Command for creating a boolean expression role.
 * @author bjoern
 */
public class CreateIfExpCommand extends org.eclipse.gef.commands.Command 
{
	private	IfExp ifexp;
	private Point loc;
	private NavExp parent;
	private Shell shell;
	private boolean isHorizontal;
	private boolean isNegated;
	private int splitPercentage1;
	private int splitPercentage2;
	
	/**
	 * Constructor.
	 *
	 */	
	public CreateIfExpCommand( IfExp ifexp, NavExp parent) 
	{
		setLabel("create bool expression");
		this.ifexp = ifexp;
		this.parent = parent;
	}
	
	/**
	 * Returns the if expression.
	 * @return the if expression
	 */
	public IfExp getIfExp()
	{
		return ifexp;
	}
	
	/**
	 * Sets the command attributes.
	 * @param isHorizontal the horizontal flag
	 * @param splitPercentage1 the split percentage 1
	 * @param splitPercentage2 the split percentage 2
	 * @param isNegated the flag shows the negation
	 */
	public void setCommandAttributes( boolean isHorizontal, int splitPercentage1, int splitPercentage2, boolean isNegated )
	{
		this.splitPercentage1 = splitPercentage1;
		this.splitPercentage2 = splitPercentage2;
		this.isHorizontal = isHorizontal;
		this.isNegated = isNegated;
	}
	
	/**
	 * Entry point for command execution.
	 */
	public void execute() 
	{   
		parent.getExpressions().add( ifexp );
		if (loc != null) {
			ifexp.setX(loc.x);
			ifexp.setY(loc.y);
		}
		ifexp.setSplitHorizontal( isHorizontal );
		ifexp.setSplitPercent1( splitPercentage1 );
		ifexp.setSplitPercent2( splitPercentage2 );
		ifexp.setNegated( isNegated );
	}
		
	/**
	 * Redo command execution.
	 */
	public void redo() {
		parent.getExpressions().add( ifexp );
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		parent.getExpressions().remove( ifexp );
	}
	
	/**
	 * Sets the location.
	 * @param loc the location to set
	 */
	public void setLocation (Point loc) {
		this.loc = loc;
	}	
}
