/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.commands.Command;
import org.eclipse.swt.widgets.Shell;

import vocleditor.VOCLEditorPlugin;

import VOCLModel.Method;
import VOCLModel.ClassifierRole;
import VOCLModel.VariableDeclaration;


/**
 * @author tugstugi
 * Command for creating a method.
 */
public class CreateMethodCommand extends Command {
	private	Method method;
	private ClassifierRole parent;
	private Shell shell;
	private String methodName;
	private String returnType;
	private VariableDeclaration returnVariable;
	private List parameters = new ArrayList();
	
	/**
	 * The default constructor.
	 */	
	public CreateMethodCommand() {
		setLabel("create method");
	}
	
	/**
	 * Entry point for command execution.
	 */
	public void execute() {  
		redo();
	}	
	
	/**
	 * Redo command execution.
	 */
	public void redo() {
		parent.setMethod(method);		
		method.setName(methodName);
		method.setReturnType(returnType);
		method.setClassifierRole(parent);
		method.setReturnVariable(returnVariable);
		method.getArguments().addAll(parameters);
		VOCLEditorPlugin.getConstraint( method ).getAllVariables().addAll( parameters );
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		method.getArguments().clear();
		method.setReturnVariable( null );
		VOCLEditorPlugin.getConstraint( method ).getAllVariables().removeAll( parameters );
		parent.setMethod(null);
	}
	
	/**
	 * Sets the method.
	 * @param method the methos to set
	 */
	public void setMethod(Method method ) {
		this.method = method;
	}
	
	/**
	 * Returns the method. 
	 * @return the method
	 */
	public Method getMethod(){
		return method;
	}
	
	/**
	 * Returns the parent classifier role.
	 * @return the parent classifier role
	 */
	public ClassifierRole getParent() {
		return parent;
	}
	
	/**
	 * Sets the parent classifier role.
	 * @param newParent the parent to set
	 */
	public void setParent(ClassifierRole newParent) {
		parent = newParent;
	}
	
	/**
	 * Sets the name.
	 * @param methodName the name to set
	 */	
	public void setName(String methodName) {
		this.methodName = methodName;
	}
	
	/**
	 * Sets the returnType.
	 * @param returnType the return type to set
	 */
	public void setReturnType(String returnType){
		this.returnType = returnType;
	}
	
	/**
	 * Sets the return value.
	 * @param returnVariable the return value to set
	 */
	public void setReturnVariable(VariableDeclaration returnVariable){
		this.returnVariable = returnVariable;
	}
	
	/**
	 * Sets the list of parameters.
	 * @param parameters the list of the parameters
	 */
	public void setParameters(List parameters){
		this.parameters = parameters;
	}
	
	/**
	 * Sets the command attributes.
	 * @param name the method
	 * @param returnVariable the return variable
	 * @param parameters the parameter list
	 */
	public void setCommandAttributes(String name, String returnType, VariableDeclaration returnVariable, List parameters) {
		setName(name);
		setReturnType(returnType);
		setReturnVariable(returnVariable);
		setParameters(parameters);		
	}
}
