/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Iterator;
import java.util.Vector;


import vocleditor.VOCLEditorPlugin;
import vocleditor.model.VOCLTools;

import VOCLModel.ClassifierRole;
import VOCLModel.AssociationRole;
import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.Context;
import VOCLModel.Method;
import VOCLModel.NavExp;

/**
 * Command for deleting a ClassifierRole.
 */
public class DeleteClassifierRoleCommand extends org.eclipse.gef.commands.Command {
	private	ClassifierRole classifierrole;
	private NavExp parent; 
	private Constraint constraint;
	private Map postAssoMap = new HashMap();
	private Map preAssoMap = new HashMap();
	private List deletedVariables;
	
	private ClearContextSubCommand clearContextCommand = null;
	private ClearLetExpsSubCommand clearLetexpsCommand = null;
	private DeleteMethodCommand deleteMethodCommand = null;
	private List deleteConditionCommands = new Vector();
	
	/**
	 * Deletes a Classifier Role from a NavExp
	 */
	public DeleteClassifierRoleCommand() {
		setLabel("delete classifier role");
	}

	/**
	 * Delete the association roles which are connected with the classifier role.
	 * @param classifier role the classifier ole
	 */
	private void deleteAssociationRoles(ClassifierRole classifierrole) {
		for (Iterator it = classifierrole.getSource().iterator(); it.hasNext();) {
			AssociationRole assoRole = (AssociationRole) it.next();
			for (Iterator classIt = parent.getClassifierRoles().iterator(); classIt.hasNext(); ) {
				ClassifierRole classRole = (ClassifierRole) classIt.next();
				if (classRole.getTarget().contains(assoRole)) {
					postAssoMap.put(assoRole, classRole);
					classRole.getTarget().remove(assoRole);
				}
			}
			if (parent.getAssociationRoles().contains(assoRole)) {
				parent.getAssociationRoles().remove(assoRole);	
			}
		}
		for (Iterator it = classifierrole.getTarget().iterator(); it.hasNext();) {
			AssociationRole assoRole = (AssociationRole) it.next();
			for (Iterator classIt = parent.getClassifierRoles().iterator(); classIt.hasNext(); ) {
				ClassifierRole classRole = (ClassifierRole) classIt.next();
				if (classRole.getSource().contains(assoRole)) {
					preAssoMap.put(assoRole, classRole);
					classRole.getSource().remove(assoRole);
				}
			}
			if (parent.getAssociationRoles().contains(assoRole)) {
				parent.getAssociationRoles().remove(assoRole);	
			}
		}
	}
	
	/**
	 * Entry point for command execution.
	 */
	public void execute() {
		deleteAssociationRoles(classifierrole);
		if ( clearContextCommand != null )
			clearContextCommand.execute();
		if ( deleteMethodCommand != null )
			deleteMethodCommand.execute();
		parent.getClassifierRoles().remove(classifierrole);
		deletedVariables = VOCLTools.deleteUnusedVariables( constraint );
		clearLetexpsCommand = new ClearLetExpsSubCommand( parent );
		clearLetexpsCommand.execute();
		List conditions = VOCLEditorPlugin.getAllConditions(constraint);
		for(int i=0; i<conditions.size(); i++){
			Condition condition = (Condition) conditions.get(i);
			if(condition.getLeft() != null && deletedVariables.contains(condition.getLeft())
					|| condition.getRight() != null && deletedVariables.contains(condition.getRight())){
				DeleteConditionCommand deleteConditionCommand = new DeleteConditionCommand();
				deleteConditionCommand.setCondition(condition);
				deleteConditionCommand.setParent((NavExp) condition.eContainer());
				deleteConditionCommand.execute();
				deleteConditionCommands.add(deleteConditionCommand);
			}				
		}
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();		
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		for (Iterator it = deletedVariables.iterator(); it.hasNext();)
			constraint.getAllVariables().add( it.next() );

		for (Iterator it = classifierrole.getSource().iterator(); it.hasNext();) {
			AssociationRole assoRole = (AssociationRole) it.next();
			ClassifierRole classRole = (ClassifierRole) postAssoMap.get(assoRole);
			classRole.getTarget().add(assoRole);
			parent.getAssociationRoles().add(assoRole);	
		}
		for (Iterator it = classifierrole.getTarget().iterator(); it.hasNext();) {
			AssociationRole assoRole = (AssociationRole) it.next();
			ClassifierRole classRole = (ClassifierRole) preAssoMap.get(assoRole);
			classRole.getSource().add(assoRole);
			parent.getAssociationRoles().add(assoRole);	
		}
		parent.getClassifierRoles().add(classifierrole);
		if ( deleteMethodCommand != null )
			deleteMethodCommand.undo();
		if ( clearContextCommand != null ) 
			clearContextCommand.undo();
		
		for(int i=0; i<deleteConditionCommands.size(); i++){
			DeleteConditionCommand deleteConditionCommand = (DeleteConditionCommand) deleteConditionCommands.get(i);
			deleteConditionCommand.undo();
		}
		clearLetexpsCommand.undo();
	}

	/**
	 * sets the classifier to be deleted
	 * 
	 * Stores information about the classifier and eventually about the context
	 * to enable clean command execution.
	 * @param classifierrole
	 */
	public void setClassifierRole (ClassifierRole classifierrole) {
		this.classifierrole = classifierrole;
		this.constraint = VOCLEditorPlugin.getConstraint( classifierrole );
		Context context = VOCLEditorPlugin.getConstraint( classifierrole ).getContext();
		ClassifierRole concl = context.getContextClassifier();
		Method conmtd = context.getContextMethod();
		// if this is the context classifier or this.getMethod() is the context method: 
		if ( concl == classifierrole ||
			 ( conmtd != null &&  classifierrole.getMethod() == conmtd )) 
		{ 
			clearContextCommand = new ClearContextSubCommand( context );
		}
		// create a DeleteMethodCommand if a method exists
		if ( classifierrole.getMethod() != null ) {
			deleteMethodCommand = new DeleteMethodCommand( true );
			deleteMethodCommand.setMethod( classifierrole.getMethod() );
			deleteMethodCommand.setParent( classifierrole );
		}
	}

	/**
	 * Sets the parent navigation expression.
	 * @param voclmodel the parent navigation expression
	 */
	public void setParent( NavExp voclmodel) { 
		this.parent = voclmodel;
	}	
}
