/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.gef.commands.Command;

import vocleditor.VOCLEditorPlugin;
import vocleditor.model.VOCLTools;

import VOCLModel.*;

/**
 * Abstract Command for deleting a VOCLExpression.
 * This class cares for 
 *  - clearing the context if affected
 *  - deleting free VariableDeclarations from the constraint 
 * @author bjoern
 */
public abstract class DeleteExpCommand extends Command 
{
	private VOCLExpression exp;
	private NavExp parent;
	private Constraint constraint;
	private ClearContextSubCommand clearContextCommand = null;
	private List deletedVariables;
	private List deleteConditionCommands = new Vector();
	private ClearLetExpsSubCommand clearLetexps;
	
	/**
	 * Creates a new DeleteConstraintCommand.
	 */
	protected DeleteExpCommand() {}

	/**
	 * initializes this command with a expression object and classifiers within it
	 * @param exp The VOCLExpression to be deleted
	 * @param classifiers a list of classifiers within the VOCLExpression
	 */
	protected void init( VOCLExpression exp, List classifiers ) 
	{
		setLabel("delete expression");
		this.exp  = exp;
		this.parent = exp.getParentExp();
		this.constraint = VOCLEditorPlugin.getConstraint( parent );

		// check if something in this boolexp is context related
		Context context = constraint.getContext();
		List methods = VOCLTools.getAllMethods( classifiers, new Vector() );
		if ( classifiers.contains( context.getContextClassifier() ) || methods.contains( context.getContextMethod() ) )
			clearContextCommand = new ClearContextSubCommand( context );	
		clearLetexps = new ClearLetExpsSubCommand( parent );
	}
	
	/**
	 * Entry point for command execution.
	 */
	public void execute() {
		if ( clearContextCommand != null )
			clearContextCommand.execute();
		parent.getExpressions().remove( exp );	
		deletedVariables = VOCLTools.deleteUnusedVariables( constraint );
		
		List conditions = VOCLEditorPlugin.getAllConditions(constraint);
		for(int i=0; i<conditions.size(); i++){
			Condition condition = (Condition) conditions.get(i);
			if(condition.getLeft() != null && deletedVariables.contains(condition.getLeft())
					|| condition.getRight() != null && deletedVariables.contains(condition.getRight())){
				DeleteConditionCommand deleteConditionCommand = new DeleteConditionCommand();
				deleteConditionCommand.setCondition(condition);
				deleteConditionCommand.setParent((NavExp) condition.eContainer());
				deleteConditionCommand.execute();
				deleteConditionCommands.add(deleteConditionCommand);
			}				
		}
		clearLetexps.execute();
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		for (Iterator it = deletedVariables.iterator(); it.hasNext();)
			constraint.getAllVariables().add( it.next() );
		parent.getExpressions().add( exp );
		if ( clearContextCommand != null ) 
			clearContextCommand.undo();
		
		for(int i=0; i<deleteConditionCommands.size(); i++){
			DeleteConditionCommand deleteConditionCommand = (DeleteConditionCommand) deleteConditionCommands.get(i);
			deleteConditionCommand.undo();
		}
		clearLetexps.undo();
	}	
}
