/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import java.util.List;
import java.util.Vector;

import org.eclipse.gef.commands.Command;

import vocleditor.VOCLEditorPlugin;

import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.Context;
import VOCLModel.Method;
import VOCLModel.ClassifierRole;
import VOCLModel.NavExp;

/**
 * @author tugstugi
 * Command for deleting a place.
 */
public class DeleteMethodCommand extends Command {
	private	Method method;
	private ClassifierRole parent;	
	private Constraint constraint;

	private ClearContextSubCommand clearContextCommand = null;
	private boolean ignoreContext = false;  // set to true from DeleteClassifierCommand
	private List deleteConditionCommands = new Vector();
	private ClearLetExpsSubCommand clearLetexpsCommand = null;
	
	/**
	 * Creates a new DeleteMethodCommand.
	 */
	public DeleteMethodCommand() {
		setLabel("delete method");
	}

	/**
	 * Creates a new DeleteMethodCommand which doesnt care for the context.
	 */
	public DeleteMethodCommand( boolean ignoreContext ) {
		setLabel("delete method");
		this.ignoreContext = ignoreContext;
	}
	
	
	/**
	 * Sets the method to be deleted
	 * @param method the method to be deleted
	 */
	public void setMethod(Method method){
		this.method = method;
		constraint = VOCLEditorPlugin.getConstraint( method );
		Context context = constraint.getContext();
		if ( ! ignoreContext && context.getContextMethod() == method  )	{ 
			clearContextCommand = new ClearContextSubCommand( context );
		}
	}

	/**
	 * Sets the ClassifierRole the method is to be deleted from
	 * @param parent ClassifierRole which owns the method to be deleted
	 */	
	public void setParent(ClassifierRole parent) {
		this.parent = parent;
	}
	
	/**
	 * Entry point for command execution.
	 */
	public void execute() {		
		List deletedParameters = new Vector();
		parent.setMethod(null);
		if(method.getReturnVariable() != null){
			constraint.getAllVariables().remove(method.getReturnVariable());	
			deletedParameters.add(method.getReturnVariable());
		}
		constraint.getAllVariables().removeAll( method.getArguments() );
		deletedParameters.addAll(method.getArguments());

		clearLetexpsCommand = new ClearLetExpsSubCommand( parent.getNavExp() );
		clearLetexpsCommand.execute();
		
		if ( clearContextCommand != null )
			clearContextCommand.execute();
		List conditions = VOCLEditorPlugin.getAllConditions(constraint);
		for(int i=0; i<conditions.size(); i++){
			Condition condition = (Condition) conditions.get(i);
			if(condition.getLeft() != null && deletedParameters.contains(condition.getLeft())
					|| condition.getRight() != null && deletedParameters.contains(condition.getRight())){
				DeleteConditionCommand deleteConditionCommand = new DeleteConditionCommand();
				deleteConditionCommand.setCondition(condition);
				deleteConditionCommand.setParent((NavExp) condition.eContainer());
				deleteConditionCommand.execute();
				deleteConditionCommands.add(deleteConditionCommand);
			}				
		}
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}

	/**
	 * Undo command execution.
	 */
	public void undo() {
		parent.setMethod( method );
		if ( method.getReturnVariable() != null )
			constraint.getAllVariables().add( method.getReturnVariable() );
		constraint.getAllVariables().addAll( method.getArguments() );
		if ( clearContextCommand != null ) 
			clearContextCommand.undo();
		
		for(int i=0; i<deleteConditionCommands.size(); i++){
			DeleteConditionCommand deleteConditionCommand = (DeleteConditionCommand) deleteConditionCommands.get(i);
			deleteConditionCommand.undo();
		}
		clearLetexpsCommand.undo();
	}
	
}
