/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Iterator;
import java.util.Vector;

import org.eclipse.gef.commands.Command;

import vocleditor.VOCLEditorPlugin;
import vocleditor.model.VOCLTools;

import VOCLModel.ClassifierRole;
import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.SetClassifierRole;
import VOCLModel.AssociationRole;
import VOCLModel.NavExp;

/**
 * @author tugstugi
 * Command for deleting a set classifier role.
 */
public class DeleteSetClassifierRoleCommand extends Command {
	private	SetClassifierRole setClassifierRole;
	private NavExp parent;  
	private Map postAssociationRolesMap = new HashMap();
	private Map preAssociationRolesMap = new HashMap();
	private List deletedVariables = new ArrayList();
	private List deleteConditionCommands = new Vector();
	private ClearLetExpsSubCommand clearLetexpsCommand = null;
	
	/**
	 * The constructor.
	 */
	public DeleteSetClassifierRoleCommand() {
		setLabel("delete set classifier role");
	}

	/**
	 * Delete the association roles which are connected with the classifier roles.
	 * @param classifierRole the classifier role
	 */
	private void deleteAssociationRoles(ClassifierRole classifierrole) {
		for (Iterator it = classifierrole.getSource().iterator(); it.hasNext();) {
			AssociationRole assoRole = (AssociationRole) it.next();
			for (Iterator classIt = parent.getClassifierRoles().iterator(); classIt.hasNext(); ) {
				ClassifierRole classRole = (ClassifierRole) classIt.next();
				if (classRole.getTarget().contains(assoRole)) {
					postAssociationRolesMap.put(assoRole, classRole);
					classRole.getTarget().remove(assoRole);
				}
			}
			if (parent.getAssociationRoles().contains(assoRole)) {
				parent.getAssociationRoles().remove(assoRole);	
			}
		}
		for (Iterator it = classifierrole.getTarget().iterator(); it.hasNext();) {
			AssociationRole assoRole = (AssociationRole) it.next();
			for (Iterator classIt = parent.getClassifierRoles().iterator(); classIt.hasNext(); ) {
				ClassifierRole classRole = (ClassifierRole) classIt.next();
				if (classRole.getSource().contains(assoRole)) {
					preAssociationRolesMap.put(assoRole, classRole);
					classRole.getSource().remove(assoRole);
				}
			}
			if (parent.getAssociationRoles().contains(assoRole)) {
				parent.getAssociationRoles().remove(assoRole);	
			}
		}
	}
	
	/**
	 * Entry point for command execution.
	 */
	public void execute() {
		deleteAssociationRoles(setClassifierRole);
		parent.getClassifierRoles().remove(setClassifierRole);
		Constraint constraint = VOCLEditorPlugin.getConstraint( parent);
		deletedVariables = VOCLTools.deleteUnusedVariables(constraint);

		clearLetexpsCommand = new ClearLetExpsSubCommand( parent );
		clearLetexpsCommand.execute();

		List conditions = VOCLEditorPlugin.getAllConditions(constraint);
		for(int i=0; i<conditions.size(); i++){
			Condition condition = (Condition) conditions.get(i);
			if(condition.getLeft() != null && deletedVariables.contains(condition.getLeft())
					|| condition.getRight() != null && deletedVariables.contains(condition.getRight())){
				DeleteConditionCommand deleteConditionCommand = new DeleteConditionCommand();
				deleteConditionCommand.setCondition(condition);
				deleteConditionCommand.setParent((NavExp) condition.eContainer());
				deleteConditionCommand.execute();
				deleteConditionCommands.add(deleteConditionCommand);
			}				
		}
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		Constraint constraint = VOCLEditorPlugin.getConstraint( parent);
		for (Iterator it = deletedVariables.iterator(); it.hasNext();)
			constraint.getAllVariables().add( it.next() );
		
		for (Iterator it = setClassifierRole.getSource().iterator(); it.hasNext();) {
			AssociationRole assoRole = (AssociationRole) it.next();
			ClassifierRole classRole = (ClassifierRole) postAssociationRolesMap.get(assoRole);
			classRole.getTarget().add(assoRole);
			parent.getAssociationRoles().add(assoRole);	
		}
		for (Iterator it = setClassifierRole.getTarget().iterator(); it.hasNext();) {
			AssociationRole assoRole = (AssociationRole) it.next();
			ClassifierRole classRole = (ClassifierRole) preAssociationRolesMap.get(assoRole);
			classRole.getSource().add(assoRole);
			parent.getAssociationRoles().add(assoRole);	
		}
		parent.getClassifierRoles().add(setClassifierRole);		
		
		for(int i=0; i<deleteConditionCommands.size(); i++){
			DeleteConditionCommand deleteConditionCommand = (DeleteConditionCommand) deleteConditionCommands.get(i);
			deleteConditionCommand.undo();
		}
		clearLetexpsCommand.undo();
	}	

	/**
	 * Returns the parent nav exp.
	 * @return the parent nav exp
	 */
	public NavExp getParent() {
		return parent;
	}
	
	/**
	 * Sets the parent nav exp.
	 * @param newParent the new parent nav exp to set
	 */
	public void setParent(NavExp newParent) {
		parent = newParent;
	}
	
	/**
	 * Returns the set classifier role.
	 * @return the set classifier role
	 */
	public SetClassifierRole getSetClassifierRole() {
		return setClassifierRole;
	}
	
	/**
	 * Sets the set classifier role.
	 * @param setClassifierRole the set classifier role to set
	 */
	public void setSetClassifierRole(SetClassifierRole setClassifierRole ) {
		this.setClassifierRole = setClassifierRole;			
	}	
}
