/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import org.eclipse.gef.commands.Command;

import VOCLModel.AssociationRole;
import VOCLModel.ClassifierRole;
import VOCLModel.NavExp;

/**
 * Edit association role properties.
 */
public class EditAssociationRoleCommand extends Command{
	private AssociationRole associationrole;
	private NavExp parent; 
	private String name_old = "";
	private String name_new = "";
	private ClassifierRole source_old; 
	private ClassifierRole source_new;
	private ClassifierRole target_old;
	private ClassifierRole target_new;
	private byte routingType_old;
	private byte routingType_new;
	
	/**
	 * The default constructor.
	 */
	public EditAssociationRoleCommand(){
		setLabel("edit association role");
	}
	
	/**
	 * Checks whether {@link #source_new}, {@link #source_old},
	 * {@link #target_new} and {@link #target_old} are not null.
	 */
	public boolean canExecute() {
		if(source_old != null &&
				source_new != null &&
				target_old != null &&
				target_new != null)
			return true;
		return false;
	
	}
	
	/**
	 * @param associationrole The {@link AssociationRole} to modify.
	 */
	public void setAssociationRole(AssociationRole associationrole) {
		this.associationrole = associationrole;
		name_old = associationrole.getName();
		source_old = associationrole.getAssociationSource();
		target_old = associationrole.getAssociationTarget();
		routingType_old = associationrole.getRoutingType();
	}

	/**
	 * Command execution - Swaps target and source if necessary
	 */
	public void execute() {	
		associationrole.setName(name_new);
		if(source_new != source_old && target_new != target_old) {
			source_old.getTarget().remove(associationrole);
			source_new.getTarget().add(associationrole);
			target_old.getSource().remove(associationrole);
			target_new.getSource().add(associationrole);
			associationrole.setAssociationSource(source_new);
			associationrole.setAssociationTarget(target_new);
			int temp = associationrole.getSourceAnchorAt();
			associationrole.setSourceAnchorAt(associationrole.getTargetAnchorAt());
			associationrole.setTargetAnchorAt(temp);
		}
		associationrole.setRoutingType(routingType_new);
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		associationrole.setName(name_old);
		if(source_new != source_old && target_new != target_old) {
			associationrole.setAssociationTarget(target_old);
			associationrole.setAssociationSource(source_old);
			source_new.getTarget().remove(associationrole);
			source_old.getTarget().add(associationrole);
			target_new.getSource().remove(associationrole);
			target_old.getSource().add(associationrole);
			int temp = associationrole.getSourceAnchorAt();
			associationrole.setSourceAnchorAt(associationrole.getTargetAnchorAt());
			associationrole.setTargetAnchorAt(temp);
		}
		associationrole.setRoutingType(routingType_old);
	}
	
	/**
	 * Redo command execution.
	 */
	public void redo() {
	     execute();
	}
	
	/**
	 * Sets the source classifier role.
	 * @param source the source classifier role to set
	 */
	public void setSource(ClassifierRole source) {
		this.source_new=source;
	}
	
	/**
	 * Sets the target classifier role.
	 * @param target the target classifier role to set
	 */
	public void setTarget(ClassifierRole target) {
		this.target_new=target;
	}
	
	/**
	 * Sets the association role name.
	 * @param name the association role name
	 */
	public void setName (String name) {
		this.name_new=name;
	}
	
	/**
	 * Sets the command attributes.
	 * @param source the source classifier role
	 * @param target the target classifier role
	 * @param name the association role name
	 * @param routingType the routing type
	 */
	public void setCommandAttributes (ClassifierRole source,ClassifierRole target,String name, byte routingType) {
		setSource(source);
		setTarget(target);
		setName(name);
		routingType_new = routingType;
	}
}
