/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import java.util.List;
import java.util.Vector;

import org.eclipse.gef.commands.Command;

import vocleditor.VOCLEditorPlugin;
import vocleditor.model.ModelCreationFactory;
import VOCLModel.Attribute;
import VOCLModel.ClassifierRole;
import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.NavExp;
import VOCLModel.SetClassifierRole;
import VOCLModel.VariableDeclaration;

/**
 * Command class for edit the attribute properties.
 */
public class EditAttributeCommand extends Command {
	private ClassifierRole parent;
	private Attribute attribute;
	private String oldName, oldType, oldValue;
	private String newName, newType, newValue;
	private String newVariableName, oldVariableName;
	private VariableDeclaration newVariable, oldVariable;
	private VariableDeclaration iteratorVariable = null;
	private VariableDeclaration sumVariable;
	private List deleteConditionCommands = new Vector();
	
	/**
	 * The default constructor.
	 */
	public EditAttributeCommand(){
		setLabel("edit attribute");
	}

	/**
	 * Executes the command.
	 */
	public void execute() {
		Constraint constraint = VOCLEditorPlugin.getConstraint( parent );
		attribute.setName(newName);
		attribute.setType(newType);
		attribute.setValue(newValue);
		if(oldVariable != null){
			if(newVariableName != null && !"".equals(newVariableName)){
				oldVariable.setName(newVariableName);
				oldVariable.setType(newType);
				attribute.setVariable(oldVariable);
			}else{			
				attribute.setVariable(null);
				if(parent instanceof SetClassifierRole && "sum".equals(((SetClassifierRole) parent).getOpKind())){
					if(oldVariable.equals(((SetClassifierRole) parent).getIterator())){
						iteratorVariable = oldVariable;
						sumVariable = ((SetClassifierRole) parent).getSum();
						((SetClassifierRole) parent).setOpKind("");
						((SetClassifierRole) parent).setIterator(null);
						((SetClassifierRole) parent).setSum(null);
					}
				}
				
				List conditions = VOCLEditorPlugin.getAllConditions(constraint);
				for(int i=0; i<conditions.size(); i++){
					Condition condition = (Condition) conditions.get(i);
					VariableDeclaration left = condition.getLeft();
					VariableDeclaration right = condition.getRight();
					if(left.equals(oldVariable) || ((right!=null) && (right.equals(oldVariable)))
							|| left.equals(sumVariable) || ((right!=null) && (right.equals(sumVariable)))){
						DeleteConditionCommand deleteConditionCommand = new DeleteConditionCommand();
						deleteConditionCommand.setCondition(condition);
						deleteConditionCommand.setParent((NavExp) condition.eContainer());
						deleteConditionCommand.execute();
						deleteConditionCommands.add(deleteConditionCommand);
					}
				}
				
				constraint.getAllVariables().remove(oldVariable);
				if(sumVariable != null)
					constraint.getAllVariables().remove(sumVariable);
			}
		}else{
			if(newVariable != null){
				VOCLEditorPlugin.getConstraint( parent ).getAllVariables().add(newVariable);
				attribute.setVariable(newVariable);
			}
		}
	}
	
	/** 
	 * Undo command execution.
	 */
	public void undo() {
		attribute.setName( oldName );
		attribute.setType( oldType );
		attribute.setValue( oldValue );		
		if(oldVariable != null){
			if(newVariableName != null && !"".equals(newVariableName)){
				oldVariable.setName(oldVariableName);
				oldVariable.setName(oldType);
			}else{
				VOCLEditorPlugin.getConstraint( parent ).getAllVariables().add(oldVariable);
			}
		}else{
			if(newVariable != null){
				VOCLEditorPlugin.getConstraint( parent ).getAllVariables().remove(newVariable);
			}
		}
		
		attribute.setVariable( oldVariable );
		
		if(parent instanceof SetClassifierRole && iteratorVariable != null && sumVariable != null){			
			VOCLEditorPlugin.getConstraint( parent ).getAllVariables().add(sumVariable);
			((SetClassifierRole) parent).setOpKind("sum");
			((SetClassifierRole) parent).setIterator(iteratorVariable);
			((SetClassifierRole) parent).setSum(sumVariable);
		}
		
		for(int i=0; i<deleteConditionCommands.size(); i++){
			DeleteConditionCommand deleteConditionCommand = (DeleteConditionCommand) deleteConditionCommands.get(i);
			deleteConditionCommand.undo();
		}
	}
	
	/**
	 * Redo command execution.
	 */
	public void redo(){
		execute();
	}
	
	/**
	 * Sets the attribute.
	 * @param attribute the attribute
	 */
	public void setAttribute(Attribute attribute) {
		this.attribute = attribute;
		oldName = attribute.getName();
		oldType = attribute.getType();
		oldValue = attribute.getValue();
		oldVariable = attribute.getVariable();
		if(oldVariable != null)
			oldVariableName = oldVariable.getName();
	}
	
	/**
	 * Sets the parent classifier role.
	 * @param newParent the parent
	 */
	public void setParent(ClassifierRole newParent){
		this.parent = newParent;
	}
	
	/**
	 * Sets the name of the attribute.
	 * @param name the name
	 */
	public void setName(String name) {
		newName=name;
	}
	
	/**
	 * Sets the type of the attribute.
	 * @param type the type
	 */
	public void setType(String type) {
		newType=type;
	}
	
	/**
	 * Sets the value of the attribute.
	 * @param value the value
	 */
	public void setValue(String value) {
		newValue = value;
	}
	
	/**
	 * Sets the variable name.
	 * @param variableName the variable name
	 */
	public void setVariableName(String variableName){
		newVariableName = variableName;		
		if(oldVariable == null && newVariableName != null && !"".equals(newVariableName)){
			newVariable = (VariableDeclaration) (new ModelCreationFactory(VariableDeclaration.class)).getNewObject();
			newVariable.setName(newVariableName);
			newVariable.setType(newType);
		}
	}
	
	/**
	 * Sets the command attributes.
	 * @param name the attribute name
	 * @param type the attribute type
	 * @param value the attribute value
	 * @param variableName the attribute variable name
	 */
	public void setCommandAttributes(String name, String type, String value, String variableName) {
		setName(name);
		setType(type);
		setValue(value);
		setVariableName(variableName);
	}
}
