/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import org.eclipse.gef.commands.Command;

import VOCLModel.BoolExp;
import VOCLModel.BoolOperation;

/**
 * Command class for edit the properties of a boolean expression.
 * @author bjoern
 */
public class EditBoolExpCommand extends Command {
	private BoolExp boolexp;
	private boolean oldHorizontal, newHorizontal;
	private BoolOperation oldOp, newOp;
	private int oldSplitPercent, newSplitPercent;
	private boolean isNegated, wasNegated;
	
	/**
	 * creates a new EditBoolExpCommand
	 * @param boolexp the boolexp to be altered
	 */
	public EditBoolExpCommand( BoolExp boolexp )
	{
		setLabel("edit attribute");
		this.boolexp = boolexp;
		this.oldHorizontal = boolexp.isSplitHorizontal();
		this.oldOp = boolexp.getKind();
		this.oldSplitPercent = boolexp.getSplitPercent();
		this.wasNegated = boolexp.isNegated();
	}

	/**
	 * sets the new boolexp attributes to be set in this commmand
	 * @param operation  the new boolean operation kind
	 * @param isHorizontal the new split horizontal flag
	 */
	public void setCommandAttributes( BoolOperation operation, boolean isHorizontal, int splitPercentage, boolean isNegated )
	{
		this.newOp = operation;
		this.newSplitPercent = splitPercentage;
		this.newHorizontal = isHorizontal;
		this.isNegated = isNegated;
	}
	
	/**
	 * execute this command
	 */
	public void execute() {
		boolexp.setKind( newOp );
		boolexp.setSplitHorizontal( newHorizontal );
		boolexp.setSplitPercent( newSplitPercent );
		boolexp.setNegated( isNegated );
	}
	
	/** 
	 * Undo command execution.
	 */
	public void undo() {
		boolexp.setKind( oldOp );
		boolexp.setSplitHorizontal( oldHorizontal );
		boolexp.setSplitPercent( oldSplitPercent );
		boolexp.setNegated( wasNegated );
	}
	
	/**
	 * Redo command execution.
	 */
	public void redo(){
		execute();
	}	
}
