/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.ArrayList;
import java.util.Vector;

import org.eclipse.gef.commands.Command;

import vocleditor.VOCLEditorPlugin;

import VOCLModel.ClassifierRole;
import VOCLModel.Attribute;
import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.NavExp;
import VOCLModel.VariableDeclaration;



/**
 * Command class for edit the place properties.
 */
public class EditClassifierRoleCommand extends Command {	
	private ClassifierRole classifierRole;
	private String name_old = "";
	private String name_new = "";
	private String type_old = "";
	private String type_new = "";
	private List  attributes_old=new ArrayList();
	private List  attributes_new=new ArrayList();
	private Hashtable changedVariables = new Hashtable();
	private Hashtable initialOldVariables = new Hashtable();
	private Hashtable initialNewVariables = new Hashtable();	
	
	private Constraint constraint;
	private List deleteConditionCommands = new Vector();

	/**
	 * The default constructor.
	 */
	public EditClassifierRoleCommand(){
		setLabel("edit classifier role");
	}
	
	/**
	 * @param classifierRole The {@link ClassifierRole} to modify.
	 */
	public void setClassifierRole(ClassifierRole classifierRole) {
		this.classifierRole = classifierRole;
		name_old = classifierRole.getName();
		type_old = classifierRole.getType();
		for(int i=0; i<classifierRole.getAttributes().size(); i++){
			Attribute attribute = (Attribute) classifierRole.getAttributes().get(i);
			attributes_old.add(attribute);
			if(attribute.getVariable() != null)
				initialOldVariables.put(attribute, attribute.getVariable());
		}
		constraint = VOCLEditorPlugin.getConstraint(classifierRole);
	}

	/**
	 * Executes the command.
	 */
	public void execute() {
		List deletedVariables = new Vector();
		
		classifierRole.setName(name_new);
		classifierRole.setType(type_new);
		
		for (int i=0; i<attributes_old.size(); i++){
			Attribute attribute = (Attribute) attributes_old.get(i);
			if(!attributes_new.contains(attribute)){
				classifierRole.getAttributes().remove(attribute);
				if(constraint.getAllVariables().contains(attribute.getVariable())){
					VariableDeclaration oldVariable = (attribute).getVariable();
					constraint.getAllVariables().remove(oldVariable);
					deletedVariables.add(oldVariable);
					oldVariable.getAttributes().remove(attribute);
				}
			}
		}
		
		for (int i=0; i<attributes_new.size(); i++){
			Attribute attribute = (Attribute) attributes_new.get(i);
			if(! attributes_old.contains(attribute)){
				if(initialNewVariables.containsKey(attribute))
					attribute.setVariable((VariableDeclaration) initialNewVariables.get(attribute));
				classifierRole.getAttributes().add(attribute);
				if(attribute.getVariable() != null && !constraint.getAllVariables().contains(attribute.getVariable())){
					VariableDeclaration newVariable = attribute.getVariable();
					if(changedVariables.containsKey(newVariable)){
						VariableDeclaration oldVariable = (VariableDeclaration) changedVariables.get(newVariable);
						String oldName = oldVariable.getName();
						String oldType = oldVariable.getType();
						oldVariable.setName(newVariable.getName());
						oldVariable.setType(newVariable.getType());
						newVariable.setName(oldName);
						newVariable.setType(oldType);
						attribute.setVariable(oldVariable);
						constraint.getAllVariables().add(oldVariable);
						deletedVariables.remove(oldVariable);
					}else{				
						attribute.setVariable(newVariable);
						constraint.getAllVariables().add(newVariable);
					}
				}				
			}
		}
		
		List conditions = VOCLEditorPlugin.getAllConditions(constraint);
		for(int i=0; i<conditions.size(); i++){
			Condition condition = (Condition) conditions.get(i);
			if(condition.getLeft() != null && deletedVariables.contains(condition.getLeft())
					|| condition.getRight() != null && deletedVariables.contains(condition.getRight())){
				DeleteConditionCommand deleteConditionCommand = new DeleteConditionCommand();
				deleteConditionCommand.setCondition(condition);
				deleteConditionCommand.setParent((NavExp) condition.eContainer());
				deleteConditionCommand.execute();
				deleteConditionCommands.add(deleteConditionCommand);
			}	
		}	
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		classifierRole.setName(name_old);
		classifierRole.setType(type_old);
		
		for(int i=0; i<attributes_new.size(); i++){
			Attribute attribute = (Attribute) attributes_new.get(i);
			if(! attributes_old.contains(attribute)){
				classifierRole.getAttributes().remove(attribute);
				if(constraint.getAllVariables().contains(attribute.getVariable())){
					VariableDeclaration newVariable = (attribute).getVariable();
					constraint.getAllVariables().remove(newVariable);
					newVariable.getAttributes().remove(attribute);
				}
			}
		}
		
		for(int i=0; i<attributes_old.size(); i++){
			Attribute attribute = (Attribute) attributes_old.get(i);
			if(!attributes_new.contains(attribute)){			
				if(initialOldVariables.containsKey(attribute))
					attribute.setVariable((VariableDeclaration) initialOldVariables.get(attribute));
				classifierRole.getAttributes().add(attribute);
				if(attribute.getVariable() != null && !constraint.getAllVariables().contains(attribute.getVariable())){
					VariableDeclaration oldVariable = (attribute).getVariable();
					if(changedVariables.containsValue(oldVariable)){
						VariableDeclaration newVariable = null;
						Enumeration enumeration = changedVariables.keys();
						while(enumeration.hasMoreElements()){
							Object key = enumeration.nextElement();
							if(oldVariable.equals(changedVariables.get(key))){
								newVariable = (VariableDeclaration) key;
								break;
							}								
						}
						String newName = oldVariable.getName();
						String newType = oldVariable.getType();
						oldVariable.setName(newVariable.getName());
						oldVariable.setType(newVariable.getType());
						newVariable.setName(newName);
						newVariable.setType(newType);
						attribute.setVariable(oldVariable);
						attribute.setVariable(oldVariable);
						constraint.getAllVariables().add(oldVariable);
					}else{				
						attribute.setVariable(oldVariable);
						constraint.getAllVariables().add(oldVariable);
					}
				}
			}
		}
		
		for(int i=0; i<deleteConditionCommands.size(); i++){
			DeleteConditionCommand deleteConditionCommand = (DeleteConditionCommand) deleteConditionCommands.get(i);
			deleteConditionCommand.undo();
		}
	}
	
	/**
	 * Redo command execution.
	 */
	public void redo(){
		execute();
	}	
	
	/**
	 * Sets the new name.
	 * @param name the new name
	 */
	public void setName(String name) {
		this.name_new = name;
	}
	
	/**
	 * Sets the new type.
	 * @param type the new type
	 */
	public void setType(String type){
		this.type_new = type;
	}
	
	/**
	 * Sets the the hashtable of changed variables.
	 * @param changedVariables the hashtable of changed variables.
	 */
	public void setChangedVariables(Hashtable changedVariables){
		this.changedVariables = changedVariables;
	}
	
	/**
	 * Sets the new list of attributes.
	 * @param attributes the new list of attributes
	 */
	public void setAttributes(List attributes){
		for(int i=0; i<attributes.size(); i++){
			Attribute attribute = (Attribute) attributes.get(i);
			attributes_new.add(attribute);
			if(attribute.getVariable() != null)
				initialNewVariables.put(attribute, attribute.getVariable());
		}	
	}
	
	/**
	 * Sets the command attributes.
	 * @param name the new name
	 * @param type the new type
	 * @param changedVariables the list of changed variables
	 * @param attributes the new list of attributes
	 */
	public void setCommandAttributes(String name, String type, Hashtable changedVariables, List attributes) {
		setName(name);
		setType(type);		
		setChangedVariables(changedVariables);
		setAttributes(attributes);	
	}
}
