/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import org.eclipse.gef.commands.Command;

import VOCLModel.IfExp;

/**
 * Command class for edit the properties of a boolean expression.
 * @author bjoern
 */
public class EditIfExpCommand extends Command {
	private IfExp ifexp;
	private boolean oldHorizontal, newHorizontal;
	private int oldSplitPercent1, newSplitPercent1;
	private int oldSplitPercent2, newSplitPercent2;
	private boolean isNegated, wasNegated;
	
	/**
	 * creates a new EditIfExpCommand
	 * @param ifexp the ifexp to be altered
	 */
	public EditIfExpCommand( IfExp ifexp )
	{
		setLabel("edit if expression");
		this.ifexp = ifexp;
		this.oldHorizontal = ifexp.isSplitHorizontal();
		this.oldSplitPercent1 = ifexp.getSplitPercent1();
		this.oldSplitPercent2 = ifexp.getSplitPercent2();
		this.wasNegated = ifexp.isNegated();

	}

	/**
	 * sets the new ifexp attributes to be set in this commmand
	 * @param isHorizontal the new split horizontal flag
	 * @param splitPercentage1 Split percentage for upper and bottom part (or left/right if splitted vertically)
	 * @param splitPercentage2 Split percentage for left and right (or top/bottom if splitted vertically)
	 * @param isNegated indicating whether the Expression is negated
	 */
	public void setCommandAttributes( boolean isHorizontal, int splitPercentage1, int splitPercentage2, boolean isNegated )
	{
		this.newSplitPercent1 = splitPercentage1;
		this.newSplitPercent2 = splitPercentage2;
		this.newHorizontal = isHorizontal;
		this.isNegated = isNegated;
	}
	
	/**
	 * execute this command
	 */
	public void execute() {
		ifexp.setSplitHorizontal( newHorizontal );
		ifexp.setSplitPercent1( newSplitPercent1 );
		ifexp.setSplitPercent2( newSplitPercent2 );
		ifexp.setNegated( isNegated );
	}
	
	/** 
	 * Undo command execution.
	 */
	public void undo() {
		ifexp.setSplitHorizontal( oldHorizontal );
		ifexp.setSplitPercent1( oldSplitPercent1 );
		ifexp.setSplitPercent2( oldSplitPercent2 );
		ifexp.setNegated( wasNegated );
	}
	
	/**
	 * Redo command execution.
	 */
	public void redo(){
		execute();
	}	
}
