/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.ArrayList;
import java.util.Vector;

import org.eclipse.gef.commands.Command;

import vocleditor.VOCLEditorPlugin;
import vocleditor.model.ModelCreationFactory;
import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.Method;
import VOCLModel.NavExp;
import VOCLModel.VariableDeclaration;



/**
 * @author tugstugi
 * Command class for edit the place properties.
 */
public class EditMethodCommand extends Command {	
	private Method method;
	private String name_old = "";
	private String name_new = "";
	private String type_old = "";
	private String type_new = "";
    private VariableDeclaration varOld = null;
    private VariableDeclaration varNew = null;  // only exists oldVar == null 
	private String returnNameOld = null;
	private String returnNameNew = null;
	private List  parameters_old = new ArrayList();
	private List  parameters_new = new ArrayList();
	private Hashtable changedParameters = new Hashtable();
	private Constraint constraint;
	private List deleteConditionCommands = new Vector();
	
	/**
	 * The default constructor.
	 */
	public EditMethodCommand(){
		setLabel("edit method");
	}
	
	/**
	 * Sets the method.
	 * @param method the method to set.
	 */
	public void setMethod(Method method) {
		this.method = method;
		name_old = method.getName();
		type_old = method.getReturnType();
		varOld = method.getReturnVariable();
		if (  varOld != null )
			returnNameOld = varOld.getName();
		parameters_old.clear();
		parameters_old.addAll(method.getArguments());
		constraint = VOCLEditorPlugin.getConstraint( method );
	}

	/**
	 * Executes the command.
	 */
	public void execute() {
		redo();
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		method.setName(name_old);
		method.setReturnType(type_old);
		method.setReturnVariable( varOld );
		if ( varOld != null ) {
			varOld.setName( returnNameOld );
			varOld.setType( type_old );
			constraint.getAllVariables().add( varOld );			
		}
		if ( varNew != null )
			constraint.getAllVariables().remove( varNew );
		
		method.getArguments().clear();
		constraint.getAllVariables().removeAll(parameters_new);
		for(int i=0; i<parameters_old.size(); i++){
			VariableDeclaration oldParameter = (VariableDeclaration) parameters_old.get(i);
			if(changedParameters.containsValue(oldParameter)){
				VariableDeclaration newParameter = null;
				Enumeration enumeration = changedParameters.keys();
				while(enumeration.hasMoreElements()){
					Object key = enumeration.nextElement();
					if(oldParameter.equals(changedParameters.get(key))){
						newParameter = (VariableDeclaration) key;
						break;
					}								
				}
				String newName = oldParameter.getName();
				String newType = oldParameter.getType();
				oldParameter.setName(newParameter.getName());
				oldParameter.setType(newParameter.getType());
				newParameter.setName(newName);
				newParameter.setType(newType);
			}
			method.getArguments().add(oldParameter);
			constraint.getAllVariables().add(oldParameter);
		}
		
		for(int i=0; i<deleteConditionCommands.size(); i++){
			DeleteConditionCommand deleteConditionCommand = (DeleteConditionCommand) deleteConditionCommands.get(i);
			deleteConditionCommand.undo();
		}
	}
	
	/**
	 * Redo command execution.
	 */
	public void redo(){
		List deletedParameters = new Vector();
		method.setName(name_new);
		method.setReturnType(type_new);	
		// update or delete old returnVariable -- or create a new one
		if ( varOld != null ) {
			if ( returnNameNew != null ) {  // update return variable
				varOld.setName( returnNameNew );
				varOld.setType( type_new );
			} else {  // delete return variable				
				method.setReturnVariable( null );
				constraint.getAllVariables().remove( varOld );
				deletedParameters.add(varOld);
			}
		} else if ( returnNameNew != null ) { // varOld == null
			varNew = (VariableDeclaration) (new ModelCreationFactory(VariableDeclaration.class)).getNewObject();
			varNew.setName( returnNameNew );
			varNew.setType( type_new );
			method.setReturnVariable( varNew );
			constraint.getAllVariables().add( varNew );	
		}
		// update list of arguments (parameters)
		method.getArguments().clear();
		constraint.getAllVariables().removeAll(parameters_old);
		deletedParameters.addAll(parameters_old);
		for(int i=0; i<parameters_new.size(); i++){
			VariableDeclaration newParameter = (VariableDeclaration) parameters_new.get(i);
			if(changedParameters.containsKey(newParameter)){
				VariableDeclaration oldParameter = (VariableDeclaration) changedParameters.get(newParameter);
				String oldName = oldParameter.getName();
				String oldType = oldParameter.getType();
				oldParameter.setName(newParameter.getName());
				oldParameter.setType(newParameter.getType());
				newParameter.setName(oldName);
				newParameter.setType(oldType);
				method.getArguments().add(oldParameter);
				constraint.getAllVariables().add(oldParameter);
				deletedParameters.remove(oldParameter);
			}else{
				method.getArguments().add(newParameter);
				constraint.getAllVariables().add(newParameter);
				if(parameters_old.contains(newParameter))
					deletedParameters.remove(newParameter);
			}
		}
		List conditions = VOCLEditorPlugin.getAllConditions(constraint);
		for(int i=0; i<conditions.size(); i++){
			Condition condition = (Condition) conditions.get(i);
			if(condition.getLeft() != null && deletedParameters.contains(condition.getLeft())
					|| condition.getRight() != null && deletedParameters.contains(condition.getRight())){
				DeleteConditionCommand deleteConditionCommand = new DeleteConditionCommand();
				deleteConditionCommand.setCondition(condition);
				deleteConditionCommand.setParent((NavExp) condition.eContainer());
				deleteConditionCommand.execute();
				deleteConditionCommands.add(deleteConditionCommand);
			}				
		}
	}	
	
	/**
	 * Sets the method name.
	 * @param name the method name to set
	 */
	public void setName(String name) {
		this.name_new = name;
	}
	
	/**
	 * Sets the method return type.
	 * @param type the method return type to set
	 */
	public void setReturnType(String type){
		this.type_new = type;
	}
	
	/**
	 * Sets the return variable's name.
	 * @param returnVariableName the new name of the return variable
	 */
	public void setReturnVariableName( String returnVariableName ) {
		this.returnNameNew = returnVariableName;
	}
	
	/**
	 * Sets the parameter list.
	 * @param parameters the parameters list to set
	 */
	public void setParameters(List parameters){
		parameters_new = parameters;		
	}
	
	/**
	 * Sets the the hashtable of changed parameters.
	 * @param changedParameters the hashtable of changed parameters.
	 */
	public void setChangedParameters(Hashtable changedParameters){
		this.changedParameters = changedParameters;
	}
	
	/**
	 * Sets the command attributes.
	 * @param name the method
	 * @param returnType the return variable's type
	 * @param returnVarNameNew the new return variable's name
	 * @param changedParameters the list of changed parameters
	 * @param parameters the parameter list
	 */
	public void setCommandAttributes(String name, String returnType, String returnVarNameNew, Hashtable changedParameters,  List parameters) {
		setName(name);
		setReturnType(returnType);
		setReturnVariableName(returnVarNameNew);
		setChangedParameters(changedParameters);
		setParameters(parameters);		
	}
}
