/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.commands;

import org.eclipse.gef.commands.Command;
import org.eclipse.swt.widgets.Shell;


import VOCLModel.ClassifierRole;
import VOCLModel.AssociationRole;
import vocleditor.edit.VOCLAssociationRoleEditPart;



/**
 * Connects two ClassifierRoles via an AssociationRole.
 */
public class ReconnectAssociationRoleCommand extends Command {
	protected ClassifierRole source;
	protected ClassifierRole source_old;
	protected ClassifierRole target;
	protected ClassifierRole target_old;
	protected AssociationRole associationrole;
	protected VOCLAssociationRoleEditPart edPart;
	private Shell shell;

	/** 
	 * Remembers the anchor before moving
	 */
	int lastAnchorAt = -1;
	/** 
	 * Remembers the anchor after moving
	 */
	int anchorAt;
	/**
	 * indicates whether we're moving target or source
	 */
	boolean isMovingSource;

	/**
	 * Constructor for the command class.
	 *
	 */
	public ReconnectAssociationRoleCommand(AssociationRole associationrole, boolean isMovingSource) {
		setLabel("reconnect association role");
		this.associationrole = associationrole;
		this.isMovingSource = isMovingSource;
	}

	/**
	 * True if the command could be executed, false otherwise.
	 */
	public boolean canExecute() {
		if (source == null || target == null || associationrole == null || source == target)
			return false;
		else return true; 
	}

	/**
	 * Entry point for command execution.
	 */
	public void execute() {
		if (!source.getTarget().contains(associationrole)) {
			source_old.getTarget().remove(associationrole);
			source.getTarget().add(associationrole);
			associationrole.setAssociationSource(source);
		}
		if (!target.getSource().contains(associationrole)) {
			target_old.getSource().remove(associationrole);
			target.getSource().add(associationrole);
			associationrole.setAssociationTarget(target);
		}
		if(isMovingSource)
			anchorAt = associationrole.getSourceAnchorAt();
		else
			anchorAt = associationrole.getTargetAnchorAt();
		
		lastAnchorAt = edPart.lastAnchorAt;
		edPart.isMoving = false;

		// does nothing but fires changed for update, because the graphics won't be updated if AnchorPoint is changed else
		associationrole.setRoutingType(associationrole.getRoutingType());
		
	}
	
	public ClassifierRole getSource() {
		return source;
	}
	
	public AssociationRole getAssociationRole(){
		return associationrole;
	}
	
	public ClassifierRole getTarget() {
		return target;
	}

	/**
	 * Redo command execution.
	 */
	public void redo() {
		if(isMovingSource)
			associationrole.setSourceAnchorAt(anchorAt);
		else
			associationrole.setTargetAnchorAt(anchorAt);
		execute();
	}
	
	/**
	 * @return Returns the shell.
	 */
	public Shell getShell() {
		return shell;
	}
	/**
	 * @param shell The shell to set.
	 */
	public void setShell(Shell shell) {
		this.shell = shell;
	}

	public void setSource(ClassifierRole newSource) {
		source = newSource;
	}

	public void setTarget(ClassifierRole newTarget) {
		target = newTarget;
	}
	public void setAssociationRole(AssociationRole newArc){
		associationrole = newArc;
	}
	
	public void setSource_old(ClassifierRole oldSource) {
		source_old = oldSource;
	}
	
	public void setTarget_old(ClassifierRole oldTarget) {
		target_old = oldTarget;
	}
	
	public void setIsMovingSource(boolean isMovingSource) {
		this.isMovingSource = isMovingSource;
	}
	
	public void setEdPart(VOCLAssociationRoleEditPart edPart) {
		this.edPart = edPart;
	}
	
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		if (!source_old.getTarget().contains(associationrole)) {
			source.getTarget().remove(associationrole);
			source_old.getTarget().add(associationrole);
			associationrole.setAssociationSource(source_old);
		}
		if (!target_old.getSource().contains(associationrole)) {
			target.getSource().remove(associationrole);
			target_old.getSource().add(associationrole);
			associationrole.setAssociationTarget(target_old);
		}
		if(isMovingSource)
			associationrole.setSourceAnchorAt(lastAnchorAt);
		else
			associationrole.setTargetAnchorAt(lastAnchorAt);
	}
}
