<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:voclmodel="http://VOCLModel" 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xmlns:xlink="http://www.w3.org/1999/xlink" 
xmlns:str="http://exslt.org/strings"
xmlns:func="http://exslt.org/functions"
xmlns:exsl="http://exslt.org/common"
extension-element-prefixes="str func exsl"> 

<func:function name="str:tokenize">
	<xsl:param name="string" select="''" />
  <xsl:param name="delimiters" select="' &#x9;&#xA;'" />
  <xsl:choose>
    <xsl:when test="not($string)">
      <func:result select="/.." />
    </xsl:when>
    <xsl:when test="not(function-available('exsl:node-set'))">
      <xsl:message terminate="yes">
        ERROR: EXSLT - Functions implementation of str:tokenize relies on exsl:node-set().
      </xsl:message>
    </xsl:when>
    <xsl:otherwise>
      <xsl:variable name="tokens">
        <xsl:choose>
          <xsl:when test="not($delimiters)">
            <xsl:call-template name="str:_tokenize-characters">
              <xsl:with-param name="string" select="$string" />
            </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="str:_tokenize-delimiters">
              <xsl:with-param name="string" select="$string" />
              <xsl:with-param name="delimiters" select="$delimiters" />
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <func:result select="exsl:node-set($tokens)/token" />
    </xsl:otherwise>
  </xsl:choose>
</func:function>

<xsl:template name="str:_tokenize-characters">
  <xsl:param name="string" />
  <xsl:if test="$string">
    <token><xsl:value-of select="substring($string, 1, 1)" /></token>
    <xsl:call-template name="str:_tokenize-characters">
      <xsl:with-param name="string" select="substring($string, 2)" />
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<xsl:template name="str:_tokenize-delimiters">
  <xsl:param name="string" />
  <xsl:param name="delimiters" />
  <xsl:variable name="delimiter" select="substring($delimiters, 1, 1)" />
  <xsl:choose>
    <xsl:when test="not($delimiter)">
      <token><xsl:value-of select="$string" /></token>
    </xsl:when>
    <xsl:when test="contains($string, $delimiter)">
      <xsl:if test="not(starts-with($string, $delimiter))">
        <xsl:call-template name="str:_tokenize-delimiters">
          <xsl:with-param name="string" select="substring-before($string, $delimiter)" />
          <xsl:with-param name="delimiters" select="substring($delimiters, 2)" />
        </xsl:call-template>
      </xsl:if>
      <xsl:call-template name="str:_tokenize-delimiters">
        <xsl:with-param name="string" select="substring-after($string, $delimiter)" />
        <xsl:with-param name="delimiters" select="$delimiters" />
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="str:_tokenize-delimiters">
        <xsl:with-param name="string" select="$string" />
        <xsl:with-param name="delimiters" select="substring($delimiters, 2)" />
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:output method="xml" version="1.0" encoding="UTF-8" doctype-system="gxl.dtd" indent="yes" /> 


<xsl:template match="/voclmodel:VOCLDiagram"> 

<gxl xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:xalan="http://xml.apache.org/xalan" xmlns:lxslt="http://xml.apache.org/xslt">

<graph id="InstanceGraph">

  <!-- VOCLDiagram -->
  <node id="N{generate-id(.)}">
	<type xlink:href="VOCLDiagram%:RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:"/>
	<attr name="inLoop">
	  <boolean>false</boolean>
	</attr>
	<attr name="isNegated">
	  <boolean>false</boolean>
	</attr>
  </node>
  
  <!-- Constraint -->
  <xsl:for-each select="constraints" >  
    <node id="N{generate-id(.)}">
	<type xlink:href="Constraint%:RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:"/>
	<attr name="name">
	   <string><xsl:value-of select="@name"/></string>
	</attr>
	<attr name="kind">
        <string><xsl:value-of select="@kind"/></string>
	</attr>
	</node>
	
    <!-- Edge: VOCLDiagram -> Constraint -->
    <edge id="E{generate-id(.)}" from="N{generate-id(/voclmodel:VOCLDiagram)}" to="N{generate-id(.)}">
	<type xlink:href="constraintNode%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    </edge>
    
    <!-- AllVariables -->
    <xsl:for-each select="allVariables" >
       <node id="N{generate-id(.)}">
			      <type xlink:href="Variable%:RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:"/>
			      <attr name="name">
	  			 	<string><xsl:value-of select="@name" /></string>
				  </attr>
				  <attr name="type">
	  				<string><xsl:value-of select="@type" /></string>
			      </attr>
		</node>
		
		<!-- Edge: Constraint -> Variable -->
    	<edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
			<type xlink:href="variable%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    	</edge>	
    </xsl:for-each>
    
    <!-- Context -->
    <xsl:for-each select="context" >  
    <node id="N{generate-id(.)}">
	<type xlink:href="Context%:RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:"/>
	<attr name="name">
	   <string><xsl:value-of select="@name"/></string>
	</attr>
	<attr name="isClassifier">
	<xsl:choose>
          <xsl:when test="@isClassifier">
           <boolean><xsl:value-of select="@isClassifier"/></boolean>
          </xsl:when>
          <xsl:otherwise>
         <boolean>false</boolean>
          </xsl:otherwise>
        </xsl:choose>
    </attr>
	<attr name="type">
	  <string><xsl:value-of select="@type"/></string>
	</attr>
	</node>
	 
    <!-- Edge: Constraint -> Context -->
    <edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
	<type xlink:href="contextNode%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    </edge>	
	
    </xsl:for-each>
    
   	<!-- NavExp -->
   	 
   	<xsl:for-each select="expression">  
      		 
      	<!-- Edge:  Constraint -> NavExp -->
   		<edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
				<type xlink:href="navExpNode%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    	</edge> 
    			
    </xsl:for-each>
   	   
   	<xsl:apply-templates select="expression"/>
    
   
    
   
  </xsl:for-each>
   


</graph>

</gxl>
</xsl:template>

<xsl:template match="expression | op1 | op2 | ifExp | thenExp | elseExp |let | in">

     <!-- NavExp -->
	<node id="N{generate-id(.)}">
   	<attr name="inLoop">
	  <boolean>false</boolean>
	</attr>
	<attr name="isNegated">
	  <boolean>false</boolean>
	</attr>
   	<type xlink:href="NavExp%:RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:"/>
   	</node>    
    
     <!-- Condition -->
   	<xsl:for-each select="condition" >
   		<node id="N{generate-id(.)}">
   		<type xlink:href="Condition%:RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:"/>
   		<attr name="opKind">
			<string><xsl:value-of select="@op"/></string>
		</attr>
		<attr name="value">
			<string><xsl:value-of select="@value"/></string>
		</attr>
		<attr name="leftPre">
		<xsl:choose>
          	<xsl:when test="@leftPre">
          	<boolean><xsl:value-of select="@leftPre"/></boolean>
          	</xsl:when>
          	<xsl:otherwise>
          	<boolean>false</boolean>
          	</xsl:otherwise>
     	</xsl:choose>
		</attr>
		<attr name="rightPre">
		<xsl:choose>
          	<xsl:when test="@rightPre">
          	<boolean><xsl:value-of select="@rightPre"/></boolean>
          	</xsl:when>
          	<xsl:otherwise>
          	<boolean>false</boolean>
          	</xsl:otherwise>
     	</xsl:choose>
     	</attr>
   		</node>
    	
   		<!-- Edge:  NavExp -> Condition -->
   		<edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
		<type xlink:href="conditionNode%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    	</edge>
    
    	<xsl:if test="@left">
        
         <!-- Edge: Condition-> LeftVariable -->
    	
      			  <edge id="E1{generate-id(.)}" from="N{generate-id(.)}" to="N{generate-id(/voclmodel:VOCLDiagram/constraints[substring-before(substring-after(@left,'.'),'/')+1]/allVariables[substring-after(substring-after(@left,'.'),'.')+1])}">
	  			  <type xlink:href="conditionLeftVar%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
      		      </edge>
      		      			      
		</xsl:if>
		
		<xsl:if test="@right">
		
	     <!-- Edge: Condition -> RightVariable -->
	     
			      	     
      			  <edge id="E2{generate-id(.)}" from="N{generate-id(.)}" to="N{generate-id(/voclmodel:VOCLDiagram/constraints[substring-before(substring-after(@right,'.'),'/')+1]/allVariables[substring-after(substring-after(@right,'.'),'.')+1])}">
	  			  <type xlink:href="conditionRightVar%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
      		      </edge>
      		      			      
		</xsl:if>
    
    </xsl:for-each>
    
      	<!-- ClassifierRole/SetClassifierRole -->
  		<xsl:for-each select="classifierRoles" >  
    	<node id="N{generate-id(.)}">
    	<xsl:choose>
          <xsl:when test="not(@opKind)">
           <type xlink:href="ClassifierRole%:RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:"/>
          </xsl:when>
          <xsl:otherwise>
            <type xlink:href="SetClassifierRole%:RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:"/>
          </xsl:otherwise>
        </xsl:choose>
        
		<attr name="name">
	  	<string><xsl:value-of select="@name"/></string>
		</attr>
		<attr name="type">
	  	<string><xsl:value-of select="@type"/></string>
		</attr>
    	<xsl:if test="@opKind">
    	<attr name="opKind">
	  	<string><xsl:value-of select="@opKind"/></string>
		</attr>
    	</xsl:if>
    	</node>
    	
		<xsl:choose>
          <xsl:when test="not(@opKind)">
          <!-- Edge: NavExp -> ClassifierNode -->
          <edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
          <type xlink:href="classifierNode%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
          </edge>
          </xsl:when>
          <xsl:otherwise>
          <!-- Edge: NavExp -> SetClassifierNode -->
          <edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
          <type xlink:href="setClassifierNode%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
          </edge>
          <!-- Edge: SetClassifier -> Variable if opKind==size -->
      	  <xsl:if test="@opKind='size'">
			   
        	<edge id="E1{generate-id(.)}" from="N{generate-id(.)}" to="N{generate-id(/voclmodel:VOCLDiagram/constraints[substring-before(substring-after(@size,'.'),'/')+1]/allVariables[substring-after(substring-after(@size,'.'),'.')+1])}">
	  	  	     <type xlink:href="setClassifierVar%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
      	  	</edge> 
      	  	
      	  </xsl:if>
      	  <xsl:if test="@opKind='sum'">
			   
        	<edge id="E1{generate-id(.)}" from="N{generate-id(.)}" to="N{generate-id(/voclmodel:VOCLDiagram/constraints[substring-before(substring-after(@sum,'.'),'/')+1]/allVariables[substring-after(substring-after(@sum,'.'),'.')+1])}">
	  	  	     <type xlink:href="sumVar%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
      	  	</edge> 
      	  	
      	  	<edge id="E2{generate-id(.)}" from="N{generate-id(.)}" to="N{generate-id(/voclmodel:VOCLDiagram/constraints[substring-before(substring-after(@iterator,'.'),'/')+1]/allVariables[substring-after(substring-after(@iterator,'.'),'.')+1])}">
	  	  	     <type xlink:href="iteratorVar%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
      	  	</edge> 
      	  	
      	  </xsl:if>
          </xsl:otherwise>
        </xsl:choose>
        
        <xsl:if test="@context">
    	  <edge id="E3{generate-id(.)}" from="N{generate-id(/voclmodel:VOCLDiagram/constraints[substring-before(substring-after(@context,'.'),'/')+1]/context)}" to="N{generate-id(.)}">
	  			  <type xlink:href="contextStartClassifier%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
      	  </edge>	      	  			      
		</xsl:if>
		<xsl:if test="@variable">
	
        <edge id="E1{generate-id(.)}" from="N{generate-id(.)}" to="N{generate-id(/voclmodel:VOCLDiagram/constraints[substring-before(substring-after(@variable,'.'),'/')+1]/allVariables[substring-after(substring-after(@variable,'.'),'.')+1])}">
	  			  <type xlink:href="classifierVar%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/> 
         </edge>
      	 </xsl:if>
    	
    
    		<!-- Attribute -->
    		<xsl:for-each select="attributes" >  
      		<node id="N{generate-id(.)}">
			<type xlink:href="Attribute%:RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:"/>
			<attr name="type">
	  			<string><xsl:value-of select="@type"/></string>
			</attr>
			<attr name="value">
	  			<string><xsl:value-of select="@value"/></string>
			</attr>
			<attr name="name">
	  			<string><xsl:value-of select="@name"/></string>
			</attr>
			</node>
			
			<!-- Edge: ClassifierNode -> Attribute -->
      		<edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
	  		<type xlink:href="attribute%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
      		</edge>
			
			<xsl:if test="@variable">
	
      		  <edge id="E1{generate-id(.)}" from="N{generate-id(.)}" to="N{generate-id(/voclmodel:VOCLDiagram/constraints[substring-before(substring-after(@variable,'.'),'/')+1]/allVariables[substring-after(substring-after(@variable,'.'),'.')+1])}">
	  			  <type xlink:href="attributeVar%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
      		  </edge>
      		       			      
			</xsl:if>
    		</xsl:for-each>
    		
    		<!-- Method -->
    		<xsl:for-each select="method">
    		<node id="N{generate-id(.)}">
			<type xlink:href="Method%:RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:"/>
			<attr name="name">
	  			<string><xsl:value-of select="@name"/></string>
			</attr>  
			<attr name="returnType">
	  			<string><xsl:value-of select="@returnType"/></string>
			</attr>
			</node>
			
			
      		<!-- Edge: ClassifierNode -> Method -->
      		<edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
	  		<type xlink:href="method%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
      		</edge>
      		
      		
      		<xsl:if test="@returnVariable">
			      
			      <!-- Edge: Method -> ReturnVariable -->
			      
    		 <edge id="E1{generate-id(.)}" from="N{generate-id(.)}" to="N{generate-id(/voclmodel:VOCLDiagram/constraints[substring-before(substring-after(@returnVariable,'.'),'/')+1]/allVariables[substring-after(substring-after(@returnVariable,'.'),'.')+1])}">
	  			  <type xlink:href="returnVar%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
      		 </edge>
      		      			      
		   </xsl:if>
		   
		   
   	   	  <xsl:if test="@context">			   
      			  <edge id="E2{generate-id(.)}" from="N{generate-id(/voclmodel:VOCLDiagram/constraints[substring-before(substring-after(@context,'.'),'/')+1]/context)}" to="N{generate-id(.)}">
	  			  <type xlink:href="contextStartMethod%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
      		      </edge>	      			      
		  </xsl:if>
		   
		   <xsl:if test="@arguments">
		   
		   <xsl:variable name="method" select="."/>
		   <xsl:variable name="variables" select="/voclmodel:VOCLDiagram/constraints[substring-before(substring-after(@arguments,'.'),'/')+1]"/>
		   
		   <xsl:for-each select="str:tokenize(@arguments,' ')">  
		  		   
		     <!-- Edge: Method -> Parameter -->
      		  <edge id="E3{generate-id(.)}" from="N{generate-id($method)}" to="N{generate-id($variables/allVariables[substring-after(substring-after(.,'.'),'.')+1])}">
	  			  <type xlink:href="param%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/> 
      		  </edge>
      		  
      		  <xsl:if test="position()&gt;1">
      		  
      		  <xsl:variable name="pos" select="position()-1"/>
      		  <xsl:variable name="last" select="str:tokenize($method/@arguments,' ')[$pos]"/>
      		      
      		  <!-- Edge: Parameter -> NextParameter -->
      		  <edge id="E4{generate-id(.)}" from="N{generate-id($variables/allVariables[substring-after(substring-after($last,'.'),'.')+1])}" to="N{generate-id($variables/allVariables[substring-after(substring-after(.,'.'),'.')+1])}">
	  			  <type xlink:href="nextParam%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
      		  </edge> 
		   
		      </xsl:if>
		     
		      
		   </xsl:for-each>
		   </xsl:if>
      		
      		    		
    		</xsl:for-each>
    		
  		</xsl:for-each>

  		<!-- AssociationRole -->
  		<xsl:for-each select="associationRoles" >  
    	<node id="N{generate-id(.)}">
		<type xlink:href="AssociationRole%:RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:"/>
		<attr name="name">
	  		<string><xsl:value-of select="@name"/></string>
		</attr>
    	</node>
    	
    	<!-- Edge: NavExp -> AssociationNode -->
    	<edge id="E1{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
		<type xlink:href="associationNode%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    	</edge>
    	
    	<xsl:variable name="source">
    	<xsl:call-template name="lastPartofString">
          <xsl:with-param name="theString" select="@AssociationSource" />
        </xsl:call-template>
       </xsl:variable>
       
       <xsl:variable name="target">
    	<xsl:call-template name="lastPartofString">
          <xsl:with-param name="theString" select="@AssociationTarget" />
        </xsl:call-template>
       </xsl:variable>
       
       
    	
    	<!-- Edge: AssociationNode -> source ClassifierNode -->
    
    	<edge id="E2{generate-id(.)}" from="N{generate-id(.)}" to="N{generate-id(../classifierRoles[$source+1])}">
		<type xlink:href="source%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    	</edge>
    
    	<!-- Edge: AssociationNode -> target ClassifierNode -->
    	
    	<edge id="E3{generate-id(.)}" from="N{generate-id(.)}" to="N{generate-id(../classifierRoles[$target+1])}">
		<type xlink:href="target%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    	</edge>
 
 		</xsl:for-each>	
 		
 		<!-- Expressions -->
   	<xsl:for-each select="expressions" > 
       	<xsl:choose>
          	<xsl:when test="@xsi:type='VOCLModel:BoolExp'">
      			<node id="N{generate-id(.)}">
      			<type xlink:href="BoolExp%:RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:"/>
      			<attr name="kind">
      			<xsl:choose>
          			<xsl:when test="@kind">
          				<string><xsl:value-of select="@kind"/></string>
        			</xsl:when>
          			<xsl:otherwise>
          				<string>or</string>
          			</xsl:otherwise>
     			</xsl:choose>
     			</attr>
     			
     			<attr name="negated">
     			<xsl:choose>
     	   			 <xsl:when test="@negated">
          			<boolean><xsl:value-of select="@negated"/></boolean>
          			</xsl:when>
          			<xsl:otherwise>
          			<boolean>false</boolean>
          			</xsl:otherwise>
          		</xsl:choose>
          		</attr>
          		</node>
				
			 </xsl:when>
			 <xsl:when test="@xsi:type='VOCLModel:IfExp'">
      			<node id="N{generate-id(.)}">
      			<type xlink:href="ifExp%:RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:"/>
      			<attr name="negated">
      			<xsl:choose>
     	   			 <xsl:when test="@negated">
          			<boolean><xsl:value-of select="@negated"/></boolean>
          			</xsl:when>
          			<xsl:otherwise>
          			<boolean>false</boolean>
          			</xsl:otherwise>
          		</xsl:choose>
          		</attr>
          		</node>
      			
				
			 </xsl:when>
          	 <xsl:when test="@xsi:type='VOCLModel:LetExp'">
      			<node id="N{generate-id(.)}">
      			<type xlink:href="LetExp%:RECT:java.awt.Color[r=0,g=0,b=0]::[NODE]:"/>
      			 <attr name="negated">
      			 <xsl:choose>
     	   			 <xsl:when test="@negated">
          			<boolean><xsl:value-of select="@negated"/></boolean>
          			</xsl:when>
          			<xsl:otherwise>
          			<boolean>false</boolean>
          			</xsl:otherwise>
          		</xsl:choose>
          		</attr>
          		</node>
      			
							    
			 </xsl:when>
			 
     	</xsl:choose>
     	   
			
			<!-- Edge: NavExp -> SubExp -->
      		<edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
	  		<type xlink:href="subExp%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
      		</edge>
      		
      			<xsl:for-each select="op1">  
      		 
      		 	<!-- Edge:  BoolExp -> op1NavExp -->
   				<edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
					<type xlink:href="op1%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    			</edge> 
    			
    			</xsl:for-each>
		    	<xsl:apply-templates select="op1"/>
		    	
		    	
      			<xsl:for-each select="op2">  
      		 
      		 	<!-- Edge:  BoolExp -> op2NavEXp -->
   				<edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
					<type xlink:href="op2%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    			</edge> 
    			
    			</xsl:for-each>
		    	<xsl:apply-templates select="op2"/>
		    	
		    	<xsl:for-each select="ifExp">  
      		 
      		 	<!-- Edge:  IfExp -> ifNavExp -->
   				<edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
					<type xlink:href="if%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    			</edge> 
    			
    			</xsl:for-each>
		    	<xsl:apply-templates select="ifExp"/>
		    	
		    	<xsl:for-each select="thenExp">  
      		 
      		 	<!-- Edge:  IfExp -> thenNavExp -->
   				<edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
					<type xlink:href="then%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    			</edge> 
    			
    			</xsl:for-each>
		    	<xsl:apply-templates select="thenExp"/>
		    	
		    	<xsl:for-each select="elseExp">  
      		 
      		 	<!-- Edge:  IfExp -> ifNavExp -->
   				<edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
					<type xlink:href="else%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    			</edge> 
    			
    			</xsl:for-each>
		    	<xsl:apply-templates select="elseExp"/>
		    	
		    	<xsl:for-each select="let">
		    	
		    	<!-- Edge:  LetExp -> letNavExp -->
   				<edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
					<type xlink:href="let%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    			</edge>
    			   			
    			</xsl:for-each>
    			
    			<xsl:if test="@letVar">
				     <edge id="E1{generate-id(.)}" from="N{generate-id(.)}" to="N{generate-id(/voclmodel:VOCLDiagram/constraints[substring-before(substring-after(@letVar,'.'),'/')+1]/allVariables[substring-after(substring-after(@letVar,'.'),'.')+1])}">
	  					<type xlink:href="letVar%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
      				</edge>
      			</xsl:if>
      			
		    	<xsl:apply-templates select="let"/>
		    	
		    	<xsl:for-each select="in">
		    	
		    	<!-- Edge:  LetExp -> inNavExp -->
   				<edge id="E{generate-id(.)}" from="N{generate-id(..)}" to="N{generate-id(.)}">
					<type xlink:href="in%:SOLID_LINE:java.awt.Color[r=0,g=0,b=0]::[EDGE]:"/>
    			</edge> 
    			
    			</xsl:for-each>
		    	<xsl:apply-templates select="in"/>
	        			
    	</xsl:for-each>
 		
 		  	
</xsl:template>
<xsl:template name="lastPartofString">
    <xsl:param name="theString" />
    <xsl:variable name="theRest" select="substring-after($theString,'.')" />
    <xsl:choose>
    	<xsl:when test="$theRest">
    	 <xsl:call-template name="lastPartofString">
          <xsl:with-param name="theString" select="$theRest" />
         </xsl:call-template>
    	</xsl:when>
    	<xsl:otherwise>
    	<xsl:value-of select="$theString"/>
        </xsl:otherwise>
    </xsl:choose>
</xsl:template>

    


</xsl:stylesheet>



