/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.dialog;

import java.util.List;
import java.util.Vector;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import vocleditor.VOCLEditorPlugin;

import VOCLModel.Attribute;
import VOCLModel.ClassifierRole;
import VOCLModel.Constraint;
import VOCLModel.VariableDeclaration;

/**
 * @author bjoern, stefan
 * JFace based dialog used to edit Attribute properties 
 */
public class AttributePropertiesDialog extends TitleAreaDialog implements SelectionListener, ModifyListener{
	private Text nameText;
	private Combo typeCombo;;
	private Text valueText;
	private Text variableText;
	private String name;
	private String type;
	private String value;
	private String variable;
	private Button valueCheck;
	private boolean valueBoolean = false;
	private Button variableCheck;
	private boolean variableBoolean = false;
	private Attribute attribute;
	private Composite composite;
	private Composite superComposite;
	private Constraint constraint;
	private ClassifierRole classifierRole;
	private List newVariableNames = new Vector();
	private List newNames = new Vector();
	private List newTypes = new Vector();
	
	/**
	 * Constructs a new AttributePropertiesDialog
	 * @param parentShell   parent shell (becomes blocked when this dialog is open)
	 * @param constraint the constraint
	 * @param classifierRole the classifier role
	 * @param attribute the Attribute to be edited
	 */
	public AttributePropertiesDialog(Shell parentShell, Constraint constraint, ClassifierRole classifierRole, Attribute attribute)
	{
		super(parentShell);
		setShellStyle( getShellStyle() | SWT.RESIZE );
		this.constraint = constraint;
		this.classifierRole = classifierRole;
		this.attribute = attribute;
		
		name  = attribute.getName()  != null ? attribute.getName()  : ""; 
		type  = attribute.getType()  != null ? attribute.getType()  : "";
		value = attribute.getValue() != null ? attribute.getValue() : "";
		variable = attribute.getVariable() != null ? attribute.getVariable().getName() : "";
	}
	
	/**
	 * Constructs a new AttributePropertiesDialog
	 * @param parentShell   parent shell (becomes blocked when this dialog is open)
	 * @param constraint the constraint
	 * @param classifierRole the classifier role
	 * @param attribute the Attribute to be edited
	 * @param newVariableNames the list of new variable names
	 * @param newTypes the list of new types.
	 */
	public AttributePropertiesDialog(Shell parentShell, Constraint constraint, ClassifierRole classifierRole, Attribute attribute, List newVariableNames, List newNames, List newTypes)
	{
		super(parentShell);
		setShellStyle( getShellStyle() | SWT.RESIZE );
		this.constraint = constraint;
		this.classifierRole = classifierRole;
		this.attribute = attribute;
		
		this.newVariableNames = newVariableNames;
		this.newNames = newNames;
		this.newTypes = newTypes;
		
		name  = attribute.getName()  != null ? attribute.getName()  : ""; 
		type  = attribute.getType()  != null ? attribute.getType()  : "";
		value = attribute.getValue() != null ? attribute.getValue() : "";
		variable = attribute.getVariable() != null ? attribute.getVariable().getName() : "";	
	}

	/**
	 * Helper function, adding a Button with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @return the button that was added to composite
	 */
	Button addButtonToComposite( Composite composite) 
	{
		Button button = new Button( composite, SWT.CHECK );
		button.addSelectionListener( this );
		GridData gd = new GridData(); 
		gd.widthHint = 20;
		button.setLayoutData( gd ); 
		return button;
	}

	/**
	 *  
	 * Helper function, adding a Label with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 * 
	 * @author bjoern
	 */
	Label addLabelToComposite( Composite composite, String text , int length) 
	{
		Label label = new Label( composite, 0 );
		label.setText( text );
		GridData gd = new GridData(); 
		gd.widthHint = length;
		label.setLayoutData( gd );
		return label;
	}

	/**
	 * @author bjoern
	 * 
	 * Helper function, adding a Text (-field) with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Text addTextToComposite( Composite composite, String textValue, ModifyListener textListener ) 
	{
		Text text = new Text( composite, SWT.SINGLE | SWT.BORDER );
		text.setText( textValue );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		text.setLayoutData( gd );
		if ( textListener != null )
			text.addModifyListener( textListener );
		return text;
	}
	
	/**
	 * Helper function, adding a Combo with GridLayout-Data to a Composite
	 * @param composite The composite to add the label to
	 * @param textValue The text of the Text
	 * @return the combobox that was added to composite
	 */
	Combo addComboToComposite(Composite composite, String textValue) {
    	Combo com=new Combo(composite,SWT.DROP_DOWN | SWT.SIMPLE);
    	com.setText(textValue);
    	GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		com.setLayoutData( gd );
		com.addModifyListener( this );		
		return com;
    }
	
	
	/** 
	 * Creates the content of the dialog.
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{
		superComposite = (Composite) super.createDialogArea(parent);
		composite = new Composite( superComposite, 0 );
		composite.setLayoutData( new GridData( GridData.FILL_HORIZONTAL ) );
		
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		composite.setLayout( layout );
		
		addLabelToComposite( composite, "Name:", 100);
		addLabelToComposite( composite, "" , 20);
		nameText  = addTextToComposite( composite, name, this );
		addLabelToComposite( composite, "Type:", 100);
		addLabelToComposite( composite, "" , 20);
		typeCombo  = addComboToComposite( composite, type);
		List allTypes = VOCLEditorPlugin.getAllTypes(constraint);
		for(int i=0; i<allTypes.size(); i++){
			typeCombo.add((String) allTypes.get(i));
		}
		for(int i=0; i<newTypes.size(); i++){
			if(typeCombo.indexOf((String) newTypes.get(i)) == -1)
				typeCombo.add((String) newTypes.get(i));
		}
		
		addLabelToComposite( composite, "Value:", 100);
		valueCheck = addButtonToComposite(composite);		
		valueText = addTextToComposite( composite, value, this );
		
		addLabelToComposite( composite, "Varname:", 100);
		variableCheck = addButtonToComposite(composite);		
		variableText = addTextToComposite( composite, variable, this );
		
		if(variable != null && !"".equals(variable)){
			variableCheck.setSelection(true);
			variableBoolean = true;
			valueText.setEditable(false);
		} else {
			valueCheck.setSelection(true);
			valueBoolean = true;
			variableText.setEditable(false);			
		}
		
		return superComposite;
	}

	/**
	 * Implementation of ModifyListener.modifyText() used to show error messages and
	 * Disable ok button on incomplete input. 
	 */
	public void modifyText(ModifyEvent e)
	{
		if ( nameText.getCharCount() == 0 || typeCombo.getText().length() == 0) {
			setErrorMessage( "You need to specify a name and a type value!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}else if ( nameText.getCharCount() >0 && existsAttributeName(nameText.getText()) && !nameText.getText().equals(attribute.getName())) {
			setErrorMessage( "The attribute " + nameText.getText() + " is allready exists!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}else if ( nameText.getCharCount() >0 && existsWhiteSpace(nameText.getText())) {
			setErrorMessage( "Whitespace is not allowed in attribute name!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}else if(valueCheck.getSelection() && valueText.getCharCount() == 0){
			setErrorMessage( "You need to specify a value!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		} else if(variableCheck.getSelection() && variableText.getCharCount() == 0){
			setErrorMessage( "You need to specify a variable name!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		} 
		/*
		else if(variableCheck.getSelection() && variableText.getCharCount() > 0 && existsVariableName(variableText.getText()) && (attribute.getVariable() == null || !variableText.getText().equals(attribute.getVariable().getName()))){
			setErrorMessage( "The variable " + variableText.getText() + " is allready exists!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}
		*/
		else {
			setErrorMessage( null );
			setMessage( "Edit the properties of your Attribute." );
			getButton(IDialogConstants.OK_ID).setEnabled( true );
		}
		name  = nameText.getText();
		type  = typeCombo.getText();
		value = valueText.getText();
		variable = variableText.getText();
	}
	
	private boolean existsVariableName(String variableName){
		List allVariables = constraint.getAllVariables();
		for(int i=0; i<allVariables.size(); i++){
			if(variableName.equals(((VariableDeclaration) allVariables.get(i)).getName()))
				return true;
		}
		for(int i=0; i<newVariableNames.size(); i++){
			if(variableName.equals(newVariableNames.get(i)))
				return true;
		}
		return false;
	}
	
	private boolean existsAttributeName(String attributeName){
		List attributes = classifierRole.getAttributes();
		for(int i=0; i<attributes.size(); i++){
			if(attributeName.equals(((Attribute) attributes.get(i)).getName()))
				return true;
		}
		for(int i=0; i<newNames.size(); i++){
			if(attributeName.equals(newNames.get(i)))
				return true;
		}
		return false;
	}
	
	private boolean existsWhiteSpace(String name){		
		String att = name;
		for(int j=0; j<att.length(); j++){
		 char c = att.charAt(j);
		 if(Character.isWhitespace(c))
			return true;
		}		
		return false;
	}
	
	/**
	 * Creates contents.
	 * @see org.eclipse.jface.window.Window#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		Control superContent = super.createContents(parent);
		
		parent.getShell().setMinimumSize( 400, 300 );
		parent.getShell().setText( "Attribute Properties" );
		nameText.setText( nameText.getText() );  // to call listeners
		
		return superContent;
	}
	
	/**
	 * Cancel pressed.
	 * @see org.eclipse.jface.dialogs.Dialog#cancelPressed()
	 */
	protected void cancelPressed() {
		super.cancelPressed();
	}
	
	/**
	 * Ok pressed.
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed() {
		super.okPressed();
	}
	
	/**
	 * Returns the attribute name.
	 * @return the attribute name
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Returns the attribute type.
	 * @return the attribute type
	 */
	public String getType() {
		return type;
	}
	
	/**
	 * Returns the attribute value.
	 * @return the attribute value
	 */
	public String getValue() {
		if(valueBoolean)
			return value;
		return null;
	}
	
	/**
	 * Returns the name of the attribute variable.
	 * @return the name of the attribute variable
	 */
	public String getVariableName(){
		if(variableBoolean)
			return variable;
		return null;
	}
	
	/**
	 * Widged selected listener.
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent event) {
		if ( event.getSource() == valueCheck) {
			variableCheck.setSelection(false);
			valueCheck.setSelection(true);
			valueBoolean = true;
			variableBoolean = false;
			variableText.setText("");
			variableText.setEditable(false);
			valueText.setEditable(true);
		} else if(event.getSource() == variableCheck) {
			variableCheck.setSelection(true);
			valueCheck.setSelection(false);
			variableBoolean = true;
			valueBoolean = false;			
			valueText.setText("");
			valueText.setEditable(false);
			variableText.setEditable(true);
		}
		
		nameText.setText(nameText.getText());
	}
	
	/**
	 * Widged default selected listener.
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected(SelectionEvent event) {
		widgetSelected(event);
	}
}
