/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.dialog;

import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Slider;

import VOCLModel.BoolExp;
import VOCLModel.BoolOperation;

/**
 * Dialog to modify Bool Exp properties
 * @author bjoern
 */
public class BoolExpPropertiesDialog 
	extends TitleAreaDialog
	implements SelectionListener, ModifyListener
{
	BoolExp    			 boolexp;
	String     			 operation;
	boolean    			 isHorizontal, wasHorizontal;
	boolean				 negated;
	int					 splitPercent;
	int					 oldSplitPercent;
	
	Combo      opCombo;
	Button     horizontalButton, negateButton; 
	Slider     splitSlider;
	Label		splitLabel;
	
	/**
	 * Constructs a BoolExpPropertiesDialog
	 * @param parentShell parent shell
	 * @param boolexp the BoolExp to be shown/modified
	 */
	public BoolExpPropertiesDialog(Shell parentShell, BoolExp boolexp )
	{   
		super(parentShell);
		this.boolexp = boolexp;
		this.operation = boolexp.getKind().getName();
		isHorizontal = boolexp.isSplitHorizontal();
		wasHorizontal = isHorizontal;
		splitPercent = boolexp.getSplitPercent();
		oldSplitPercent = splitPercent;
		negated = boolexp.isNegated();
		setShellStyle( getShellStyle() | SWT.RESIZE );
	}
		
	/**
	 * 
	 * Helper function, adding a Label with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Label addLabelToComposite( Composite composite, String text ) 
	{
		Label label = new Label( composite, 0 );
		label.setText( text );
		GridData gd = new GridData(); gd.widthHint = 150;
		label.setLayoutData( gd );
		return label;
	}

	/**
	 * 
	 * Helper function, adding a Slider with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Slider addSliderToComposite( Composite composite, int value, int maximum, int minimum ) 
	{
		Slider slider = new Slider( composite, SWT.HORIZONTAL );
		slider.setValues( value, minimum, maximum, 10, 1, 10 );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); 
		gd.widthHint = 150;
		slider.setLayoutData( gd );
		slider.addSelectionListener( this );
		return slider;
	}	
	
	/**
	 * 
	 * Helper function, adding a Button with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the button label
	 * @return the button that was added to composite
	 */
	Button addButtonToComposite( Composite composite, String text ) 
	{
		Button button = new Button( composite, SWT.CHECK );
		button.setText( text );
		button.addSelectionListener( this );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); 
		gd.widthHint = 90;
		button.setLayoutData( gd ); 
		return button;
	}
	
	/**
	 * 
	 * Helper function, adding a ComboBox with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the button label
	 * @return the button that was added to composite
	 */	
    Combo addComboToComposite( Composite composite, String textValue ) {
    	Combo com = new Combo(composite,SWT.DROP_DOWN | SWT.SIMPLE | SWT.READ_ONLY);
    	com.setText( textValue );
    	GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
    	com.setLayoutData( gd );
    	com.addModifyListener( this );
		return com;
    }

    /** 
	 * creates the content of the dialog
	 *  
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{ 
		Composite superComposite = (Composite) super.createDialogArea(parent);
		
		Composite composite = new Composite( superComposite , 0 );
		composite.setLayoutData( new GridData( GridData.FILL_BOTH ) );

		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout( layout );
		
		addLabelToComposite( composite, "Operation:" );
		opCombo = addComboToComposite( composite, operation ); 
		opCombo.add( "or" );
		opCombo.add( "xor" );
		opCombo.add( "implies" );
		opCombo.select( opCombo.indexOf( operation ) );

		addLabelToComposite( composite, "Horizontal:" );
		horizontalButton = addButtonToComposite( composite, "split expressions horizontally");
		horizontalButton.setSelection( isHorizontal );
		setMessage( "Edit the properties of your boolean expression." );

		splitLabel = addLabelToComposite( composite, "Split at: " + splitPercent + "%");
		splitSlider = addSliderToComposite( composite, splitPercent, 90, 20 );

		addLabelToComposite( composite, "Negated:" );
		negateButton = addButtonToComposite( composite, "negate the whole expressions");
		negateButton.setSelection( negated );
		
		return composite;
	}

	/**
	 * Creates contents.
	 * @see org.eclipse.jface.window.Window#createContents(org.eclipse.swt.widgets.Composite)
	 */	 
	protected Control createContents(Composite parent) 
	{	
		Control superContent = super.createContents(parent);
		
		parent.getShell().setSize( 550, 300 );
		parent.getShell().setMinimumSize( 500, 300 );
		parent.getShell().setText( "Boolean Expression's Properties" );
		return superContent;
	}
	
	/**
	 * Cancel pressed.
	 * @see org.eclipse.jface.dialogs.Dialog#cancelPressed()
	 */
	protected void cancelPressed() {
		boolexp.setSplitPercent( oldSplitPercent );
		boolexp.setSplitHorizontal( wasHorizontal );
		isHorizontal = wasHorizontal;
		splitPercent = oldSplitPercent;
		super.cancelPressed();
	}
	
	/**
	 * Returns the context type selected in this dialog ("inv", "pre" or "post").
	 * @return the context type selected in this dialog ("inv", "pre" or "post")
	 */
	public boolean getIsHorizontal() {
		return isHorizontal;
	}

	/**
	 * Returns the bool operation.
	 * @return the bool operation
	 */
	public BoolOperation getOperation() {
		return BoolOperation.get( operation );
	}
	
	/**
	 * Returns the split percentage.
	 * @return the split percentage
	 */
	public int getSplitPercentage() {
		return splitPercent;
	}
	
	/**
	 * Returns the negation flag.
	 * @return the negation flag
	 */
	public boolean isNegated() {
		return negated;
	}
	
	/**
	 * listens and reacts to "Add", "Edit" and "Delete" buttons
	 */
	public void widgetSelected(SelectionEvent e) 
	{
		if ( e.getSource() == horizontalButton ) {
			isHorizontal = horizontalButton.getSelection();
			boolexp.setSplitHorizontal( isHorizontal ); // "the preview effect" (tm)
		} else if ( e.getSource() == negateButton ) {
			negated = negateButton.getSelection();
		} else if ( e.getSource() == splitSlider ) {
			splitPercent = splitSlider.getSelection();
			splitLabel.setText( "Split at: " + splitPercent + "%" );
			boolexp.setSplitPercent( splitPercent );  // "the preview effect" (tm)
		}
	}
	
	/**
	 * Widged default selected listener.
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected( SelectionEvent e ) 
	{
		widgetSelected( e );
	}

	/**
	 * Modifies text.
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		operation = opCombo.getText();		
	}
}
