/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.dialog;

import java.util.Hashtable;
import java.util.List;
import java.util.ArrayList;
import java.util.Vector;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

import vocleditor.VOCLEditorPlugin;
import vocleditor.model.ModelCreationFactory;

import VOCLModel.Attribute;
import VOCLModel.ClassifierRole;
import VOCLModel.Constraint;
import VOCLModel.VariableDeclaration;

/**
 * @author bjoern, Stefan
 * Dialog to modify ClassifierRole properties
 */
public class ClassifierPropertiesDialog 
	extends TitleAreaDialog
	implements SelectionListener, ModifyListener
{
	Text nameText;
	Combo typeCombo;
	String name;
	String type;
	ClassifierRole classifierRole;
	Table table;
	Button addAttribButton, editAttribButton, delAttribButton;
	Constraint constraint;
	private List attributeList=new ArrayList();
	private Hashtable changedVariables = new Hashtable();

	/**
	 * Constructs a ClassifierPropertiesDialog
	 * @param parentShell the parent shell
	 * @param constraint the constraint
	 * @param classifierRole the ClassifierRole to be shown/modified
	 */
	public ClassifierPropertiesDialog(Shell parentShell, Constraint constraint, ClassifierRole classifierRole)
	{   
		super(parentShell);
		this.classifierRole = classifierRole;
		this.constraint = constraint;
		attributeList=getInitialAttributes(classifierRole);
		setShellStyle( getShellStyle() | SWT.RESIZE );
		name = classifierRole.getName() != null ? classifierRole.getName() : ""; 
		type = classifierRole.getType() != null ? classifierRole.getType() : "";
	}

		
	/**
	 * 
	 * Helper function, adding a Label with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Label addLabelToComposite( Composite composite, String text ) 
	{
		Label label = new Label( composite, 0 );
		label.setText( text );
		GridData gd = new GridData(); gd.widthHint = 100;
		label.setLayoutData( gd );
		return label;
	}

	/**
	 * 
	 * Helper function, adding a Text (-field) with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param textValue The text of the Text
	 * @return the text that was added to composite
	 */
	Text addTextToComposite( Composite composite, String textValue ) 
	{
		Text text = new Text( composite, SWT.SINGLE | SWT.BORDER );
		text.setText( textValue );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		text.setLayoutData( gd );
		text.addModifyListener( this );
		return text;
	}

	/**
	 * 
	 * Helper function, adding a Button with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the button label
	 * @return the button that was added to composite
	 */
	Button addButtonToComposite( Composite composite, String text ) 
	{
		Button button = new Button( composite, SWT.PUSH );
		button.setText( text );
		button.addSelectionListener( this );
		GridData gd = new GridData(); 
		gd.widthHint = 90;
		button.setLayoutData( gd ); 
		return button;
	}
	
	/**
	 * Helper function, adding a Combo with GridLayout-Data to a Composite
	 * @param composite The composite to add the label to
	 * @param textValue The text of the Text
	 * @return the combobox that was added to composite
	 */
    Combo addComboToComposite(Composite composite, String textValue) {
    	Combo com=new Combo(composite,SWT.DROP_DOWN | SWT.SIMPLE);
    	com.setText(textValue);
    	GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		com.setLayoutData( gd );
		com.addModifyListener( this );
		return com;
    }

	/**
	 * implementation of ModifyListener.modifyText() used to show error messages and
	 * disable ok button on incomplete input 
	 */
	public void modifyText(ModifyEvent e)
	{		
	     if ( typeCombo.getText().length() == 0 ) {
			setErrorMessage( "You must at least specify the type of your Classifier Role!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}else 	if ( nameText.getCharCount() >0 && existsWhiteSpace(nameText.getText())) {
			setErrorMessage( "Whitespace is not allowed in classifierrole name!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}else  {
			setErrorMessage( null );
			setMessage( "Edit the properties of your Classifier Role." );
			getButton(IDialogConstants.OK_ID).setEnabled( true );
		}
		name = nameText.getText();;
		type = typeCombo.getText();;
	}
	
	private boolean existsWhiteSpace(String name){		
		String att = name;
		for(int j=0; j<att.length(); j++){
		 char c = att.charAt(j);
		 if(Character.isWhitespace(c))
			return true;
		}		
		return false;
	}
	
	/** 
	 * Creates the content of the dialog.  
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{  
		Composite superComposite = (Composite) super.createDialogArea(parent);
		
		Composite composite = new Composite( superComposite , 0 );
		composite.setLayoutData( new GridData( GridData.FILL_BOTH ) );

		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout( layout );
		
		addLabelToComposite( composite, "Name:" );
		nameText = addTextToComposite( composite, name ); 
		addLabelToComposite( composite, "Type:" );
		typeCombo = addComboToComposite( composite, type);
		List classifierRoleTypes = VOCLEditorPlugin.getAllClassifierRoleTypes(constraint);
		for(int i=0; i<classifierRoleTypes.size(); i++){
			typeCombo.add((String) classifierRoleTypes.get(i));
		}
	 
		Group attributeWidgets = new Group( composite, 0 );
		attributeWidgets.setText( "Attributes" );		
		attributeWidgets.setLayoutData( new GridData( SWT.FILL, SWT.FILL, true, true, 2, 1 ) );

		layout = new GridLayout();
		layout.numColumns = 2;
		attributeWidgets.setLayout( layout );

		table = new Table( attributeWidgets, SWT.BORDER );
		table.setHeaderVisible( true );
		table.setLayoutData( new GridData( SWT.FILL, SWT.FILL, true, true, 1, 4 ) );
		
		TableColumn column = new TableColumn( table, 0 );
		column.setText( "Name" );
		column.setWidth( 100 );
		column = new TableColumn( table, 0 );
		column.setText( "Type" );
		column.setWidth( 100 );
		column = new TableColumn( table, 0 );
		column.setText( "Value/Variable" );
		column.setWidth( 100 );
				
		addAttribButton = addButtonToComposite( attributeWidgets, "Add" ); 
		editAttribButton = addButtonToComposite( attributeWidgets, "Edit" );
		delAttribButton = addButtonToComposite( attributeWidgets, "Delete" );

		fillAttributeTable();
		
		return composite;
	}

	/**
	 * @see org.eclipse.jface.window.Window#createContents(org.eclipse.swt.widgets.Composite)
	 */	 
	protected Control createContents(Composite parent) {		
		Control superContent = super.createContents(parent);		
		parent.getShell().setSize( 550, 550 );
		parent.getShell().setMinimumSize( 400, 400 );
		parent.getShell().setText( "Classifier Role Properties" );
		nameText.setText( nameText.getText() );  // to call listeners
		
		return superContent;
	}
	
	/**
	 * @see org.eclipse.jface.dialogs.Dialog#cancelPressed()
	 */
	protected void cancelPressed() {
		super.cancelPressed();
	}
	
	/**
	 * Stores edited classifier role properties after user clicked the "ok"-button.
	 */
	protected void okPressed() {
		super.okPressed();
	}
	
	/**
	 * Returns the classifier role name.
	 * @return the classifier role name
	 */
	public String getClassifierName() {
		return name;
	}
	
	/**
	 * Returns the classifier role type.
	 * @return the classifier role type
	 */
	public String getClassifierType() {
		return type;
	}
	
	/**
	 * Returns the hashtable of changed variables.
	 * @return the hashtable of changed variables
	 */
	public Hashtable getChangedVariables(){
		return changedVariables;
	}
	
	/**
	 * Returns the list of attributes.
	 * @return the list of attributes
	 */
	public List getAttributes() {
		return attributeList;
	}
	
	/**
	 * listens and reacts to "Add", "Edit" and "Delete" buttons
	 */
	public void widgetSelected(SelectionEvent e) 
	{
		if ( e.getSource() == addAttribButton )  {
			addAttribute();
		    fillAttributeTable();
		}
		if ( e.getSource() == editAttribButton ) { 
			TableItem tis[] = table.getSelection();
			if ( tis.length > 0 )
				editAttribute((Attribute) tis[0].getData());
		}
		if ( e.getSource() == delAttribButton ) { 
			TableItem tis[] = table.getSelection();
			if ( tis.length > 0 ) {
				for ( int i = 0; i < tis.length; i++ ) { 
					Attribute attr=(Attribute) tis[0].getData();
					attributeList.remove(attr);
					if(attr.getVariable() != null && changedVariables.containsKey(attr.getVariable()))
						changedVariables.remove(attr.getVariable());
		            fillAttributeTable();
				}
			}
		}
	}

	public void widgetDefaultSelected( SelectionEvent e ) 
	{
		widgetSelected( e );
	}
	
	private Attribute createAttribute() {		
		return (Attribute) (new ModelCreationFactory(Attribute.class)).getNewObject();		
	}
	
	private VariableDeclaration createVariableDeclaration() {		
		return (VariableDeclaration) (new ModelCreationFactory(VariableDeclaration.class)).getNewObject();		
	}

	// called after "Add" (add attribute to class.role) button 
	private void addAttribute() {
		Attribute attr = createAttribute();
		AttributePropertiesDialog apd = new AttributePropertiesDialog( getShell(), constraint, classifierRole, attr, getNewVariableNames(), getNewAttributeNames(), getNewAttributeTypes());
		apd.open();
		if ( apd.getReturnCode() == Dialog.OK ) {
			attr.setName(apd.getName());
			attr.setType(apd.getType());
			attr.setValue(apd.getValue());
			String varName = apd.getVariableName();
			if(varName != null && !"".equals(varName)){
				VariableDeclaration varDecl = createVariableDeclaration();
				varDecl.setName(varName);
				varDecl.setType(attr.getType());
				attr.setVariable(varDecl);
			}
			attributeList.add(attr);
		}
	}
	
	private void editAttribute( Attribute attr ){
		AttributePropertiesDialog apd = new AttributePropertiesDialog( getShell(), constraint, classifierRole, attr, getNewVariableNames(), getNewAttributeNames(), getNewAttributeTypes());
		apd.open();
		if ( apd.getReturnCode() == Dialog.OK ) {
			int i=attributeList.indexOf(attr);
			Attribute attr_new = null;
			if(getInitialAttributes(classifierRole).contains(attr)){
				attr_new = createAttribute();
			}else{
				attr_new = attr;
			}
			attributeList.set(i,attr_new);
			attr_new.setName(apd.getName());
			attr_new.setType(apd.getType());
			
			if ( apd.getVariableName() != null && ! "".equals( apd.getVariableName() ) ) {
				VariableDeclaration var = null;
				if(!getInitialAttributes(classifierRole).contains(attr) && attr_new.getVariable() != null){
					var = attr_new.getVariable();
				}else{
					var = createVariableDeclaration();
					attr_new.setVariable( var );
				}
				var.setName( apd.getVariableName() );
				var.setType( apd.getType() );
				attr_new.setValue( "" );	// or null ???
				attr_new.setVariable(var);
				if(getInitialAttributes(classifierRole).contains(attr) && attr.getVariable() != null){
					changedVariables.put(var, attr.getVariable());
				}
			} else { // no variable name -> no variable 
				attr_new.setVariable( null );
				attr_new.setValue(apd.getValue());				
			}
			fillAttributeTable();
		}
	}
	
	private List getNewAttributeTypes(){
		List newTypes = new Vector();
		for(int i=0; i<attributeList.size(); i++){
			newTypes.add(((Attribute) attributeList.get(i)).getType());
		}
		return newTypes;
	}
	
	private List getNewAttributeNames(){
		List newTypes = new Vector();
		for(int i=0; i<attributeList.size(); i++){
			newTypes.add(((Attribute) attributeList.get(i)).getName());
		}
		return newTypes;
	}
	
	private List getNewVariableNames(){
		List newVariableNames = new Vector();
		for(int i=0; i<attributeList.size(); i++){
			if(((Attribute) attributeList.get(i)).getVariable() != null)
				newVariableNames.add(((Attribute) attributeList.get(i)).getVariable().getName());
		}
		return newVariableNames;
	}
	
	private List getInitialAttributes(ClassifierRole cr) {
       List l=new ArrayList();
		for ( int i = 0; i < classifierRole.getAttributes().size(); i++ ) {
			Attribute attr = (Attribute) classifierRole.getAttributes().get( i );
			l.add(attr);
		}
		return l;	
		
	}

	// clean and fills the attribute table widget
	private void fillAttributeTable()
	{
		table.removeAll();
		for ( int i = 0; i < attributeList.size(); i++ ) {
			Attribute attr = (Attribute) attributeList.get( i );
			TableItem item = new TableItem( table, SWT.NONE );
			String string[]; 
			if(attr.getVariable() != null){
				string = new String[] { attr.getName(), attr.getType(), attr.getVariable().getName() };
			}else{
				string = new String[] { attr.getName(), attr.getType(), attr.getValue() };
			}
			item.setText(string);
			item.setData( attr );
		}
	}

}
