/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.dialog;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import VOCLModel.ClassifierRole;
import VOCLModel.Constraint;
import VOCLModel.Context;
import VOCLModel.Method;
import VOCLModel.NavExp;

import vocleditor.VOCLEditorPlugin;
import vocleditor.model.VOCLTools;

/**
 * Dialog to modify Context properties
 * @author bjoern
 */
public class ContextPropertiesDialog 
	extends TitleAreaDialog
	implements SelectionListener, ModifyListener
{
	Context    			 context;
	Constraint 			 constraint;
	String     			 name;
	String     			 contextType;
	boolean    			 isClassifier;
	ClassifierRole      contextClassifier;
	Method				 contextMethod;
	
	Text       nameText;
	Combo      contextCombo;
	Button     invButton, preButton, postButton; 
	Label      mthdClsfrLabel;
	
	List allClassifiers;
	List allMethods;
	
	/**
	 * Constructs a ContextPropertiesDialog
	 * @param parentShell parent shell
	 * @param context the {@link Contex} to be shown/modified
	 */
	public ContextPropertiesDialog(Shell parentShell, Context context )
	{   
		super(parentShell);
		this.context       = context;
		this.constraint    = context.getConstraint();
		this.contextType   = context.getType();
		this.isClassifier  = context.isIsClassifier();
		this.contextClassifier = context.getContextClassifier();
		this.contextMethod     = context.getContextMethod();
		this.name = constraint.getName() == null ? "" : constraint.getName();

		getAllClassifiers( contextClassifier );
		getAllMethods( contextMethod );
		setShellStyle( getShellStyle() | SWT.RESIZE );
	}

		
	/**
	 * 
	 * Helper function, adding a Label with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Label addLabelToComposite( Composite composite, String text ) 
	{
		Label label = new Label( composite, 0 );
		label.setText( text );
		GridData gd = new GridData(); gd.widthHint = 150;
		label.setLayoutData( gd );
		return label;
	}

	/**
	 * 
	 * Helper function, adding a Text (-field) with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param textValue The text of the Text
	 * @return the text that was added to composite
	 */
	Text addTextToComposite( Composite composite, String textValue ) 
	{
		Text text = new Text( composite, SWT.SINGLE | SWT.BORDER );
		text.setText( textValue );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		text.setLayoutData( gd );
		text.addModifyListener( this );
		return text;
	}

	/**
	 * 
	 * Helper function, adding a Button with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the button label
	 * @return the button that was added to composite
	 */
	Button addButtonToComposite( Composite composite, String text ) 
	{
		Button button = new Button( composite, SWT.PUSH );
		button.setText( text );
		button.addSelectionListener( this );
		GridData gd = new GridData(); 
		gd.widthHint = 90;
		button.setLayoutData( gd ); 
		return button;
	}
	
	/**
	 * 
	 * Helper function, adding a ComboBox with GridLayout-Data to a Composite
	 * 
	 * @param composite The composite to add the label to
	 * @param text The text of the button label
	 * @return the button that was added to composite
	 */	
    Combo addComboToComposite( Composite composite, String textValue ) {
    	Combo com = new Combo(composite,SWT.DROP_DOWN | SWT.SIMPLE | SWT.READ_ONLY);
    	com.setText( textValue );
    	GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		com.setLayoutData( gd );
		return com;
    }


	/**
	 * implementation of ModifyListener.modifyText() used to show error messages and
	 * disable ok button on incomplete input 
	 */
	public void modifyText(ModifyEvent e)
	{
		name = nameText.getText();	
	}
	
	/** 
	 * Creates the content of the dialog.
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite parent) 
	{  
		Composite superComposite = (Composite) super.createDialogArea(parent);
		
		Composite composite = new Composite( superComposite , 0 );
		composite.setLayoutData( new GridData( GridData.FILL_BOTH ) );

		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout( layout );
		
		addLabelToComposite( composite, "Name:" );
		nameText = addTextToComposite( composite, name ); 

		addLabelToComposite( composite, "Constraint Type:" );
		
		Composite contextVariants = new Composite( composite, 0 );
		contextVariants.setLayout( new RowLayout() );
		
		invButton = new Button( contextVariants, SWT.RADIO );
		invButton.setText( "Invariant" );
		
		preButton = new Button( contextVariants, SWT.RADIO );
		preButton.setText( "Pre-Condition" );

		postButton = new Button( contextVariants, SWT.RADIO );
		postButton.setText( "Post-Condition" );

		mthdClsfrLabel = addLabelToComposite( composite, "Method or Classifier" );
		contextCombo = addComboToComposite( composite, "Hmmm" );
		
		if ( "inv".equals( contextType ) ) {
			mthdClsfrLabel.setText( "Classifier Role:");
			invButton.setSelection( true );
			setComboClassifiers( contextClassifier );
		}
		
		if ( "pre".equals( contextType ) ) {
			mthdClsfrLabel.setText( "Method:");
			preButton.setSelection( true );
			setComboMethods( contextMethod );
		}

		if ( "post".equals( contextType ) ) {
			postButton.setSelection( true );
			setComboMethods( contextMethod );
		}
		
		invButton.addSelectionListener( this );
		postButton.addSelectionListener( this );
		preButton.addSelectionListener( this );

		contextCombo.addModifyListener( this );
		contextCombo.addSelectionListener( this );
		
		setMessage( "Edit the properties of your constraint's context." );
		
		return composite;
	}

	protected void checkGood()
	{
		VOCLEditorPlugin.printDebug( "CONTEXT_DIALOG: name = '" + getContextName() + "'  type = '" + getContextType() + "' isClassifier = " + getIsClassifier() + " classifier = '" + ((getContextClassifier() != null) ? getContextClassifier().getName() : "(none)") + " method = '" + ((getContextMethod() != null) ? getContextMethod().getName() : "(none)") );
		if ( contextType == "undefined" || 
			 contextCombo.getText() == null || contextCombo.getText().equals( "" ) ) {
			setErrorMessage( "You have to specify a context type and a method or classifier role." );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}
		else { 
			setErrorMessage( null );
			getButton(IDialogConstants.OK_ID).setEnabled( true );
		}
	}
	
	/**
	 * Creates contents.
	 * @see org.eclipse.jface.window.Window#createContents(org.eclipse.swt.widgets.Composite)
	 */	 
	protected Control createContents(Composite parent) 
	{	
		Control superContent = super.createContents(parent);
		
		parent.getShell().setSize( 550, 300 );
		parent.getShell().setMinimumSize( 500, 300 );
		parent.getShell().setText( "Context Properties" );
		checkGood();

		return superContent;
	}
	
	/**
	 * Cancel pressed.
	 * @see org.eclipse.jface.dialogs.Dialog#cancelPressed()
	 */
	protected void cancelPressed() {
		super.cancelPressed();
	}
	
	/**
	 * Stores edited classifier role properties after user clicked the "ok"-button.
	 */
	protected void okPressed() {
		if (isClassifier)contextMethod = null;
		else contextClassifier=null;
		super.okPressed();
	}
	
	/**
	 * Returns the context type selected in this dialog ("inv", "pre" or "post").
	 * @return the context type selected in this dialog ("inv", "pre" or "post")
	 */
	public String getContextType() {
		return contextType;
	}
	
	/**
	 * Returns the context name.
	 * @return the context name
	 */
	public String getContextName() {
		return name;
	}
	
	/**
	 * Returns true if the context is a classifier.
	 * @return true if the context is a classifier
	 */
	public boolean getIsClassifier() {
		return isClassifier;
	}

	/**
	 * Returns the classifier role of the context.
	 * @return the classifier role of the context
	 */
	public ClassifierRole getContextClassifier() {
		return contextClassifier;
	}

	/**
	 * Returns the method of the context.
	 * @return the method of the context
	 */
	public Method getContextMethod() {
		return contextMethod;
	}
	
	/**
	 * listens and reacts to "Add", "Edit" and "Delete" buttons
	 */
	public void widgetSelected(SelectionEvent e) 
	{
		if ( e.getSource() == invButton )  {
			isClassifier = true;
			setComboClassifiers( contextClassifier );
			mthdClsfrLabel.setText( "Classifier Role:");
			contextType = "inv";
		} else if ( e.getSource() == preButton || e.getSource() == postButton )  {
			isClassifier = false;
			setComboMethods( contextMethod );
			mthdClsfrLabel.setText( "Method:");
			contextType = e.getSource() == preButton ? "pre" : "post";
		} else if ( e.getSource() == contextCombo )  {
			if ( isClassifier ) {
				VOCLEditorPlugin.printDebug( "CONTEXT_DIALOG: Selected Item: " + contextCombo.getSelectionIndex() );
			}
		}
		if ( isClassifier && ! "".equals( contextCombo.getText() ) ) {
			contextClassifier = (ClassifierRole) allClassifiers.get( contextCombo.getSelectionIndex() );			
		} else {
			if ( contextCombo.getSelectionIndex() >= 0 )
				contextMethod = (Method) allMethods.get( contextCombo.getSelectionIndex() );
			else
				contextMethod = null;
		}
		name = nameText.getText(); 
		checkGood();
	}
	
	/**
	 * Widged default selected listener.
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected( SelectionEvent e ) 
	{
		widgetSelected( e );
	}
	
	
	protected void setComboMethods( Method selectMethod)
	{
		contextCombo.removeAll();

		for ( int i = 0; i < allMethods.size(); i++ ) {
			Method m = (Method) allMethods.get(i);
			contextCombo.add( m.getName() + "() : " + m.getReturnType() );
			if ( m == selectMethod )
				contextCombo.select( i );			
		}
		if ( selectMethod == null ) 
			contextCombo.select( 0 );
	}

	
	protected void setComboClassifiers( ClassifierRole selectClassifier )
	{
		contextCombo.removeAll();
		
		for ( int i = 0; i < allClassifiers.size(); i++ ) {
			ClassifierRole c = (ClassifierRole) allClassifiers.get(i);
			contextCombo.add( c.getName() + ": " + c.getType() );
			if ( c == selectClassifier )
				contextCombo.select( i );
		}
		if ( selectClassifier == null ) 
			contextCombo.select( 0 );
	}
	
	private void getAllClassifiers( ClassifierRole preferredClassifier )
	{
		List classifiers = VOCLTools.getAllClassifiers( (NavExp) constraint.getExpression(), new Vector() );
		allClassifiers = new Vector();
		if ( preferredClassifier != null )
			allClassifiers.add( preferredClassifier );
		for ( Iterator i = classifiers.iterator(); i.hasNext(); ) {
			ClassifierRole in = (ClassifierRole) i.next();
			boolean alreadyThere = false;
			for ( Iterator j = allClassifiers.iterator(); j.hasNext(); ) {
				ClassifierRole drin = (ClassifierRole) j.next();
				if ( in.getName().equals( drin.getName() ) && in.getType().equals( drin.getType() ) ) {
					alreadyThere = true;
					break;
				}
			}
			if ( ! alreadyThere )
				allClassifiers.add( in );
		}
	}
		
	private void getAllMethods( Method preferredMethod )
	{
		List classifiers = VOCLTools.getAllClassifiers( (NavExp) constraint.getExpression(), new Vector() );
		List methods = VOCLTools.getAllMethods( classifiers, new Vector() );		
		allMethods = new Vector();
		if ( preferredMethod != null )
			allMethods.add( preferredMethod );
		
		for ( Iterator i = methods.iterator(); i.hasNext(); ) {
			Method in = (Method) i.next();
			String inS = in.getName() + "() : " + in.getReturnType();
			boolean alreadyThere = false;
			for ( Iterator j = allMethods.iterator(); j.hasNext(); ) {
				Method drin = (Method) j.next();
				if ( inS.equals( drin.getName() + "() : " + drin.getReturnType() ) ) {
					alreadyThere = true;
					break;
				}
			}
			if ( ! alreadyThere )
				allMethods.add( in );
		}
	}
}
