/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.dialog;

import java.util.Hashtable;
import java.util.List;
import java.util.ArrayList;
import java.util.Vector;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

import vocleditor.VOCLEditorPlugin;
import vocleditor.model.ModelCreationFactory;

import VOCLModel.Attribute;
import VOCLModel.Constraint;
import VOCLModel.SetClassifierRole;
import VOCLModel.VariableDeclaration;

/**
 * @author bjoern, Stefan, tugstugi
 * Dialog to modify SetSetClassifierRole properties
 */
public class SetClassifierRolePropertiesDialog extends TitleAreaDialog implements SelectionListener, ModifyListener {
	private Text nameText;
	private Combo typeCombo;
	private Combo opKindCombo;
	private Text sizeText;
	private Combo iteratorCombo;
	private Text sumText;	
	private Label sizeLabel;
	private Label iteratorLabel;
	private Label sumLabel;
	private String name;
	private String type;
	private String opKind;
	private String size;
	private String iterator;
	private String sum;
	private Constraint constraint;
	private SetClassifierRole setClassifierRole;
	private Table table;	
	private Button addAttribButton, editAttribButton, delAttribButton;
	private List attributeList = new ArrayList();
	private Hashtable changedVariables = new Hashtable();
	

	/**
	 * Constructs a SetClassifierPropertiesDialog
	 * @param parentShell the parent shell
	 * @param constraint the constraint
	 * @param setClassifierRole the SetClassifierRole to be shown/modified
	 */
	public SetClassifierRolePropertiesDialog(Shell parentShell, Constraint constraint, SetClassifierRole setClassifierRole){   
		super(parentShell);
		this.constraint = constraint;
		this.setClassifierRole = setClassifierRole;
		attributeList = getInitialAttributes(setClassifierRole);
		setShellStyle( getShellStyle() | SWT.RESIZE );
		name = setClassifierRole.getName() != null ? setClassifierRole.getName() : ""; 
		type = setClassifierRole.getType() != null ? setClassifierRole.getType() : "";
		opKind = setClassifierRole.getOpKind() != null ? setClassifierRole.getOpKind() : "";
		size = setClassifierRole.getSize() != null ? setClassifierRole.getSize().getName() : "";
		iterator = setClassifierRole.getIterator() != null ? setClassifierRole.getIterator().getName() : "";
		sum = setClassifierRole.getSum() != null ? setClassifierRole.getSum().getName() : "";
	}

		
	/**
	 * Helper function, adding a Label with GridLayout-Data to a Composite
	 * @param composite The composite to add the label to
	 * @param text The text of the label to be added to the composite
	 * @return the label that was added to composite
	 */
	Label addLabelToComposite( Composite composite, String text ) {
		Label label = new Label( composite, 0 );
		label.setText( text );
		GridData gd = new GridData(); gd.widthHint = 100;
		label.setLayoutData( gd );
		return label;
	}

	/**
	 * Helper function, adding a Text (-field) with GridLayout-Data to a Composite
	 * @param composite The composite to add the label to
	 * @param textValue The text of the Text
	 * @return the text that was added to composite
	 */
	Text addTextToComposite( Composite composite, String textValue ) {
		Text text = new Text( composite, SWT.SINGLE | SWT.BORDER );
		text.setText( textValue );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		text.setLayoutData( gd );
		text.addModifyListener( this );
		return text;
	}

	/**
	 * Helper function, adding a Button with GridLayout-Data to a Composite
	 * @param composite The composite to add the label to
	 * @param text The text of the button label
	 * @return the button that was added to composite
	 */
	Button addButtonToComposite( Composite composite, String text )	{
		Button button = new Button( composite, SWT.PUSH );
		button.setText( text );
		button.addSelectionListener( this );
		GridData gd = new GridData(); 
		gd.widthHint = 90;
		button.setLayoutData( gd ); 
		return button;
	}
	
	/**
	 * Helper function, adding a Combo with GridLayout-Data to a Composite
	 * @param composite The composite to add the label to
	 * @param textValue The text of the Text
	 * @return the combobox that was added to composite
	 */
    Combo addComboToComposite(Composite composite, String textValue) {
    	Combo com = new Combo(composite,SWT.DROP_DOWN | SWT.SIMPLE);
    	com.setText(textValue);
    	GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		com.setLayoutData( gd );
		com.addModifyListener( this );
		return com;
    }
    
    /**
	 * Helper function, adding a read only Combo with GridLayout-Data to a Composite
	 * @param composite The composite to add the label to
	 * @param textValue The text of the Text
	 * @return the combobox that was added to composite
	 */
    Combo addReanOnlyComboToComposite(Composite composite, String textValue) {
    	Combo com = new Combo(composite,SWT.DROP_DOWN | SWT.SIMPLE | SWT.READ_ONLY);
    	com.add(textValue);
    	com.select(com.indexOf(textValue));
    	GridData gd = new GridData( GridData.FILL_HORIZONTAL ); gd.minimumWidth = 100; gd.grabExcessHorizontalSpace = true;
		com.setLayoutData( gd );
		com.addModifyListener( this );
		return com;
    }
    
    private void refreshIteratorCombo(){
    	String selectedText = iteratorCombo.getText();
    	iteratorCombo.removeAll();
    	for(int i=0; i<attributeList.size(); i++){
    		Attribute attribute = (Attribute) attributeList.get(i);
    		if(attribute.getVariable() != null)
    			iteratorCombo.add(attribute.getVariable().getName());
    	}
    	String items[] = iteratorCombo.getItems();
    	for(int i=0; i<items.length; i++){
    		if(selectedText.equals(items[i])){
    			iteratorCombo.select(iteratorCombo.indexOf(selectedText));
    			return;
    		}    			
    	}
    	iteratorCombo.setText("");
    }


	/**
	 * Implementation of ModifyListener.modifyText() used to show error messages and
	 * disable ok button on incomplete input. 
	 */
	public void modifyText(ModifyEvent e){	
		if("size".equals(opKindCombo.getText())){
			sizeText.setVisible(true);
			sizeLabel.setVisible(true);
			iteratorCombo.setVisible(false);
			iteratorLabel.setVisible(false);
			sumText.setVisible(false);
			sumLabel.setVisible(false);
		}else if("sum".equals(opKindCombo.getText())){
			sizeText.setVisible(false);
			sizeLabel.setVisible(false);
			iteratorCombo.setVisible(true);
			iteratorLabel.setVisible(true);
			sumText.setVisible(true);
			sumLabel.setVisible(true);
		}else{
			sizeText.setVisible(false);
			sizeLabel.setVisible(false);
			iteratorCombo.setVisible(false);
			iteratorLabel.setVisible(false);
			sumText.setVisible(false);
			sumLabel.setVisible(false);
		}
		
		
		if ( typeCombo.getText().length() == 0 ) {
			setErrorMessage( "You must at least specify the type of your Set Classifier Role!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		} else if("size".equals(opKindCombo.getText()) && sizeText.getCharCount() == 0){
			setErrorMessage( "You must at least specify the return variable for the size operation!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}else if ( nameText.getCharCount() >0 && existsWhiteSpace(nameText.getText())) {
			setErrorMessage( "Whitespace is not allowed in setclassifierrole name!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}else if ( sumText.getCharCount() >0 && existsWhiteSpace(sumText.getText())) {
			setErrorMessage( "Whitespace is not allowed in sum variable name!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}else if ( sizeText.getCharCount() >0 && existsWhiteSpace(sizeText.getText())) {
			setErrorMessage( "Whitespace is not allowed in size variable name!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}
		/*
		else if("size".equals(opKindCombo.getText()) && sizeText.getCharCount() > 0 && existsVariableName(sizeText.getText(), false) && (setClassifierRole.getSize() == null || !sizeText.getText().equals(setClassifierRole.getSize().getName()))){
			setErrorMessage( "The variable " + sizeText.getText() + " is allready exists!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}		
		else if("sum".equals(opKindCombo.getText()) && sumText.getCharCount() >0 && existsVariableName(sumText.getText(), true) && (setClassifierRole.getSum() == null || !sumText.getText().equals(setClassifierRole.getSum().getName()))){
			setErrorMessage( "The variable " + sumText.getText() + " is allready exists!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		}
		*/ 
		else if("sum".equals(opKindCombo.getText()) && (iteratorCombo.getText().length() == 0 || sumText.getCharCount() ==0 )){
			setErrorMessage( "You must at least specify the iterator variable  and sum variable for the sum operation!" );
			getButton(IDialogConstants.OK_ID).setEnabled( false );
		} else {
			setErrorMessage( null );
			setMessage( "Edit the properties of your Set Classifier Role." );
			getButton(IDialogConstants.OK_ID).setEnabled( true );
		}
		name = nameText.getText();
		type = typeCombo.getText();
		opKind = opKindCombo.getText();
		size = sizeText.getText();
		iterator = iteratorCombo.getText();
		sum = sumText.getText();
	}
	
	private boolean existsWhiteSpace(String name){		
			String att = name;
			for(int j=0; j<att.length(); j++){
			 char c = att.charAt(j);
			 if(Character.isWhitespace(c))
				return true;
			}		
		
		return false;
	}
	
	/** 
	 * Creates the content of the dialog.
	 */
	protected Control createDialogArea(Composite parent) {  
		Composite superComposite = (Composite) super.createDialogArea(parent);		
		Composite composite = new Composite( superComposite , 0 );
		composite.setLayoutData( new GridData( GridData.FILL_BOTH ) );
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		composite.setLayout( layout );		
		addLabelToComposite( composite, "Name:" );
		nameText = addTextToComposite( composite, name ); 
		addLabelToComposite( composite, "Type:" );
		typeCombo = addComboToComposite( composite, type);
		List classifierRoleTypes = VOCLEditorPlugin.getAllClassifierRoleTypes(constraint);
		for(int i=0; i<classifierRoleTypes.size(); i++){
			typeCombo.add((String) classifierRoleTypes.get(i));
		}
		addLabelToComposite( composite, "Operation:" );
		opKindCombo = addReanOnlyComboToComposite( composite, opKind);		
		opKindCombo.add("isEmpty");
		opKindCombo.add("notEmpty");
		opKindCombo.add("size");
		opKindCombo.add("sum");
		opKindCombo.add("");
		sizeLabel = addLabelToComposite( composite, "Size variable:" );
		sizeText = addTextToComposite( composite, size );
		iteratorLabel = addLabelToComposite( composite, "Iterator variable:" );
		iteratorCombo = addReanOnlyComboToComposite( composite, iterator );
		sumLabel = addLabelToComposite( composite, "Sum variable:" );
		sumText = addTextToComposite( composite, sum );
		Group attributeWidgets = new Group( composite, 0 );
		attributeWidgets.setText( "Attributes" );		
		attributeWidgets.setLayoutData( new GridData( SWT.FILL, SWT.FILL, true, true, 2, 1 ) );
		layout = new GridLayout();
		layout.numColumns = 2;
		attributeWidgets.setLayout( layout );
		table = new Table( attributeWidgets, SWT.BORDER );
		table.setHeaderVisible( true );
		table.setLayoutData( new GridData( SWT.FILL, SWT.FILL, true, true, 1, 4 ) );		
		TableColumn column = new TableColumn( table, 0 );
		column.setText( "Name" );
		column.setWidth( 100 );
		column = new TableColumn( table, 0 );
		column.setText( "Type" );
		column.setWidth( 100 );
		column = new TableColumn( table, 0 );
		column.setText( "Value/Variable" );
		column.setWidth( 100 );
				
		addAttribButton = addButtonToComposite( attributeWidgets, "Add" ); 
		editAttribButton = addButtonToComposite( attributeWidgets, "Edit" );
		delAttribButton = addButtonToComposite( attributeWidgets, "Delete" );

		fillAttributeTable();
		
		return composite;
	}

	/**
	 * Creates contents.
	 * @see org.eclipse.jface.window.Window#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {		
		Control superContent = super.createContents(parent);		
		parent.getShell().setSize( 550, 550 );
		parent.getShell().setMinimumSize( 400, 400 );
		parent.getShell().setText( "Set Classifier Role Properties" );
		nameText.setText( nameText.getText() );  // to call listeners		
		refreshIteratorCombo();
		return superContent;
	}
	
	/**
	 * Returns the set classifier role name.
	 * @return the set classifier role name
	 */
	public String getClassifierName() {
		return name;
	}
	
	/**
	 * Returns the set classifier role type.
	 * @return the set classifier role type
	 */
	public String getClassifierType() {
		return type;
	}
	
	/**
	 * Returns the operation kind of the set classifier role.
	 * @return the operation kind of the set classifier role
	 */
	public String getOpKind() {
		return opKind;
	}
	
	/**
	 * Returns the name of the size variable.
	 * @return the name of the size variable
	 */
	public String getSizeVariable(){
		return size;
	}
	
	/**
	 * Returns the name of the iterator variable.
	 * @return the name of the iterator variable
	 */
	public String getIteratorVariable(){
		return iterator;
	}
	
	/**
	 * Returns the name of the sum variable.
	 * @return the name of the sum variable
	 */
	public String getSumVariable(){
		return sum;
	}
	
	/**
	 * Returns the hashtable of the changed variables.
	 * @return the hashtable of the changed variables 
	 */
	public Hashtable getChangedVariables(){
		return changedVariables;
	}
	
	/**
	 * Returns the list of attributes.
	 * @return the list of attribtues
	 */
	public List getAttributes() {
		return attributeList;
	}
	
	/**
	 * Listens and reacts to "Add", "Edit" and "Delete" buttons
	 */
	public void widgetSelected(SelectionEvent e) 
	{
		if ( e.getSource() == addAttribButton )  {
			addAttribute();
		    fillAttributeTable();
		    refreshIteratorCombo();
		}
		if ( e.getSource() == editAttribButton ) { 
			TableItem tis[] = table.getSelection();
			if ( tis.length > 0 )
				editAttribute((Attribute) tis[0].getData());
			refreshIteratorCombo();
		}
		if ( e.getSource() == delAttribButton ) { 
			TableItem tis[] = table.getSelection();
			if ( tis.length > 0 ) {
				for ( int i = 0; i < tis.length; i++ ) { 
					Attribute attr=(Attribute) tis[0].getData();
					attributeList.remove(attr);
					if(attr.getVariable() != null && changedVariables.containsKey(attr.getVariable()))
						changedVariables.remove(attr.getVariable());
		            fillAttributeTable();
				}
			}
			refreshIteratorCombo();
		}
	}

	/**
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected( SelectionEvent e ){
		widgetSelected( e );
	}
	
	private Attribute createAttribute() {		
		return (Attribute) (new ModelCreationFactory(Attribute.class)).getNewObject();		
	}
	
	private VariableDeclaration createVariableDeclaration() {		
		return (VariableDeclaration) (new ModelCreationFactory(VariableDeclaration.class)).getNewObject();		
	}

	// called after "Add" (add attribute to class.role) button 
	private void addAttribute() {
		Attribute attr = createAttribute();
		AttributePropertiesDialog apd = new AttributePropertiesDialog( getShell(), constraint, setClassifierRole, attr, getNewVariableNames(), getNewAttributeNames(), getNewAttributeTypes());
		apd.open();
		if ( apd.getReturnCode() == Dialog.OK ) {
			attr.setName(apd.getName());
			attr.setType(apd.getType());
			attr.setValue(apd.getValue());
			String varName = apd.getVariableName();
			if(varName != null && !"".equals(varName)){
				VariableDeclaration varDecl = createVariableDeclaration();
				varDecl.setName(varName);
				varDecl.setType(attr.getType());
				attr.setVariable(varDecl);
			}
			attributeList.add(attr);
		}
	}
	
	private void editAttribute( Attribute attr ){
		AttributePropertiesDialog apd = new AttributePropertiesDialog( getShell(), constraint, setClassifierRole, attr, getNewVariableNames(), getNewAttributeNames(), getNewAttributeTypes());
		apd.open();
		if ( apd.getReturnCode() == Dialog.OK ) {
			int i=attributeList.indexOf(attr);
			Attribute attr_new = null;
			if(getInitialAttributes(setClassifierRole).contains(attr)){
				attr_new = createAttribute();
			}else{
				attr_new = attr;
			}
			attributeList.set(i,attr_new);
			attr_new.setName(apd.getName());
			attr_new.setType(apd.getType());
			
			if ( apd.getVariableName() != null && ! "".equals( apd.getVariableName() ) ) {
				VariableDeclaration var = null;
				if(!getInitialAttributes(setClassifierRole).contains(attr) && attr_new.getVariable() != null){
					var = attr_new.getVariable();
				}else{
					var = createVariableDeclaration();
					attr_new.setVariable( var );
				}
				var.setName( apd.getVariableName() );
				var.setType( apd.getType() );
				attr_new.setValue( "" );	// or null ???
				attr_new.setVariable(var);
				if(getInitialAttributes(setClassifierRole).contains(attr) && attr.getVariable() != null){
					changedVariables.put(var, attr.getVariable());
				}
			} else { // no variable name -> no variable 
				attr_new.setVariable( null );
				attr_new.setValue(apd.getValue());				
			}
			fillAttributeTable();
		}
	}
	
	private List getNewAttributeTypes(){
		List newTypes = new Vector();
		for(int i=0; i<attributeList.size(); i++){
			newTypes.add(((Attribute) attributeList.get(i)).getType());
		}
		return newTypes;
	}
	
	private List getNewAttributeNames(){
		List newTypes = new Vector();
		for(int i=0; i<attributeList.size(); i++){
			newTypes.add(((Attribute) attributeList.get(i)).getName());
		}
		return newTypes;
	}
	
	private List getNewVariableNames(){
		List newVariableNames = new Vector();
		if(sum.length()>0)
			newVariableNames.add(sum);
		if(size.length()>0)
			newVariableNames.add(size);
		for(int i=0; i<attributeList.size(); i++){
			if(((Attribute) attributeList.get(i)).getVariable() != null)
				newVariableNames.add(((Attribute) attributeList.get(i)).getVariable().getName());
		}
		return newVariableNames;
	}
	
	private boolean existsVariableName(String variableName, boolean sumFlag){
		List allVariables = constraint.getAllVariables();
		for(int i=0; i<allVariables.size(); i++){
			if(variableName.equals(((VariableDeclaration) allVariables.get(i)).getName()))
				return true;
		}
		List newVariableNames = getNewVariableNames();
		if(sumFlag)
			newVariableNames.remove(sum);
		else
			newVariableNames.remove(size);
		for(int i=0; i<newVariableNames.size(); i++){
			if(variableName.equals(newVariableNames.get(i)))
				return true;
		}
		return false;
	}
	
	private List getInitialAttributes(SetClassifierRole cr) {
       List l=new ArrayList();
		for ( int i = 0; i < setClassifierRole.getAttributes().size(); i++ ) {
			Attribute attr = (Attribute) setClassifierRole.getAttributes().get( i );
			l.add(attr);
		}
		return l;	
		
	}

	// clean and fills the attribute table widget
	private void fillAttributeTable() {
		table.removeAll();
		for ( int i = 0; i < attributeList.size(); i++ ) {
			Attribute attr = (Attribute) attributeList.get( i );
			TableItem item = new TableItem( table, SWT.NONE );
			String string[]; 
			if(attr.getVariable() != null){
				string = new String[] { attr.getName(), attr.getType(), attr.getVariable().getName() };
			}else{
				string = new String[] { attr.getName(), attr.getType(), attr.getValue() };
			}
			item.setText(string);
			item.setData( attr );
		}
	}
}
