/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.edit;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.swt.widgets.Shell;

import VOCLModel.Attribute;
import VOCLModel.BoolExp;
import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.Context;
import VOCLModel.IfExp;
import VOCLModel.LetExp;
import VOCLModel.Method;
import VOCLModel.NavExp;
import VOCLModel.VOCLDiagram;
import VOCLModel.ClassifierRole;
import VOCLModel.AssociationRole;
import VOCLModel.Label;

import vocleditor.VOCLEditorPlugin;

/**
 * Factory class for graphical edit parts.
 * Here, the corresponding edit parts for vocl model elements are created.
 */
public class GraphicalEditPartsFactory implements EditPartFactory {	
	private Shell shell;
	
	public GraphicalEditPartsFactory(Shell shell){
		super();
		this.shell = shell;
	}
	
	public EditPart createEditPart(EditPart context, Object obj){
		if(obj instanceof VOCLDiagram){
			VOCLEditPart p =  new VOCLEditPart((VOCLDiagram)obj);
			p.setShell( shell );
			return p;
		} else if(obj instanceof ClassifierRole){
			VOCLClassifierRoleEditPart p = new VOCLClassifierRoleEditPart((ClassifierRole)obj);
			p.setShell( shell );
			return p;
		} else if ( obj instanceof Constraint) {
			VOCLConstraintEditPart p = new VOCLConstraintEditPart( (Constraint) obj );
			p.setShell( shell );
			return p;
		} else if (obj instanceof AssociationRole) {
			VOCLAssociationRoleEditPart p = new VOCLAssociationRoleEditPart( (AssociationRole) obj );
			p.setLayerEditPart( (NavExpEditPart) ((VOCLClassifierRoleEditPart) context).getParent() );
			return p;
		} else if (obj instanceof Attribute) {
			AttributeEditPart p = new AttributeEditPart( (Attribute) obj );
			p.setShell( shell );
			return p;
		} else if (obj instanceof Method) {
			MethodEditPart p = new MethodEditPart( (Method) obj );
			p.setShell( shell );
			return p;
		} else if (obj instanceof Condition) {
			ConditionEditPart p = new ConditionEditPart( (Condition) obj );
			p.setShell( shell );
			return p;
		} else if (obj instanceof NavExp) {
			NavExpEditPart p = new NavExpEditPart( (NavExp) obj );
			p.setShell( shell );
			return p;
		} else if (obj instanceof BoolExp) {
			BoolExpEditPart p = new BoolExpEditPart( (BoolExp) obj );
			p.setShell( shell );
			return p;
		} else if (obj instanceof LetExp) {
			LetExpEditPart p = new LetExpEditPart( (LetExp) obj );
			p.setShell( shell );
			return p;
		} else if (obj instanceof IfExp) {
			IfExpEditPart p = new IfExpEditPart( (IfExp) obj );
			p.setShell( shell );
			return p;
		} else if (obj instanceof Context) {
			ContextEditPart p = new ContextEditPart( (Context) obj );
			p.setShell( shell );
			VOCLEditorPlugin.printDebug( "CREATING CONTEXT PART");
			return p;
		} else if (obj instanceof Label) {
			LabelGraphicalEditPart p = new LabelGraphicalEditPart( ((Label)obj).getAssociationRole() );
			p.setModel(obj);
			return p;
		} else
			System.err.println( "Not creating an edit part for unknown class: " + obj.getClass().getName() );

		return null;
	}
}
