/*
 * Created on May 31, 2005
 */

package vocleditor.edit;



import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;

import vocleditor.VOCLEditorPlugin;
import vocleditor.commands.EditMethodCommand;
import vocleditor.dialog.MethodPropertiesDialog;
import vocleditor.figures.MethodFigure;

import VOCLModel.ClassifierRole;
import VOCLModel.Method;
import VOCLModel.VariableDeclaration;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.Shell;

/**
 * The edit part for a method.
 *
 * @author tugstugi
 */
public class MethodEditPart extends AbstractVOCLEditPart{
	
	/**
	 * The default constructor.
	 * @param method the method
	 */
	public MethodEditPart(Method method){
		setModel(method);
	}
	
	/**
	 * Converts the method as string.
	 * @return the name
	 */
	public String getMethodAsString(){
		Method method = (Method) getModel(); 	
		String name = method.getName();
		if(method.getReturnVariable() != null)
			name = method.getReturnVariable().getName() + " = " + name;
		if(method.getArguments().size() == 0){
			name = name + "()";
		} else{
			name = name + "(";
			for(int i=0; i<method.getArguments().size(); i++){
				name = name + ((VariableDeclaration) method.getArguments().get(i)).getName();
				if(i != method.getArguments().size() - 1)
					name = name + ", ";
			}
			name = name + ")";
		}
		return name;
	}
	
	/**
	 * Creates a {@link MethodFigure}
	 */
	protected IFigure createFigure() {		
		MethodFigure methodFigure = new MethodFigure();		
		methodFigure.setName(getMethodAsString());
		return methodFigure;
	}

	/**
	 * Installs an {@link MethodComponentEditPolicy}
	 */
	protected void createEditPolicies() {		
		installEditPolicy( EditPolicy.COMPONENT_ROLE, new MethodComponentEditPolicy() );
	}

	/**
	 * Refreshs the figure's name and calls {@link #refreshVisuals()}
	 */
	public void notifyChanged(Notification arg0){		
		((MethodFigure) getFigure()).setName(getMethodAsString());
		refreshVisuals();
	}
	
	/**
	 * Calculates position and size and refreshs itsself
	 */
	protected void refreshVisuals() {
		Method method = (Method) getModel();
		ClassifierRole classifierRole = method.getClassifierRole();
		if(classifierRole == null)
			return;
		int x = classifierRole.getX();
		int y = classifierRole.getY();
		y = y - getFigure().getMinimumSize().height;
		List parentChildren = ((GraphicalEditPart) getParent()).getChildren();
		for(int i=0; i<parentChildren.size(); i++){
			if(classifierRole.equals(((GraphicalEditPart) parentChildren.get(i)).getModel())){
				IFigure parentFigure = ((GraphicalEditPart) parentChildren.get(i)).getFigure();
				x = x + parentFigure.getMinimumSize().width/2 - MethodFigure.ARROW_HEAD_WIDTH/2;
				break;
			}
		}
		((GraphicalEditPart) getParent()).setLayoutConstraint( this, getFigure(), new Rectangle(x,y, getFigure().getMinimumSize().width, getFigure().getMinimumSize().height));
	}

	/**
	 * Opens the properties dialog and creates and executes an
	 * {@link EditMethodCommand} if necessary
	 */
	public void performRequest(Request request) {
		if(request.getType() == RequestConstants.REQ_OPEN) {
			Command command = null;
			
			AbstractVOCLEditPart editpart = (MethodEditPart) this;
			command = new EditMethodCommand();
			Method method = (Method) editpart.getModel();
			((EditMethodCommand) command).setMethod(method);			
			MethodPropertiesDialog mpd = new MethodPropertiesDialog( new Shell(), VOCLEditorPlugin.getConstraint(method.getClassifierRole()) , method.getClassifierRole(), method);
			mpd.open();
			if( mpd.getReturnCode() == Dialog.CANCEL ) 
				return;
			((EditMethodCommand) command).setCommandAttributes(mpd.getName(), mpd.getReturnType(), mpd.getReturnVariableName(), mpd.getChangedParameters(), mpd.getParameters());

			getRoot().getViewer().getEditDomain().getCommandStack().execute(command);
		}
	}
}
