/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.edit;

import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.RootComponentEditPolicy;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.Shell;

import VOCLModel.Constraint;
import VOCLModel.BoolExp;
import VOCLModel.ClassifierRole;
import VOCLModel.IfExp;
import VOCLModel.LetExp;
import VOCLModel.NavExp;

import vocleditor.commands.EditBoolExpCommand;
import vocleditor.commands.EditIfExpCommand;
import vocleditor.commands.EditLetExpCommand;
import vocleditor.dialog.BoolExpPropertiesDialog;
import vocleditor.dialog.IfExpPropertiesDialog;
import vocleditor.dialog.LetExpPropertiesDialog;
import vocleditor.figures.NavExpFigure;


/**
 * EditPart for a NavExp ( (V)OCL Navigation Expression). 
 * @author bjoern 
 */
public class NavExpEditPart extends AbstractVOCLEditPart
{
	
	/**
	 * Creates a new NavExpEditPart instance.
	 * @param navexp The NavExp model element this EditPart if there for
	 */
	public NavExpEditPart( NavExp navexp )
	{
		setModel( navexp );
	}


	/**
	 * Returns the model children (ClassifierRoles and Methods) for this NavExp.
	 * @return the model children (ClassifierRoles and Methods) for this NavExp.
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelChildren()
	 */
	protected List getModelChildren() 
	{
 		Vector children = new Vector();
		children.addAll( ((NavExp)getModel()).getClassifierRoles() );
		List classifierRoles = ((NavExp)getModel()).getClassifierRoles();
		for(int i=0; i<classifierRoles.size(); i++){
			ClassifierRole classifierRole = (ClassifierRole) classifierRoles.get(i);
			if(classifierRole.getMethod() != null)
				children.add(classifierRole.getMethod());
		}
		children.addAll( ((NavExp)getModel()).getExpressions() );
		if(((NavExp)getModel()).getCondition() != null)
			children.add(((NavExp)getModel()).getCondition());
 		return children;
	}
	
	/**
	 * Refreshes the child edit parts.
	 * 
	 * (Redefined with visibility public)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#refreshChildren()
	 */
	public void refreshChildren()
	{
		super.refreshChildren();
	}
	

	/**
	 * Recieves a Notification about model changes and reacts
	 * by refreshing the list model children and refreshing the views.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#notifyChanged()
	 */
	public void notifyChanged(Notification notification) {
		if ( notification.getEventType() == Notification.ADD ) {
			refreshChildren();
		}
		if ( notification.getEventType() == Notification.REMOVE ) {
			refreshChildren();
		}
		refreshChildren();
		refreshVisuals();
	}


	/**
	 * Creates the edit policies for this NavExpEditPart.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new NavExpLayoutEditPolicy());
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new RootComponentEditPolicy()); 	
	}	
	
	/**
	 * Returns the NavExpFigure for this NavExpEditPart
	 * 
	 * @return the NavExpFigure for this NavExpEditPart
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		return new NavExpFigure();
	}

	
	/**
	 * Returns the content pane of this NavExpEditPart's figure.
	 *  
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getContentPane()
	 */
	public IFigure getContentPane() {
		return ((NavExpFigure) getFigure()).getContentPane();
	}
	
	/**
	  * Returns the connection layer of this NavExpEditPart's figure.
	  * 
	  * @return the connection layer of this NavExpEditPart's figure.
	  */
	  protected IFigure getConnectionLayer()
	  {
  		  return ((NavExpFigure) getFigure()).getConnectionLayer();
	  }
	  
	  /**
		 * Opens the properties dialog on mouse double-click and creates and executes an
		 * {@link EditBoolExpCommand},{@link EditIfExpCommand} or {@link EditLetExpCommand}dependent
		 *  from the current parent edit part
		 */
	  public void performRequest(Request request) {
		if(request.getType() == RequestConstants.REQ_OPEN) {
			Command command = null;
			EditPart editpart = this.getParent();
			if (editpart.getModel() instanceof Constraint)
				return;
			else if (editpart  instanceof BoolExpEditPart) {
			BoolExp boolexp = (BoolExp)editpart.getModel();
			command = new EditBoolExpCommand( boolexp );
			BoolExpPropertiesDialog bpd = new BoolExpPropertiesDialog( new Shell(), boolexp );
			bpd.open();
			if( bpd.getReturnCode() == Dialog.CANCEL)
				return;
			((EditBoolExpCommand)command).setCommandAttributes(  bpd.getOperation(), bpd.getIsHorizontal(), bpd.getSplitPercentage(), bpd.isNegated() );
			}
			else if (editpart instanceof IfExpEditPart ) {
				IfExp ifexp = (IfExp)editpart.getModel();
				command = new EditIfExpCommand( ifexp );
				IfExpPropertiesDialog ipd = new IfExpPropertiesDialog( new Shell(), ifexp );
				ipd.open();
				if( ipd.getReturnCode() == Dialog.CANCEL)
					return;
				((EditIfExpCommand)command).setCommandAttributes(  ipd.getIsHorizontal(), ipd.getSplitPercentage1(), ipd.getSplitPercentage2(), ipd.isNegated() );
			}
			else if (editpart instanceof LetExpEditPart ) {
				LetExp letexp = (LetExp)editpart.getModel();
				command = new EditLetExpCommand( letexp );
				LetExpPropertiesDialog bpd = new LetExpPropertiesDialog( new Shell(), letexp );
				bpd.open();
				if( bpd.getReturnCode() == Dialog.CANCEL)
					return;
				((EditLetExpCommand)command).setCommandAttributes( bpd.getIsHorizontal(), bpd.getSplitPercentage(), bpd.getVariableDeclaration(), bpd.isNegated() );
				}
			getRoot().getViewer().getEditDomain().getCommandStack().execute(command);
		}
	}
	  
}
