/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.edit;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.CreateRequest;

import vocleditor.commands.CreateBoolExpCommand;
import vocleditor.commands.CreateClassifierRoleCommand;
import vocleditor.commands.CreateConditionCommand;
import vocleditor.commands.CreateIfExpCommand;
import vocleditor.commands.CreateLetExpCommand;
import vocleditor.commands.CreateSetClassifierRoleCommand;
import vocleditor.commands.SetConstraintCommand;
import VOCLModel.BoolExp;
import VOCLModel.ClassifierRole;
import VOCLModel.Condition;
import VOCLModel.IfExp;
import VOCLModel.LetExp;
import VOCLModel.NavExp;
import VOCLModel.SetClassifierRole;

/**
 * EditPolicy for XY-Layout.
 */
public class NavExpLayoutEditPolicy extends XYLayoutEditPolicy {
	
	protected Command createAddCommand(EditPart child, Object constraint) {
		return null;
	}
	
	protected Command getCreateCommand(CreateRequest request) {
		Object	newObjectType = request.getNewObjectType();
		Command	createCommand = null;
		NavExpEditPart navExpEP = (NavExpEditPart) getHost();
		NavExp navexp = (NavExp) navExpEP.getModel();
		Point np = new Point(  navExpEP.getFigure().getBounds().x, navExpEP.getFigure().getBounds().y );
		navExpEP.getFigure().translateToAbsolute( np );
		Point loco = new Point( request.getLocation().x - np.x, request.getLocation().y - np.y );

	    if( newObjectType == SetClassifierRole.class ) {
			CreateSetClassifierRoleCommand create = new CreateSetClassifierRoleCommand();
			create.setSetClassifierRole( (SetClassifierRole) request.getNewObject() );
			create.setParent( navexp ); 
			create.setLocation( loco );
			createCommand = create;
		}else if( newObjectType == BoolExp.class ) {
			CreateBoolExpCommand create = new CreateBoolExpCommand( (BoolExp) request.getNewObject(), navexp );
			create.setLocation( loco );
			createCommand = create;
		}else if( newObjectType == LetExp.class ) {
			CreateLetExpCommand create = new CreateLetExpCommand( (LetExp) request.getNewObject(), navexp );
			create.setLocation( loco );
			createCommand = create;
		}else if( newObjectType == IfExp.class ) {
			CreateIfExpCommand create = new CreateIfExpCommand( (IfExp) request.getNewObject(), navexp );
			create.setLocation( loco );
			createCommand = create;
		}else if( newObjectType == ClassifierRole.class ) {
			CreateClassifierRoleCommand create = new CreateClassifierRoleCommand();
			create.setClassifierRole( (ClassifierRole)request.getNewObject() );
			create.setParent( navexp ); 
			create.setLocation( loco );
			createCommand = create;
		}else if( newObjectType == Condition.class ) {
			CreateConditionCommand create = new CreateConditionCommand();
			create.setCondition( (Condition)request.getNewObject() );
			create.setParent( navexp ); 
			createCommand = create;
		}
		return createCommand;
	}
	
	protected Command getDeleteDependantCommand(Request request) {
		return null;
	}
	
	protected EditPolicy createChildEditPolicy(EditPart child) {
		if ( child instanceof ConditionEditPart )
			return null;
		if  ( child instanceof ExpEditPart )
			return new ResizableEditPolicy();
		else 
			return new NonResizableEditPolicy();
	}
	
	protected Dimension getMinimumSizeFor(GraphicalEditPart child) {
		return child.getContentPane().getMinimumSize();
	}
	
	protected Command createChangeConstraintCommand(EditPart child, Object constraint) {
		SetConstraintCommand locationCommand = new SetConstraintCommand();
		locationCommand.setObject((EObject)child.getModel());
		locationCommand.setLocation((Rectangle)constraint);
		return locationCommand;
	}
}
