/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.ManhattanConnectionRouter;
import org.eclipse.draw2d.RelativeBendpoint;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import org.eclipse.jface.dialogs.Dialog;

import vocleditor.commands.EditAssociationRoleCommand;
import vocleditor.dialog.AssociationPropertiesDialog;
import vocleditor.edit.VOCLConnectionEndpointEditPolicy;

import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.views.properties.IPropertySource;
import vocleditor.model.EObjectPropertySource;
import BendpointStuff.VOCLBendpointEditPolicy;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.BendpointConnectionRouter;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import VOCLModel.AssociationRole;
import BendpointStuff.VOCLBendpoint;
import vocleditor.VOCLEditorConstants;
import vocleditor.edit.LabelGraphicalEditPart;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.gef.RootEditPart;

/**
 * EditPart for an AssociationRole.
 */
public class VOCLAssociationRoleEditPart extends AbstractConnectionEditPart 
{
	NavExpEditPart layerEditPart;      // the part that has our ConnectionLayer (NavExpPart for now)
	LabelGraphicalEditPart endRoleLabel = null;
	AREPAdapter adapter;

	/**
	 * indicates whether the EditPart is currently selected
	 */
	boolean isSelected;
	
	/**
	 * This is variable is for a pretty much dirty hack.
	 * 
	 * The set<source/target>AnchorAt functions are fired before a ReconnectionCommand is constructed. So we remember the last anchor position here
	 * to get it reconstructed from the Command.  
	 */
	public int lastAnchorAt;
	
	/**
	 * @see VOCLAssociationRoleEditPart#lastAnchorAt
	 * 
	 * this variable is reset by VOCLAssociationRoleEditPart.execute
	 */
	public boolean isMoving = false;
	
	/**
	 * Constructs a new VOCLAssociationRoleEditPart for the given AssociationRole
	 * 
	 * @param associationrole an AssociationRole model element
	 */
	public VOCLAssociationRoleEditPart(AssociationRole associationrole) {
		super();
		setModel(associationrole);
		adapter = new AREPAdapter( this );
		isSelected = false;
		this.setFigure(new PolylineConnection());
	}

	/**
	 * Installs edit policies. {@link VOCLBendpointEditPolicy} is only installed
	 * if the connection is bendpoint-routed.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 * @see VOCLConnectionEndpointEditPolicy
	 * @see VOCLAssociationRoleEditPolicy
	 */
	protected void createEditPolicies() {
		installEditPolicy( EditPolicy.CONNECTION_ENDPOINTS_ROLE, new VOCLConnectionEndpointEditPolicy() );
		installEditPolicy( EditPolicy.CONNECTION_ROLE, new VOCLAssociationRoleEditPolicy());
		if(((AssociationRole)getModel()).getRoutingType() != 1)
			installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE, new VOCLBendpointEditPolicy());
	}

	/**
	 * If the Connection is selected, it will be drawn in @see VOCLEditorConstants#ASSOCIATION_ROLE_SELECTED_COLOR. The label will also
	 * get this color. refreshVisuals also forces the label's figure to be updated via calling @see LabelGraphicalEditPart#getFigure().
	 * Setting the ConnectionRouter according to @see AssociationRole#getRoutingType() is done here, too. 
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		PolylineConnection con = (PolylineConnection)getFigure();
		if(children != null && children.get(0) != null && children.get(0) instanceof LabelGraphicalEditPart) {
			((LabelGraphicalEditPart)(this.children.get(0))).getFigure();
		}
		super.refreshVisuals();

		if(isSelected)
			con.setForegroundColor(VOCLEditorConstants.ASSOCIATION_ROLE_SELECTED_COLOR);
		else
			con.setForegroundColor(VOCLEditorConstants.ASSOCIATION_COLOR);
		switch(((AssociationRole)getModel()).getRoutingType()) {
			case 1:
				con.setConnectionRouter(new ManhattanConnectionRouter());
				break;
			case 0:
			default:
				con.setConnectionRouter(new BendpointConnectionRouter());
				break;
		}

		refreshBendpoints();
		this.refreshSourceAnchor();
		this.refreshTargetAnchor();
	}
		
	/**
	 * starts listening for model changes
	 * @see org.eclipse.gef.EditPart#activate()
	 */
	public void activate() {
		if (isActive())
			return;
		// start listening for changes in the model
		((Notifier)getModel()).eAdapters().add( adapter );
		super.activate();
	}
	
	/**
	 * stops listening for model changes
	 * @see org.eclipse.gef.EditPart#deactivate()
	 */
	public void deactivate() {
		if (!isActive())
			return;
		// stop listening for changes in the model
		((Notifier)getModel()).eAdapters().remove( adapter );

		super.deactivate();		
	}

	/**
	 * Returns the {@link VOCLModel.Label} for this role in the List
	 * 
	 * @return a Vector containing the {@link VOCLModel.Label} for this role
	 */
	protected List getModelChildren() 
	{
 		Vector children = new Vector();
		children.add( ((AssociationRole)getModel()).getLabelDecoration() );
 		return children;
	}
	
	/** 
	 * returns a property source for thr model object
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
        if (IPropertySource.class == key) {
			return new EObjectPropertySource( (EObject) getModel() );
		}
		return null;
	}
	
	/**
	 * Refreshs the bendpoint list.
	 */
	private void refreshBendpoints()
	{
	  	if (getConnectionFigure().getConnectionRouter() instanceof ManhattanConnectionRouter)
			return;
	  	if( (((AssociationRole)getModel()).getBendpointsFirstWidthList().size() !=
	  		 ((AssociationRole)getModel()).getBendpointsSecondWidthList().size()) ||
	  		 	(((AssociationRole)getModel()).getBendpointsFirstHeightList().size() !=
	  		 		((AssociationRole)getModel()).getBendpointsSecondHeightList().size()) ||
	  		 			(((AssociationRole)getModel()).getBendpointsFirstWidthList().size() !=
	  		 				((AssociationRole)getModel()).getBendpointsFirstHeightList().size()))
	  		return;
		List modelConstraint = ((AssociationRole)getModel()).getBendpointsFirstWidthList();
		List figureConstraint = new ArrayList();
		for (int i=0; i<modelConstraint.size(); i++) {
			RelativeBendpoint rbp = new RelativeBendpoint(getConnectionFigure());
			rbp.setRelativeDimensions(
					new Dimension(
							((Integer)((AssociationRole)getModel()).getBendpointsFirstWidthList().get(i)).intValue(),
							((Integer)((AssociationRole)getModel()).getBendpointsFirstHeightList().get(i)).intValue()
					),
					new Dimension(
							((Integer)((AssociationRole)getModel()).getBendpointsSecondWidthList().get(i)).intValue(),
							((Integer)((AssociationRole)getModel()).getBendpointsSecondHeightList().get(i)).intValue()
					)
			);
			rbp.setWeight((i+1) / ((float)modelConstraint.size()+1));
			figureConstraint.add(rbp);
		}
		getConnectionFigure().setRoutingConstraint(figureConstraint);
	}
	
	/**
	  * Inserts a bendpoint
	  * @param index Index of the new bendpoint
	  * @param point the bendpoint to insert
	  */
	public void insertBendpoint(int index, VOCLBendpoint point) {
	 	((AssociationRole)getModel()).getBendpointsFirstWidthList().add(index, 
	  			new Integer(point.getFirstRelativeDimension().width));
	  	((AssociationRole)getModel()).getBendpointsFirstHeightList().add(index, 
	  			new Integer(point.getFirstRelativeDimension().height));
	  	((AssociationRole)getModel()).getBendpointsSecondWidthList().add(index, 
	  			new Integer(point.getSecondRelativeDimension().width));
	  	((AssociationRole)getModel()).getBendpointsSecondHeightList().add(index, 
	  			new Integer(point.getSecondRelativeDimension().height));
	  	refreshBendpoints();
	}

	/**
	  * removes a bendpoint
	  * @param index the index of the bendpoint to remove
	  */
	public void removeBendpointAt(int index) {
	 	((AssociationRole)getModel()).getBendpointsFirstWidthList().remove(index);
	  	((AssociationRole)getModel()).getBendpointsFirstHeightList().remove(index);
	  	((AssociationRole)getModel()).getBendpointsSecondWidthList().remove(index);
	  	((AssociationRole)getModel()).getBendpointsSecondHeightList().remove(index);
	  	refreshBendpoints();
	}

	/**
	  * Moves a bendpoint to a new location
	  * @param index Index of the bendpoint to move
	  * @param point new value for the moved point
	  */
	public void setBendpoint(int index, VOCLBendpoint point) {
	  	((AssociationRole)getModel()).getBendpointsFirstWidthList().set(index, 
	  			new Integer(point.getFirstRelativeDimension().width));
	  	((AssociationRole)getModel()).getBendpointsFirstHeightList().set(index, 
	  			new Integer(point.getFirstRelativeDimension().height));
	  	((AssociationRole)getModel()).getBendpointsSecondWidthList().set(index, 
	  			new Integer(point.getSecondRelativeDimension().width));
	  	((AssociationRole)getModel()).getBendpointsSecondHeightList().set(index, 
	  			new Integer(point.getSecondRelativeDimension().height));
	  	refreshBendpoints();
	}
	  
	/**
	  * Retrieves the bendpoint at index
	  * @param index The index of the bendpoint to retrieve
	  * @return The bendpoint at index
	  */
	public VOCLBendpoint getBendpointAt(int index) {
	 	VOCLBendpoint bendpoint = new VOCLBendpoint();
		bendpoint.setRelativeDimensions(
			new Dimension(
					((Integer)((AssociationRole)getModel()).getBendpointsFirstWidthList().get(index)).intValue(),
					((Integer)((AssociationRole)getModel()).getBendpointsFirstHeightList().get(index)).intValue()
			),
			new Dimension(
					((Integer)((AssociationRole)getModel()).getBendpointsSecondWidthList().get(index)).intValue(),
					((Integer)((AssociationRole)getModel()).getBendpointsSecondHeightList().get(index)).intValue()
			)
		);
	  	return bendpoint;
	}
	  
	/**
	  * Set's selection mode of the Association for highlighting
	  * @param selectmode true == selected, false == not selected
	  */
	public void setSelected(boolean selectmode) {
	 	if(isSelected != selectmode) {
	  		isSelected = selectmode;
	  		refreshVisuals();
	  	}
	}
	  
	/**
	  * Returns the requested layer.
	  * Redefined to return the connection layer of the NavExp(EditPart) this Association is part of.
	  * 
	  * @return the requested layer
	  */
	protected IFigure getLayer(Object layer) 
	{
		if ( layer == LayerConstants.CONNECTION_LAYER )
			return layerEditPart.getConnectionLayer();
		else {
			IFigure sr = super.getLayer(layer);
			return sr;
		}
	}	
	  
	/**
	  * Set the NavExpEditPart the knows the ConnectionLayer for our ConnectionFigure
	  *
	  * @param part a NavExpEditPart that knows the ConnectionLayer for our ConnectionFigure
	  */		
	public void setLayerEditPart( NavExpEditPart part ) 
	{	
		layerEditPart = (NavExpEditPart) part;
	}
 
	/**
	 * Opens the properties dialog and creates and executes an
	 * {@link EditAssociationRoleCommand} if necessary
	 */
	public void performRequest(Request request) {
		if(request.getType() == RequestConstants.REQ_OPEN) {
			RootEditPart root = getRoot(); 
			Command command = null;
			command = new EditAssociationRoleCommand();
			AssociationRole ar = (AssociationRole) getModel();
			((EditAssociationRoleCommand)command).setAssociationRole(ar);			
			AssociationPropertiesDialog d = new AssociationPropertiesDialog(new Shell(), ar );
			d.open();
			if( d.getReturnCode() == Dialog.CANCEL )
				return;
			((EditAssociationRoleCommand)command).setCommandAttributes(d.getSource(),d.getTarget(),d.getName(), ar.getRoutingType());

			root.getViewer().getEditDomain().getCommandStack().execute(command);
		}
	}

	/**
	 * emf.Adapter implementation to refresh AssociationRoleEditParts figure
	 * after EObjects property change. 
	 * The AssociationRoleEditParts cannot be an Adapter itself, because of 
	 * conflicting getTarget() methods.
	 * @author bjoern
	 */
	class AREPAdapter implements Adapter
	{
		VOCLAssociationRoleEditPart assocPart;
		Notifier adapterTarget = null;

		/**
		 * constructs a new AREPAdapter 
		 * @param arep the AssociationRoleEditPart this Adapter id for
		 */
		AREPAdapter( VOCLAssociationRoleEditPart arep )
		{
			this.assocPart = arep;
		}
		
		/**
		 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
		 */
		public void setTarget(Notifier target) 
		{
			adapterTarget = target;
		}
	
	
		/**
		 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
		 */
		public Notifier getTarget() {
			return adapterTarget;
		}
		
		/** 
		 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
		 */
		public boolean isAdapterForType(Object arg0) {
			return false;
		}
	
	
		/**
		 * This method is called on model changes and causes the AREP to refresh visuals
		 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
		 */
		public void notifyChanged(Notification notification )
		{
			assocPart.refreshVisuals();
			if(((AssociationRole)assocPart.getModel()).getRoutingType()==1)
				assocPart.removeEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE);
			else if(assocPart.getEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE)==null)
				assocPart.installEditPolicy(EditPolicy.CONNECTION_BENDPOINTS_ROLE, new VOCLBendpointEditPolicy());
			refreshBendpoints();
		}
	}

}