/*
 * Created on 28.06.2005
 *
 */
package vocleditor.edit;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.ConnectionEndpointLocator;
import org.eclipse.draw2d.geometry.Point;

/**
 * A simple endpoint locator, that takes a connection and an offset and places the figure at the end or beginning of the connection
 * displaced by the offset.
 * 
 * @author jan
 */
public class VOCLConnectionEndpointLocator extends ConnectionEndpointLocator {
	
	/**
	 * The connection to reference
	 */
	private Connection conn;
	
	/**
	 * The offset as a point
	 */
	private Point offset;
	
	/**
	 * Indicating whether we take the start or the endpoint of the connection
	 */
	private boolean isEnd;
	
	private boolean isCustom;
	
	/**
	 * Constructor
	 * @param c The connection to which the figure belongs
	 * @param isEnd Indicates whether to place the figure at the beginning or the end of the connection
	 * @param offset The offset as point
	 */
	public VOCLConnectionEndpointLocator(Connection c, boolean isEnd, Point offset, boolean isCustom) {
		super(c, isEnd);
		this.conn = c;
		this.offset = offset;
		this.isCustom = isCustom;
	}
	
	/**
	 * @see org.eclipse.draw2d.Locator#relocate(org.eclipse.draw2d.IFigure)
	 */
	public void relocate(IFigure target) {
		if(isCustom) {
			setUDistance(0);
			setVDistance(0);
		}
		super.relocate(target);
		if(isCustom) {
			Point p = Point.SINGLETON;
			conn.getPoints().getPoint(p, isEnd?(conn.getPoints().size()-1):0 );
			p.translate(offset);
			target.setLocation(p);
		}
	}

}
