/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.edit;

import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editpolicies.RootComponentEditPolicy;

import VOCLModel.VOCLDiagram;

/**
 * EditPart for the main component "VOCLDiagram".
 */
public class VOCLEditPart extends AbstractVOCLEditPart {	
	/**
	 * @param diagram The model this EditPart is created for
	 */
	public VOCLEditPart(VOCLDiagram diagram) {
		setModel(diagram);
	}
    
	public VOCLDiagram getVOCLModel() {
		return (VOCLDiagram) getModel();
	}

	protected IFigure createFigure(){
		FreeformLayer layer = new FreeformLayer();
		layer.setLayoutManager(new FreeformLayout());
		layer.setBorder(new LineBorder(1));
		return layer;
	}

	protected void createEditPolicies()	{
		installEditPolicy(EditPolicy.CONTAINER_ROLE, new VOCLEditPolicy());
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new VOCLXYLayoutEditPolicy());
		installEditPolicy(EditPolicy.NODE_ROLE, null);
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, null);
		installEditPolicy(EditPolicy.SELECTION_FEEDBACK_ROLE, null);
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new RootComponentEditPolicy());    	
	}
   
	protected List getModelChildren(){
		List children = new Vector();
		children.addAll( ((VOCLDiagram)getModel()).getConstraints() );
		return children;
	}


	/**
	 * Notify changes from the model to the view.
	 */
	public void notifyChanged(Notification notification) {
		int type = notification.getEventType();
		switch( type ) {
			case Notification.ADD:
			case Notification.ADD_MANY:
			case Notification.REMOVE:
			case Notification.REMOVE_MANY:
				refreshChildren();
				break;
			case Notification.SET:
				refreshVisuals();
				break;
		}
	}
}