
package vocleditor.editor;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.tools.CreationTool;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.Shell;

import vocleditor.VOCLEditorPlugin;
import vocleditor.commands.CreateBoolExpCommand;
import vocleditor.commands.CreateClassifierRoleCommand;
import vocleditor.commands.CreateIfExpCommand;
import vocleditor.commands.CreateLetExpCommand;
import vocleditor.commands.CreateSetClassifierRoleCommand;
import vocleditor.dialog.BoolExpPropertiesDialog;
import vocleditor.dialog.ClassifierPropertiesDialog;
import vocleditor.dialog.IfExpPropertiesDialog;
import vocleditor.dialog.LetExpPropertiesDialog;
import vocleditor.dialog.SetClassifierRolePropertiesDialog;
import VOCLModel.ClassifierRole;
import VOCLModel.SetClassifierRole;

/**
 * @author Stefan
 *
 */
public class VOCLCreationTool extends CreationTool {
	/**
	 * create VOCL CreationTool
	 *
	 */
	public VOCLCreationTool() {
		super();
	}
	/**
	 * create VOCL CreationTool
	 * @param aFactory instance of ModelElement
	 */
	public VOCLCreationTool(CreationFactory aFactory) {
		super(aFactory);
	}
	
	protected void executeCurrentCommand() {
		Command command = getCurrentCommand();
		Shell shell = this.getCurrentViewer().getControl().getShell();
		if((command instanceof CreateClassifierRoleCommand) && command.canExecute()){
			CreateClassifierRoleCommand create=(CreateClassifierRoleCommand)command;
			ClassifierRole classifierrole = create.getClassifierRole();
			ClassifierPropertiesDialog cpd = new ClassifierPropertiesDialog( shell, VOCLEditorPlugin.getConstraint(create.getParent()), classifierrole);
			cpd.open();
			if( cpd.getReturnCode() == Dialog.CANCEL ) 
				return;
			((CreateClassifierRoleCommand)command).setCommandAttributes(cpd.getClassifierName(),cpd.getClassifierType(),cpd.getAttributes());
		} else if( (command instanceof CreateBoolExpCommand) && command.canExecute()){
			BoolExpPropertiesDialog bpd = new BoolExpPropertiesDialog( shell, ((CreateBoolExpCommand)command).getBoolExp() );
			bpd.open();
			if( bpd.getReturnCode() == Dialog.CANCEL ) 
				return;
			((CreateBoolExpCommand)command).setCommandAttributes( bpd.getOperation(), bpd.getIsHorizontal(), bpd.getSplitPercentage(), bpd.isNegated() );
		} else if( (command instanceof CreateIfExpCommand) && command.canExecute()){
			IfExpPropertiesDialog ipd = new IfExpPropertiesDialog( shell, ((CreateIfExpCommand)command).getIfExp() );
			ipd.open();
			if( ipd.getReturnCode() == Dialog.CANCEL ) 
				return;
			((CreateIfExpCommand)command).setCommandAttributes( ipd.getIsHorizontal(), ipd.getSplitPercentage1(), ipd.getSplitPercentage2(), ipd.isNegated() );
		} else if( (command instanceof CreateLetExpCommand ) && command.canExecute()){
			LetExpPropertiesDialog lpd = new LetExpPropertiesDialog( shell, ((CreateLetExpCommand)command).getLetExp() );
			lpd.open();
			if( lpd.getReturnCode() == Dialog.CANCEL ) 
				return;
			((CreateLetExpCommand)command).setCommandAttributes( lpd.getIsHorizontal(), lpd.getSplitPercentage(), lpd.isNegated() );
		} else if((command instanceof CreateSetClassifierRoleCommand) && command.canExecute()){
			CreateSetClassifierRoleCommand create = (CreateSetClassifierRoleCommand)command;
			SetClassifierRole classifierrole = create.getSetClassifierRole();
			SetClassifierRolePropertiesDialog cpd = new SetClassifierRolePropertiesDialog( shell, VOCLEditorPlugin.getConstraint(create.getParent()), classifierrole);
			cpd.open();
			if( cpd.getReturnCode() == Dialog.CANCEL ) 
				return;
			((CreateSetClassifierRoleCommand)command).setCommandAttributes(cpd.getClassifierName(),cpd.getClassifierType(), cpd.getOpKind(), cpd.getSizeVariable(), cpd.getIteratorVariable(), cpd.getSumVariable(), cpd.getAttributes());
		}
		super.executeCurrentCommand();
	}

}
