/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.editor;

import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.ui.parts.GraphicalViewerKeyHandler;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import vocleditor.actions.VOCLContextMenuProvider;
import vocleditor.edit.GraphicalEditPartsFactory;

/**
 * VOCL editor page.
 */
public class VOCLPage extends AbstractEditorPage
{
	/**
	 * Creates a new VOCL editor page instance.
	 * 
	 * <p>By design this page uses its own <code>EditDomain</code>.
	 * The main goal of this approach is that this page has its own
	 * undo/redo command stack.
	 * 
	 * @param parent the parent multi page editor 
	 */
	public VOCLPage(VOCLEditor parent)
	{
		super(parent, new EditDomain());
	}

	/* (non-Javadoc)
	 * @see com.ibm.itso.sal330r.gefdemo.editor.AbstractEditorPage#getPageName()
	 */
	protected String getPageName()
	{
		return "VOCLModel";
	}

	/* (non-Javadoc)
	 * @see com.ibm.itso.sal330r.gefdemo.editor.AbstractEditorPage#createPageControl(org.eclipse.swt.widgets.Composite)
	 */
	protected void createPageControl(Composite parent)
	{
		Composite composite = new Composite(parent, SWT.NONE);
		composite.setBackground(parent.getBackground());
		composite.setLayout(new GridLayout(2, false));
        
		createPaletteViewer(composite);
		GridData gd = new GridData(GridData.FILL_VERTICAL);
		gd.widthHint = 125;
		getPaletteViewer().getControl().setLayoutData(gd);
        
		createGraphicalViewer(composite);
		gd = new GridData(GridData.FILL_BOTH);
		gd.widthHint = 275;
		getGraphicalViewer().getControl().setLayoutData(gd);
		
		GraphicalViewer viewer = this.getGraphicalViewer();
		RootEditPart root = viewer.getRootEditPart();
	}

	/** the graphical viewer */
	private GraphicalViewer viewer;

	/**
	 * Creates the GraphicalViewer on the specified <code>Composite</code>.
	 * @param parent the parent composite
	 */
	private void createGraphicalViewer(Composite parent)
	{
		viewer = new ScrollingGraphicalViewer();
		viewer.createControl(parent);

		// configure the viewer
		viewer.getControl().setBackground(parent.getBackground());
		viewer.setRootEditPart(new ScalableFreeformRootEditPart());
		viewer.setKeyHandler(new GraphicalViewerKeyHandler(viewer));

		// hook the viewer into the editor
		registerEditPartViewer(viewer);

		// configure the viewer with drag and drop
		configureEditPartViewer(viewer);

		// initialize the viewer with input
		viewer.setEditPartFactory(new GraphicalEditPartsFactory(getSite().getShell()));
		viewer.setContents(getVOCLModelEditor().getVOCLModel());
		
		ContextMenuProvider provider =
			new VOCLContextMenuProvider(
				getGraphicalViewer(),
				getVOCLModelEditor().getActionRegistry());
		getGraphicalViewer().setContextMenu(provider);
		getSite().registerContextMenu(provider, getGraphicalViewer());
	}

	/* (non-Javadoc)
	 * @see com.ibm.itso.sal330r.gefdemo.editor.AbstractEditorPage#getGraphicalViewerForZoomSupport()
	 */
	protected GraphicalViewer getGraphicalViewer()
	{
		return viewer;
	}
}
