/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.figures;

import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayeredPane;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.FreeformViewport;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.ScrollPane;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.gef.LayerConstants;


/**
 * A Figure representing the VOCL NavExp element.
 * 
 * This figure provides layers for Nodes (ClassifierRole)
 * and Connections (AssociationRole) in a scrollabe fashion.
 */
public class NavExpFigure extends Figure {
	protected IFigure conditionFigure;
	protected ScrollPane scrollPane;
	protected IFigure contentPane;
	protected IFigure connectionLayer;

	/**
	 * Creates a new NavExpFigure
	 */
	public NavExpFigure() 
	{
		//setBorder( new LineBorder( ColorConstants.black, 1 ) );
		//setBackgroundColor( VOCLEditorConstants.EXPRESSION_COLOR );
		//setLayoutManager( new StackLayout() );
		setLayoutManager(new ToolbarLayout());
	    setOpaque(false);
	    
	    contentPane = new FreeformLayeredPane(){
	    	public void add(IFigure figure, Object constraint, int index){
	    		if(figure instanceof ConditionFigure){
	    			NavExpFigure.this.add(figure, constraint, index);
	    		}
	    		else
	    			super.add( figure, constraint, index );
	    	}
	    	public void remove(IFigure figure) {
	    		if (figure instanceof ConditionFigure) {
	    			NavExpFigure.this.remove(figure);
	    		} else
	    			super.remove(figure);
	    	}
	    	public void setConstraint(IFigure child, Object constraint) 
	    	{
	    		if ( child instanceof ScrollPane || child instanceof ConditionFigure)
	    			NavExpFigure.this.setConstraint( child, constraint );
	    		else
	    			super.setConstraint(child, constraint);
	    	}
	    };
		contentPane.add( new FreeformLayer(), LayerConstants.PRIMARY_LAYER);
		contentPane.add( getConnectionLayer(), LayerConstants.CONNECTION_LAYER);
		contentPane.setLayoutManager( new FreeformLayout() );
		
		scrollPane = new ScrollPane();
	    scrollPane.setViewport( new FreeformViewport() );
	    scrollPane.setContents( contentPane );    
		
	    add(scrollPane);
	}	
 
	/**
	  * Returns the connection layer of this NavExp figure.
	  * @return the connection layer of this NavExp figure
	  */
	public IFigure getConnectionLayer() {
		if ( connectionLayer == null )
			connectionLayer = new ConnectionLayer();
		return connectionLayer;
	}

	/**
	 * redefined to pass children (exept my scrollbar) to the content pane
	 * @see org.eclipse.draw2d.IFigure#add(org.eclipse.draw2d.IFigure, java.lang.Object, int)
	 */
	public void add(IFigure figure, Object constraint, int index) 
	{	
		if ( figure instanceof ScrollPane )
			super.add( figure, constraint, 0);
		else if(figure instanceof ConditionFigure){
			conditionFigure = figure;
			super.add(figure, constraint, 1);	
			validate();
		}
		else
			contentPane.add( figure, constraint, index );
	}
		
	/**
	 * redefined to pass children to be deleted (exept my scrollbar) to the content pane
	 * @see org.eclipse.draw2d.IFigure#remove(org.eclipse.draw2d.IFigure)
	 */
	public void remove(IFigure figure) {
		if ( figure instanceof ScrollPane || figure instanceof ConditionFigure) {
			if(figure instanceof ConditionFigure)
				conditionFigure = null;
			super.remove(figure);			
		} else
			contentPane.remove(figure);
	}

	/**
	 * redefined to pass children whose constraint should be set (exept my scrollbar) to the content pane
	 * @see org.eclipse.draw2d.IFigure#setConstraint(org.eclipse.draw2d.IFigure, java.lang.Object)
	 */
	public void setConstraint(IFigure child, Object constraint) 
	{
		if ( child instanceof ScrollPane || child instanceof ConditionFigure)
			super.setConstraint( child, constraint );
		else
			contentPane.setConstraint(child, constraint);
	}	
	
	/**
	 * @see org.eclipse.draw2d.IFigure#validate()
	 */
	public void validate(){
		//if ( isValid() ) return;
		int efw = getBounds().width;
		int efh = getBounds().height;
		if(conditionFigure != null)
			efh = efh - conditionFigure.getBounds().height;
		if(scrollPane != null)
			scrollPane.setPreferredSize(new Dimension( efw, efh ));
		super.validate();
	}
	
	/**
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}
	
	/**
	 * returns the ContentPane (a FreeformFigure) of this NavExpFigure,
	 * which is not directly a child, but a child of the scrollpanes viewport
	 * @return the ContentPane (a FreeformFigure) of this NavExpFigure
	 */
	public IFigure getContentPane() { 
		return contentPane;
	}

	
	/**
	 * returns the minimum size of this NavExpFigure
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) 
	{		
		return new Dimension( 50, 50 );
	}
}
		
