package vocleditor.figures;

import org.eclipse.draw2d.AbstractBorder;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.swt.graphics.Color;

/**
 * @author bjoern
 *
 * Represents the negatable border.
 */
public class NegatableBorder extends AbstractBorder
{
	private static final int BORDER_EXTEND = 4;
	private boolean negated = false;
	private Insets insets = new Insets( 0, 0, 0, 0 );
	private Color background;
	
	/**
	 * Sets true if the border negated.
	 * @param negated the negation flag
	 */
	public void setNegated( boolean negated ) {
		this.negated = negated;
		insets.bottom = insets.top = insets.left = insets.right = negated ? BORDER_EXTEND : 0;
	}

	/**
	 * Sets the background color.
	 * @param color the background color to set
	 */
	public void setBackground( Color color ) {
		this.background = color;
	}
	
	/**
	 * Returns the insets of the figute.
	 * @parem the figute 
	 * @see org.eclipse.draw2d.Border#getInsets(org.eclipse.draw2d.IFigure)
	 */
	public Insets getInsets(IFigure figure) {
		return insets;
	}
	
	/**
	 * Paints the border.
	 * @see org.eclipse.draw2d.Border#paint(org.eclipse.draw2d.IFigure, org.eclipse.draw2d.Graphics, org.eclipse.draw2d.geometry.Insets)
	 */
	public void paint(IFigure figure, Graphics graphics, Insets insets) {
		if ( negated ) {
			tempRect.setBounds( getPaintRectangle(figure, insets) );
			graphics.setForegroundColor( ColorConstants.red );
			graphics.setBackgroundColor( background );
			graphics.fillRectangle( tempRect.x, tempRect.y, tempRect.width, BORDER_EXTEND );
			graphics.fillRectangle( tempRect.x, tempRect.y + tempRect.height - BORDER_EXTEND, tempRect.width, BORDER_EXTEND );
			graphics.fillRectangle( tempRect.x + tempRect.width - BORDER_EXTEND, tempRect.y, BORDER_EXTEND, tempRect.height );
			graphics.fillRectangle( tempRect.x, tempRect.y, BORDER_EXTEND, tempRect.height );
			graphics.drawLine( tempRect.x, tempRect.y, tempRect.x + tempRect.width, tempRect.y + tempRect.height );
			graphics.drawLine( tempRect.x + tempRect.width, tempRect.y, tempRect.x, tempRect.y + tempRect.height );
		}
	}

}
