package vocleditor.model;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import VOCLModel.Attribute;
import VOCLModel.BoolExp;
import VOCLModel.ClassifierRole;
import VOCLModel.Constraint;
import VOCLModel.IfExp;
import VOCLModel.LetExp;
import VOCLModel.Method;
import VOCLModel.NavExp;
import VOCLModel.SetClassifierRole;
import VOCLModel.VOCLExpression;
import VOCLModel.VariableDeclaration;


/**
 * Containts static functions to help getting modelelements ... blah
 *
 * @author bjoern
 * TODO
 */
public class VOCLTools
{

	/**
	 * lists all ClassifierRoles within a NavExp and sub expressions
	 * @param navexp the navexp to start collecting ClassifierRoles
	 * @param list the list to add the ClassifierRoles to
	 * @return the given list
	 */
	public static List getAllClassifiers( NavExp navexp, List list )
	{
		for ( Iterator i = navexp.getClassifierRoles().iterator(); i.hasNext(); )
			list.add( i.next() );

		// recurseto sub expressions
		for ( Iterator i = navexp.getExpressions().iterator(); i.hasNext(); ) {
			VOCLExpression expr = (VOCLExpression) i.next();
			if ( expr instanceof BoolExp ) {
				BoolExp be = (BoolExp) expr;
				getAllClassifiers( (NavExp) be.getOp1(), list );
				getAllClassifiers( (NavExp) be.getOp2(), list );
			}
			else if ( expr instanceof IfExp ) {
				IfExp ie = (IfExp) expr;
				getAllClassifiers( (NavExp) ie.getIfExp(), list );
				getAllClassifiers( (NavExp) ie.getElseExp(), list );
				getAllClassifiers( (NavExp) ie.getThenExp(), list );
			}
			else if ( expr instanceof LetExp ) {
				LetExp le = (LetExp) expr;
				getAllClassifiers( (NavExp) le.getLet(), list );
				getAllClassifiers( (NavExp) le.getIn(), list );
			} else
				System.out.println( "Thought this never happens");
		}
		return list;
	}

	/**
	 * lists all Methods from a list of ClassifierRoles
	 * @param classifiers list of classifiers to search for methods
	 * @param list the list to add the Methods to
	 * @return the given list
	 */
	public static List getAllMethods( List classifiers, List list )
	{
		for ( Iterator i = classifiers.iterator(); i.hasNext(); ) {
			Method m = ((ClassifierRole) i.next()).getMethod();
			if (  m != null )
				list.add( m );
		}
		return list;
	}

	/**
	 * Collects all variables used by ClassifierRoles, Attributes, Methods of a navexp andits sub expressions
	 * @param l List to add Variables to
	 * @param navexp navexp to start looking from
	 */
	public static void collectVariables( NavExp navexp, List l )
	{
		// add all variables from all classifiers
		for ( Iterator i = getAllClassifiers( navexp, new Vector() ).iterator(); i.hasNext(); ) {
			ClassifierRole c = (ClassifierRole) i.next();
			// classifier variables
			if ( c.getVariable() != null )
				l.add( c.getVariable() );
			// additional setClassifier variables
			if (c instanceof SetClassifierRole) {
				SetClassifierRole scr = (SetClassifierRole) c;
				if ( scr.getIterator() != null )
					l.add( scr.getIterator() );
				else if ( scr.getSize() != null )
					l.add(  scr.getSize() );
				else if ( scr.getSum() != null )
					l.add(  scr.getSum() );
			}
			// method variables
			if ( c.getMethod() != null && c.getMethod().getReturnVariable() != null ) {
				l.add( c.getMethod().getReturnVariable() );
				List parameters = c.getMethod().getArguments();
				for ( int j = 0; j < parameters.size(); j++ )
					l.add( (VariableDeclaration) parameters.get(j) );
			}
			// attribute variables
			List attrs = c.getAttributes();
			for ( int j = 0; j < attrs.size(); j++ ) {
				Attribute attr = (Attribute) attrs.get(j);
				if ( attr.getVariable() != null )
					l.add( attr.getVariable() );
			}
		}
	}

	/**
	 * removes all unused variable declarations from constraint.allVariables and returns a list of removed variables
	 * @param constraint the Constraint to remove unused variables from
	 * @return a List of variables removed from the constraint
	 */
	public static List deleteUnusedVariables( Constraint constraint )
	{
		Vector reallyAllVariables = new Vector();
		Vector deletedVars  = new Vector();

		collectVariables( (NavExp) constraint.getExpression(), reallyAllVariables );

		for ( int i = 0; i < constraint.getAllVariables().size(); i++ ) {
			VariableDeclaration var = (VariableDeclaration) constraint.getAllVariables().get(i);
			if ( ! reallyAllVariables.contains( var ) )
				deletedVars.add( var );
		}

		for ( int i = 0; i < deletedVars.size(); i++ )
			constraint.getAllVariables().remove( deletedVars.get(i) );

		return deletedVars;
	}

	/**
	 * Finds or creates a VariableDeclaration inside of a constraint
	 * @param constraint the constraint to hold the VariableDeclaration
	 * @param name the name of the VariableDeclaration
	 * @param type the type of the VariableDeclaration
	 * @return an already existing or newly created VariableDeclaration with the given name and type
	 */
	public static VariableDeclaration getVariable( Constraint constraint, String name, String type )
	{
		VariableDeclaration var;
		List allVars = constraint.getAllVariables();
		for  ( int i = 0; i < allVars.size(); i++ ) {
			var = (VariableDeclaration) allVars.get(i);
			if ( name.equals( var.getName() ) && type.equals( var.getType() ) )
				return var;
		}
		var = (VariableDeclaration) (new ModelCreationFactory(VariableDeclaration.class)).getNewObject();
		var.setName( name );
		var.setType( type );
		return var;
	}
	/**
	 * helper method to find the depth of a Expression
	 * @param exp VOCLExpression
	 * @return the depth
	 */
	public static int getExpressionDepth( VOCLExpression exp )
	{
		int depth = 0;
		NavExp navexp = exp.getParentExp();
		while ( navexp.getConstraint() == null ) {
			navexp = ((VOCLExpression) navexp.eContainer()).getParentExp();
			depth++;
		}
		return depth;
	}
}
